/** \file
\brief Obsuga plikw konfiguracyjnych
\author Adam Sawicki - sawickiap@poczta.onet.pl - http://regedit.gamedev.pl/ \n

Cz biblioteki CommonLib \n
Kodowanie Windows-1250, koniec wiersza CR+LF, test: Za gl ja \n
Licencja: GNU GPL. \n
Dokumentacja: \ref Module_Config \n
Elementy moduu: \ref code_config
*/
#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif
#ifndef COMMON_CONFIG_H_
#define COMMON_CONFIG_H_

#include <typeinfo>

namespace common
{

class Stream;

/** \addtogroup code_config Modu Config
Dokumentacja: \ref Module_Config \n
Nagwek: Config.hpp */
//@{

/// \internal
class Config_pimpl;

/// Abstrakcyjna klasa bazowa dla elementw konfiguracji
class Item
{
public:
	virtual ~Item() { }
};

/// Warto tekstowa - przechowuje acuch
class Value : public Item
{
public:
	tstring Data;

	Value() { }
	Value(const tstring &s) : Data(s) { }
};

/// Lista, czyli tak naprawd wektor stringw
class List : public Item
{
public:
	STRING_VECTOR Data;
};

/// Konfiguracja
/** Tej klasy naley uywa zarwno jako gwn konfiguracj jak i
uywana jest do przechowywania podkonfiguracji.
Zawiera map: Klucz [tstring] => Element [Item]
*/
class Config : public Item
{
	DECLARE_NO_COPY_CLASS(Config)
	friend class Config_pimpl;

private:
	scoped_ptr<Config_pimpl> pimpl;

public:
	/** \name Flagi bitowe */
	//@{
	/// Zapisywane dane czy element musi wczeniej istnie. Zostaje zastpiony
	static const uint4 CM_MUST_EXIST     = 0x02;
	/// Zapisywane dane czy element nie moe wczeniej istnie. Zostaje utworzony
	static const uint4 CM_MUST_NOT_EXIST = 0x04;
	/// W razie nieistnienia, tworzona jest caa hierarchia konfiguracji a do zapisywanego elementu
	static const uint4 CM_CREATE_PATH    = 0x08;
	/// Zapisywany element, jeli ju istnieje, musi by tego samego typu co nowy
	static const uint4 CM_SAME_TYPE      = 0x10;
	//@}

	Config();
	~Config();

	/// Wczytuje konfiguracj, w razie bdu rzuca wyjtek.
	void LoadFromString(const tstring &S);
	/// Wczytuje konfiguracj, w razie bdu rzuca wyjtek.
	void LoadFromStream(Stream *S);
	/// Wczytuje konfiguracj, w razie bdu rzuca wyjtek.
	void LoadFromFile(const tstring &FileName);
	/// Zapisuje konfiguracj, w razie bdu rzuca wyjtek.
	void SaveToString(tstring *S) const;
	/// Zapisuje konfiguracj, w razie bdu rzuca wyjtek.
	void SaveToStream(Stream *S) const;
	/// Zapisuje konfiguracj, w razie bdu rzuca wyjtek.
	void SaveToFile(const tstring &FileName) const;

	/// Zwraca element o podanej ciece lub NULL, jeli nie znaleziono.
	Item * GetItem(const tstring &Path);
	/// Zwraca element o podanej ciece. Jeli nie znaleziono, rzuca wyjtek.
	Item * MustGetItem(const tstring &Path);
	/// Zwraca dane elementu o podanej ciece.
	/** Jesli nie znaleziono lub to nie jest element z danymi, zwraca false, a Data jest niezdefiniowane. */
	bool GetData(const tstring &Path, tstring *Data);
	/// Zwraca dane elementu o podanej ciece.
	/** Jesli nie znaleziono lub to nie jest element z danymi, rzuca wyjtek. */
	void MustGetData(const tstring &Path, tstring *Data);

	/// Zwraca true, jesli istnieje element o podanej ciece - niezalenie jakiego jest typu.
	bool ItemExists(const tstring &Path);
	/// Jeli element o podanej ciece nie istnieje, rzuca wyjtek.
	void ItemMustExists(const tstring &Path);

	/// Ustawia element do podanej ciezki. Jeli si nie uda, zwraca false.
	bool SetItem(const tstring &Path, Item *a_Item, uint4 CreateMode);
	/// Ustawia element do podanej ciezki. Jeli si nie uda, rzuca wyjtek.
	void MustSetItem(const tstring &Path, Item *a_Item, uint4 CreateMode);
	/// Ustawia dane elementu tekstowego o podanej ciece. Jeli sie nie uda, zwraca false.
	bool SetData(const tstring &Path, const tstring &Data, uint4 CreateMode);
	/// Ustawia dane elementu tekstowego o podanej ciece. Jeli si nie uda, rzuca wyjtek.
	void MustSetData(const tstring &Path, const tstring &Data, uint4 CreateMode);

	/// Usuwa element o podanej ciece.
	bool DeleteItem(const tstring &Path);
	void MustDeleteItem(const tstring &Path);

	/// Zwraca list nazw elementw
	void GetElementNames(STRING_VECTOR *OutNames);

	/// Zwraca element podanego typu spod podanej ciezki.
	/** Jako typ podawa pochodne Item np. DataItem czy Config.
	Jeli nie znaleziono lub element jest innego typu, zwraca NULL. */
	template <typename T> T * GetItemEx(const tstring &Path);
	/// Zwraca element podanego typu spod podanej ciezki.
	/** Jeli nie znaleziono lub element jest innego typu, rzuca wyjtek. */
	template <typename T> T * MustGetItemEx(const tstring &Path);
	/// Zwraca dane spod podanej cieki skonwertowane do podanego typu.
	/** Jeli nie znaleziono lub bd konwersji, zwraca false. */
	template <typename T> bool GetDataEx(const tstring &Path, T *Data);
	/// Zwraca dane spod podanej cieki skonwertowane do podanego typu.
	/** Jeli nie znaleziono lub bd konwersji, rzuca wyjtek. */
	template <typename T> void MustGetDataEx(const tstring &Path, T *Data);
	/// Jeli element o podanej ciece nie istnieje lub nie jest takiego typu jak podany, zwraca false.
	template <typename T> bool ItemExistsEx(const tstring &Path);
	/// Jeli element o podanej ciece nie istnieje lub nie jest takiego typu jak podany, rzuca wyjtek.
	template <typename T> void ItemMustExistsEx(const tstring &Path);
	/// Ustawia dane elementu tekstowego o podanej ciece na acuch skonwertowany z podanej wartoci danego typu.
	/** Jeli sie nie uda, zwraca false. */
	template <typename T> bool SetDataEx(const tstring &Path, const T &Data, uint4 CreateMode);
	/// Ustawia dane elementu tekstowego o podanej ciece na acuch skonwertowany z podanej wartoci danego typu.
	/** Jeli sie nie uda, rzuca wyjtek. */
	template <typename T> void MustSetDataEx(const tstring &Path, const T &Data, uint4 CreateMode);
};

template <typename T>
inline T * Config::GetItemEx(const tstring &Path)
{
	Item *i = GetItem(Path);
	if (i == 0)
		return 0;
	if (dynamic_cast<T*>(i) == 0)
		return 0;
	return i;
}

template <typename T>
inline T * Config::MustGetItemEx(const tstring &Path)
{
	Item *i = MustGetItem(Path);
	T *R;
	if ((R = dynamic_cast<T*>(i)) == 0)
		throw Error(_T("Cannot read configuration element from path \"") + Path + _T("\". Element type other than expected."));
	return R;
}

template <typename T>
inline bool Config::GetDataEx(const tstring &Path, T *Data)
{
	tstring s;
	if (!GetData(Path, &s))
		return false;
	if (!StrToSth<T>(Data, s))
		return false;
	return true;
}

template <typename T>
inline void Config::MustGetDataEx(const tstring &Path, T *Data)
{
	tstring s;
	MustGetData(Path, &s);
	if (!StrToSth<T>(Data, s))
		throw Error(_T("Cannot read configuration data from path: \"") + Path + _T("\". Conversion error."));
}

template <typename T>
inline bool Config::ItemExistsEx(const tstring &Path)
{
	Item *i = GetItem(Path);
	if (i == 0)
		return false;
	if (dynamic_cast<T*>(i) == 0)
		return false;
	return true;
}

template <typename T>
inline void Config::ItemMustExistsEx(const tstring &Path)
{
	Item *i = GetItem(Path);
	if (i == 0)
		throw Error(_T("Configuration element with path \"") + Path + _T("\" does not exist."));
	if (dynamic_cast<T*>(i) == 0)
		throw Error(_T("Configuration element with path \"") + Path + _T("\" has invalid type."));
}

template <typename T>
inline bool Config::SetDataEx(const tstring &Path, const T &Data, uint4 CreateMode)
{
	tstring s;
	SthToStr<T>(&s, Data);
	return SetData(Path, s, CreateMode);
}

template <typename T>
inline void Config::MustSetDataEx(const tstring &Path, const T &Data, uint4 CreateMode)
{
	tstring s;
	SthToStr<T>(&s, Data);
	MustSetData(Path, s, CreateMode);
}

/// Konfiguracja gwna
extern scoped_ptr<Config> g_Config;

//@}
// code config

} // namespace common

#endif
