/** \file
\brief Otoczka na pojedyncz warto dowolnego typu
\author Adam Sawicki - sawickiap@poczta.onet.pl - http://regedit.gamedev.pl/ \n

Cz biblioteki CommonLib \n
Kodowanie Windows-1250, koniec wiersza CR+LF, test: Za gl ja \n
Licencja: GNU GPL. \n
Dokumentacja: \ref Module_Dator \n
Elementy moduu: \ref code_dator
*/
#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif
#ifndef COMMON_DATOR_H_
#define COMMON_DATOR_H_

#include <map> // :(

namespace common
{

/** \addtogroup code_dator Modu Dator
Dokumentacja: \ref Module_Dator \n
Nagwek: Dator.hpp */
//@{

/// Klasa bazowa datora
class GenericDator
{
public:
	virtual ~GenericDator() { }
	virtual bool SetValue(const tstring &Value) { return false; }
	virtual bool GetValue(tstring *Value) { return false; }
};

/// Oglny dator danego typu
template <typename T>
class Dator : public GenericDator
{
private:
	T *m_Ptr;

public:
	Dator() : m_Ptr(0) { }
	Dator(T *Ptr) : m_Ptr(Ptr) { }

	T * GetPtr() { return m_Ptr; }
	void SetPtr(T *Ptr) { m_Ptr = Ptr; }

	virtual bool SetValue(const tstring &Value);
	virtual bool GetValue(tstring *Value);
};

template <typename T>
inline bool Dator<T>::SetValue(const tstring &Value)
{
	T Tmp;
	if (m_Ptr == 0) return false;
	if (!StrToSth_obj<T>::IsSupported()) return false;
	if (StrToSth<T>(&Tmp, Value))
	{
		*m_Ptr = Tmp;
		return true;
	}
	else return false;
}

template <typename T>
inline bool Dator<T>::GetValue(tstring *Value)
{
	if (m_Ptr == 0) return false;
	if (!SthToStr_obj<T>::IsSupported()) return false;
	SthToStr(Value, *m_Ptr);
	return true;
}


/// Czciowa specjalizacja datora dla wartoci staej
template <typename T>
class Dator<const T> : public GenericDator
{
private:
	const T *m_Ptr;

public:
	Dator() : m_Ptr(0) { }
	Dator(const T *Ptr) : m_Ptr(Ptr) { }

	const T * GetPtr() { return m_Ptr; }
	void SetPtr(const T *Ptr) { m_Ptr = Ptr; }

	virtual bool SetValue(const tstring &Value) { return false; }
	virtual bool GetValue(tstring *Value);
};

template <typename T>
inline bool Dator<const T>::GetValue(tstring *Value)
{
	if (m_Ptr == 0) return false;
	SthToStr(Value, *m_Ptr);
	return true;
}


/// Grupa datorw
class DatorGroup
{
private:
	typedef std::map< tstring, shared_ptr<GenericDator> > DATOR_MAP;
	DATOR_MAP m_Dators;

public:
	/// Dodaje dator do podanej wartoci pod podan nazw
	/** Jeli si nie uda (ju taki jest), zwraca false. */
	template <typename U>
	bool Add(const tstring &Name, U *Value)
	{
		return m_Dators.insert(DATOR_MAP::value_type(Name, shared_ptr<GenericDator>(new Dator<U>(Value)))).second;
	}
	/// Usuwa dator o podanej nazwie
	/** Jeli si nie uda (nie ma takiego), zwraca false. */
	bool Delete(const tstring &Name);
	/// Zwraca true, jeli dator o podanej nazwie istnieje
	bool Exists(const tstring &Name);

	/// Ustawia warto datora o podanej nazwie
	/** \return Zwracana warto:
	- 0 = OK
	- 1 = nie ma datora o tej nazwie
	- 2 = dator jest, ale nie udao si ustawi */
	int SetValue(const tstring &Name, const tstring &Value);
	/// Zwraca warto datora o podanej nazwie
	/** \return Zwracana warto:
	- 0 = OK
	- 1 = nie ma datora o tej nazwie
	- 2 = dator jest, ale nie udao si pobra */
	int GetValue(const tstring &Name, tstring *Value);
};

//@}
// code_dator

} // namespace common

#endif
