/** \file
\brief Klasy wyjtkw do obsugi bdw
\author Adam Sawicki - sawickiap@poczta.onet.pl - http://regedit.gamedev.pl/ \n

Cz biblioteki CommonLib \n
Kodowanie Windows-1250, koniec wiersza CR+LF, test: Za gl ja \n
Licencja: GNU GPL. \n
Dokumentacja: \ref Module_Error \n
Elementy moduu: \ref code_error
*/
#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif
#ifndef COMMON_ERROR_H_
#define COMMON_ERROR_H_

typedef long HRESULT;
#ifdef WIN32
	#include <windows.h>
#endif
#ifdef USE_DIRECTX
	struct ID3DXBuffer;
#endif

namespace common
{

/** \addtogroup code_error Modu Error
Dokumentacja: \ref Module_Error \n
Nagwek: Error.hpp */
//@{

/// Klasa bazowa bdw
class Error
{
private:
	std::vector<tstring> m_Msgs;

protected:
	/// Uyj tego konstruktora, jeli musisz najpierw co policzy, a dopiero potem utworzy komunikat o bdzie.
	Error() { }

public:
	/// Dodaje komunikat na stos.
	/** \param file uyj __TFILE__
	\param line uyj __LINE__ */
	void Push(const tstring &msg, const tstring &file = _T(""), int line = 0);
	/// Zwraca peny opis bdu w jednym acuchu
	/** \param Indent acuch od ktrego ma si zaczyna kady komunikat (wcicie).
	\param EOL acuch oddzielajcy komunikaty (koniec wiersza). Nie doczany na kocu. */
	void GetMessage_(tstring *Msg, const tstring &Indent = _T(""), const tstring &EOL = EOL) const;

	/// Utworzenie oglnego bdu
	Error(const tstring &msg, const tstring &file = _T(""), int line = 0)
	{
		Push(msg, file, line);
	}
};

/// Wyjtek utworzony z systemowego bdu errno
class ErrnoError : public Error
{
public:
	/// Tworzy bd z podanym kodem.
	ErrnoError(int err_code, const tstring &msg, const tstring &file = _T(""), int line = 0);
	/// Tworzy bd z kodem pobieranym ze zmiennej errno.
	ErrnoError(const tstring &msg, const tstring &file = _T(""), int line = 0);
};

#ifdef USE_SDL
	/// Wyjtek z bdem pobranym z biblioteki SDL za pomoc SDLError
	class SDLError : public Error
	{
	public:
		SDLError(const tstring &msg, const tstring &file = _T(""), int line = 0);
	};
#endif

#ifdef USE_OPENGL
	/// Wyjtek z bdem pobranym z biblioteki OpenGL za pomoc OpenGLError
	class OpenGLError : public Error
	{
	public:
		OpenGLError(const tstring &msg, const tstring &file = _T(""), int line = 0);
	};
#endif

#ifdef USE_FMOD
	/// Wyjtek z bdem pobranym z biblioteki dwikowej FMOD
	class FmodError : public Error
	{
	private:
		/** \param[out] Enum Moe by NULL.
		\param[out] Message Moe by NULL. */
		void CodeToMessage(FMOD_RESULT code, tstring *Enum, tstring *Message);

	public:
		FmodError(FMOD_RESULT code, const tstring &msg, const tstring &file = _T(""), int line = 0);
	};
#endif

#ifdef WIN32
	/// Tworzy wyjtek na podstawie bdu WinAPI pobieranego za pomoc GetLastError
	class Win32Error : public Error
	{
	public:
		Win32Error(const tstring &msg = _T(""), const tstring &file = _T(""), int line = 0);
	};
#endif

#ifdef USE_DIRECTX
	/// Tworzy wyjtek na podstawie bdu biblioteki Direct3D.
	class DirectXError : public Error
	{
	public:
		DirectXError(HRESULT hr, const tstring &Msg = _T(""), const tstring &file = _T(""), int Line = 0);
		/// Uyj tej wersji jeli funkcja DX ktr kontroluje zwraca te w przypadku bdu bufor z komunikatem
		/** Bufor zostanie przez konstruktor tego wyjtku sam zwolniony, jeli istnia. Moe nie istnie. */
		DirectXError(HRESULT hr, ID3DXBuffer *Buf, const tstring &Msg = _T(""), const tstring &file = _T(""), int Line = 0);
	};
#endif

#ifdef USE_WINSOCK
	/// Tworzy wyjtek na podstawie bdu biblioteki WinSock.
	class WinSockError : public Error
	{
	private:
		void CodeToStr(int Code, tstring *Name, tstring *Message);

	public:
		/// Tworzy komunikat bdu WinSockError
		int WinSockError(int Code, const tstring &Msg = "", const tstring &file = _T(""), int Line = 0);
		/// Tworzy komunikat bdu WinSockError, pobiera kod sam z WSAGetLastError()
		int WinSockError(const tstring &Msg = "", const tstring &file = _T(""), int Line = 0);
	};
#endif

#ifdef USE_DEVIL
	/// Tworzy wyjtek na podstawie bdu biblioteki graficznej DevIL
	/* Poniewa nie sposb niestety w DevIL sprawdzi czy jest bd bez zdjcia jednego
	kodu bdu ze szczytu stosu, naley wywoa ILenum code = ilGetError() i jeli
	code != IL_NO_ERROR, to wtedy rzuci ten wyjtek podajc jako pierwszy parametr code.
	*/
	class DevILError : public Error
	{
	public:
		DevILError(unsigned FirstCode, const tstring &Msg = "", const tstring &File = _T(""), int Line = 0);
	};
#endif

#ifdef USE_AVI_FILE
	/// Tworzy wyjtek na podstawie bdu biblioteki AviFile (cz WinAPI)
	class AVIFileError : public Error
	{
	public:
		AVIFileError(HRESULT hr, const tstring &Msg = "", const tstring &file = _T(""), int a_Line = 0);
	};
#endif

//@}
// code_error

} // namespace common

/** \addtogroup code_error */
//@{

/// Uyj aby doda komunikat do stosu wyjtku
#define ERR_ADD_FUNC(exception) { (exception).Push(__TFUNCSIG__, __TFILE__, __LINE__); }
#define ERR_TRY        { try {
#define ERR_CATCH(msg) } catch(Error &e) { e.Push((msg), __TFILE__, __LINE__); throw; } }
#define ERR_CATCH_FUNC } catch(Error &e) { ERR_ADD_FUNC(e); throw; } }

#define ERR_GUARD_BOOL(Expr) { if ((Expr) == false) throw Error(_T(#Expr), __TFILE__, __LINE__); }
#define ERR_GUARD_DIRECTX(Expr) { HRESULT hr; if (FAILED(hr = (Expr))) throw DirectXError(hr, _T(#Expr), __TFILE__, __LINE__); }
#define ERR_GUARD_DEVIL(Code) { { Code ; } ILenum c = ilGetError(); if (c != IL_NO_ERROR) throw DevILError(c, _T(#Code), __TFILE__, __LINE__); }

#ifdef _DEBUG
	#define ERR_GUARD_BOOL_D(Expr) ERR_GUARD_BOOL(Expr)
	#define ERR_GUARD_DIRECTX_D(Expr) ERR_GUARD_DIRECTX(Expr)
#else
	#define ERR_GUARD_BOOL_D(Expr) (Expr);
	#define ERR_GUARD_DIRECTX_D(Expr) (Expr);
#endif

//@}
// code_error

/// Rozszerzenie moduu bazowego o wsparcie dla bdw
/**
\ingroup code_sthtostr
\ingroup code_error */
template <typename T>
inline void MustStrToSth(T *Sth, const tstring &Str)
{
	if (!StrToSth<T>(Sth, Str))
		throw common::Error(_T("String conversion error: ") + Str);
}

#endif
