/** \file
\brief Przyrzd do pomiarw czasu i wydajnoci
\author Adam Sawicki - sawickiap@poczta.onet.pl - http://regedit.gamedev.pl/ \n

Cz biblioteki CommonLib \n
Kodowanie Windows-1250, koniec wiersza CR+LF, test: Za gl ja \n
Licencja: GNU GPL. \n
Dokumentacja: \ref Module_Profiler \n
Elementy moduu: \ref code_profiler
*/
#include "Base.hpp"
#include "Profiler.hpp"


namespace common
{

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
// Klasa ProfilerItem

ProfilerItem::ProfilerItem(const tstring &Name)
{
	m_Time = 0.0;
	m_Count = 0.0;
	m_strName = Name;
}

ProfilerItem* ProfilerItem::Begin(const tstring &Name)
{
	// Poszukiwanie istniejcego elementu
	for (std::vector<ProfilerItem>::iterator it = m_ItemVector.begin(); it != m_ItemVector.end(); ++it)
	{
		if (it->GetName() == Name)
			return &(*it);
	}
	// Nie znaleziono - utworzenie nowego
	m_ItemVector.push_back( ProfilerItem(Name) );
	return &(*(--m_ItemVector.end()));
}

void ProfilerItem::Start()
{
	m_StartTime = g_Timer.GetTimeD();
}

void ProfilerItem::Stop()
{
	m_Time += ( g_Timer.GetTimeD() - m_StartTime );
	m_Count += 1.0;
}

void ProfilerItem::FormatString(tstring *S, unsigned dwLevel)
{
	unsigned i;
	if (dwLevel > 0)
	{
		tstring Tmp;
		DupeString(&Tmp, _T("  "), dwLevel-1);
		*S += Tmp;
		*S += Format(_T("# : # ms (#)\n"))
			% m_strName
			% DoubleToStrR(GetAvgTime()*1000.0)
			% UintToStrR(static_cast<uint4>(GetCount()));
	}
	for (i = 0; i < GetItemCount(); i++)
		GetItem(i)->FormatString(S, dwLevel+1);
}


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
// Klasa Profiler

Profiler::Profiler() : m_DefaultItem(_T(""))
{
	m_ItemStack.push(&m_DefaultItem);
}

Profiler::Profiler(const tstring &Name) : m_DefaultItem(Name)
{
	m_ItemStack.push(&m_DefaultItem);
}

void Profiler::Begin(const tstring &Name)
{
	ProfilerItem* pItem = ( m_ItemStack.top() )->Begin(Name);
	pItem->Start();
	m_ItemStack.push(pItem);
}

void Profiler::End()
{
	if (m_ItemStack.size() > 1)
	{
		ProfilerItem* pItem = m_ItemStack.top();
		pItem->Stop();
		m_ItemStack.pop();
	}
	else
		// Bd !!!
		;
}

void Profiler::FormatString(tstring *S)
{
	S->clear();
	GetRootItem()->FormatString(S, 0);
}


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
// Klasa Profile

Profile::Profile(const tstring &Name) :
	m_Profiler(g_Profiler)
{
	m_Profiler.Begin(Name);
}

Profile::Profile(const tstring &Name, Profiler &Profiler_) :
	m_Profiler(Profiler_)
{
	m_Profiler.Begin(Name);
}

Profile::~Profile()
{
	m_Profiler.End();
}


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
// Elementy globalne

Profiler g_Profiler;

} // namespace common
