/** \file
\brief Przyrzd do pomiarw czasu i wydajnoci
\author Adam Sawicki - sawickiap@poczta.onet.pl - http://regedit.gamedev.pl/ \n

Cz biblioteki CommonLib \n
Kodowanie Windows-1250, koniec wiersza CR+LF, test: Za gl ja \n
Licencja: GNU GPL. \n
Dokumentacja: \ref Module_Profiler \n
Elementy moduu: \ref code_profiler
*/
#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif
#ifndef COMMON_PROFILER_H_
#define COMMON_PROFILER_H_

#include <stack> // :(

namespace common
{

/** \addtogroup code_profiler Modu Profiler
Dokumentacja: \ref Module_Profiler \n
Nagwek: Profiler.hpp */
//@{

class Profiler;

/// Pozycja danych profilera
class ProfilerItem
{
private:
	// Sumaryczny czas wykonania [s]
	double m_Time;
	// Liczba przebiegw
	double m_Count;
	// Czas rozpoczcia biecego przebiegu [s]
	double m_StartTime;
	tstring m_strName; // Nazwa elementu
	std::vector<ProfilerItem> m_ItemVector; // Podelementy

	friend class Profiler;
		ProfilerItem* Begin(const tstring &Name);
		void Start();
		void Stop();
public:
	ProfilerItem(const tstring &Name);

	const tstring &GetName() { return m_strName; }
	bool Empty() { return (around(m_Count, 0.1)); }
	double GetCount() { return m_Count; }
	double GetAvgTime() { return ( Empty() ? 0.0 : m_Time / m_Count ); }
	size_t GetItemCount() { return m_ItemVector.size(); }
	ProfilerItem* GetItem(size_t index) { return &m_ItemVector[index]; }
	void FormatString(tstring *S, unsigned dwLevel);
};

/// Profiler
class Profiler
{
private:
	std::stack<ProfilerItem*> m_ItemStack;
	ProfilerItem m_DefaultItem;

public:
	Profiler();
	Profiler(const tstring &Name);
	void Begin(const tstring &Name);
	void End();
	ProfilerItem* GetRootItem() { return &m_DefaultItem; }
	/// Zapisuje cae drzewo profilu do acucha - kada pozycja w osobnym wierszu
	/** Wcicia to dwie spacje <tt>"  "</tt>
	Koce wiersza to <tt>\\n</tt>
	Jednostka to milisekundy. */
	void FormatString(tstring *S);
};

/// Klasa, ktrej obiekt moesz dla wygody utworzy zamiast wywoywa Begin i End profilera.
/** Tworzenie tego obiektu wywouje Profiler::Begin, a usuwanie - Profiler::End.
Obiekt automatyczny utworzony na stosie sam si usunie przy wyjciu
z funkcji. Dlatego warto tworzy obiekt tej klasy jako zmienn lokaln o byle
jakiej nazwie i nieuywan nigdzie dalej na pocztku kadej profilowanej
funkcji. */
class Profile
{
private:
	Profiler &m_Profiler;
public:
	/// Uywa g_Profiler
	Profile(const tstring &Name);
	/// Uywa podanego, wasnego profilera
	Profile(const tstring &Name, Profiler &Profiler_);
	~Profile();
};

/// Gwny profiler globalny dla wtku gwnego
extern Profiler g_Profiler;

//@}
// code_profiler

} // namespace common

/** \addtogroup code_profiler */
//@{
/** Dla jeszcze wiszej wygody, zamiast tworzy obiekt klasy Profile wystarczy
na pocztku guardowanej do profilowania funkcji czy dowolnego bloku { }
postawi to makro. */
#define PROFILE_GUARD(Name) common::Profile __profile_guard_object(Name);
//@}

#endif

