/** \file
\brief Modu matematyczny
\author Adam Sawicki - sawickiap@poczta.onet.pl - http://regedit.gamedev.pl/ \n

Cz biblioteki CommonLib \n
Kodowanie Windows-1250, koniec wiersza CR+LF, test: Za gl ja \n
Licencja: GNU GPL. \n
Dokumentacja: \ref Module_Math \n
Elementy moduu: \ref code_math
*/
#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif
#ifndef COMMON_MATH_H_
#define COMMON_MATH_H_

#ifdef WIN32
	#include <windows.h> // bo bez tego w d3d9types.h s bdy bo nie zna DWORD itp., poza tym potrzebny jest Point itd.
#endif

#ifdef USE_DIRECTX
	#include <d3d9types.h>
	#include <d3dx9math.h>
#endif

namespace common
{

/** \addtogroup code_math Modu Math
Dokumentacja: \ref Module_Math \n
Nagwek: Math.hpp */
//@{

/// Szablon bazowy do konwersji midzy typami matematycznymi
/** Ta wersja nie dziaa, specjalizacje bd dziaa. */
template <typename DestT, typename SrcT>
DestT & math_cast(const SrcT &x)
{
	assert(0 && "Unsupported type in math_cast.");
}

/// Maa staa
const float FLOAT_EPSILON = 1e-6f;


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_point Punkt 2D z liczb cakowitych */
//@{

/// Punkt 2D z liczb cakowitych
/**
Kompatybilny ze struktur POINT z Win32API.
Nazywa si tak dziwnie, bo POINT to w Windows API marko i robi problem z nazw.
Konwersja do i ze stringa: "x,y", np. "10,1".
*/
struct POINT_
{
	union
	{
		struct
		{
			int x, y;
		};
		int v[2];
	};

	static const POINT_ ZERO;

	POINT_() { }
	POINT_(int x, int y) : x(x), y(y) { }

	bool operator == (const POINT_ &v) const { return x == v.x && y == v.y; }
	bool operator != (const POINT_ &v) const { return x != v.x || y != v.y; }

	POINT_ operator - () const { return POINT_(-x, -y); }
	POINT_ operator + (const POINT_ &v) const { return POINT_(x+v.x, y+v.y); }
	POINT_ operator - (const POINT_ &v) const { return POINT_(x-v.x, y-v.y); }
	POINT_ & operator += (const POINT_ &v) { x += v.x; y += v.y; return *this; }
	POINT_ & operator -= (const POINT_ &v) { x -= v.x; y -= v.y; return *this; }
	POINT_ & operator *= (int v) { x *= v; y *= v; return *this; }
	POINT_ & operator /= (int v) { x /= v; y /= v; return *this; }

	int & operator [] (uint4 Index) { return v[Index]; }
	const int & operator [] (uint4 Index) const { return v[Index]; }

	int * GetArray() { return &x; }
	const int * GetArray() const { return &x; }
};

inline POINT_ operator * (const POINT_ pt, int v) { return POINT_(pt.x*v, pt.y*v); }
inline POINT_ operator / (const POINT_ pt, int v) { return POINT_(pt.x/v, pt.y/v); }
inline POINT_ operator * (int v, const POINT_ pt) { return POINT_(pt.x*v, pt.y*v); }
inline POINT_ operator / (int v, const POINT_ pt) { return POINT_(pt.x/v, pt.y/v); }

#ifdef WIN32
	/// Konwersja z common::POINT_ do POINT
	template <> inline ::POINT & math_cast<::POINT, POINT_>(const POINT_ &x) { return (::POINT&)(x); }
	/// Konwersja z POINT do common::POINT_
	template <> inline POINT_ & math_cast<POINT_, ::POINT>(const ::POINT &x) { return (POINT_&)(x); }
#endif

inline void swap(POINT_ &v1, POINT_ &v2)
{
	std::swap(v1.x, v2.x);
	std::swap(v1.y, v2.y);
}

//@}
// math_point

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_vec Wektor floatw 2D, 3D, 4D */
//@{

#if defined(USE_DIRECTX) && !defined(DOXYGEN)
	typedef D3DXVECTOR2 VEC2;
#else
	/// Wektor floatw 2D
	struct VEC2
	{
		float x, y;

		VEC2() { }
		VEC2(float x, float y) : x(x), y(y) { }
		VEC2(const float *Array) : x(Array[0]), y(Array[1]) { }

		operator float * () { return &x; }
		operator const float * () const { return &x; }

		bool operator == (const VEC2 &v) const { return x == v.x && y == v.y; }
		bool operator != (const VEC2 &v) const { return x != v.x || y != v.y; }

		VEC2 operator + () const { return *this; }
		VEC2 operator - () const { return VEC2(-x, -y); }
		VEC2 operator + (const VEC2 &v) const { return VEC2(x+v.x, y+v.y); }
		VEC2 operator - (const VEC2 &v) const { return VEC2(x-v.x, y-v.y); }
		VEC2 & operator += (const VEC2 &v) { x += v.x; y += v.y; return *this; }
		VEC2 & operator -= (const VEC2 &v) { x -= v.x; y -= v.y; return *this; }
		VEC2 & operator *= (float v) { x *= v; y *= v; return *this; }
		VEC2 & operator /= (float v) { float vInv = 1.0f/v; x *= vInv; y *= vInv; return *this; }
	};
	inline VEC2 operator * (const VEC2 pt, float v) { return VEC2(pt.x*v, pt.y*v); }
	inline VEC2 operator / (const VEC2 pt, float v) { float vInv = 1.0f/v; return VEC2(pt.x*vInv, pt.y*vInv); }
	inline VEC2 operator * (float v, const VEC2 pt) { return VEC2(pt.x*v, pt.y*v); }
	inline VEC2 operator / (float v, const VEC2 pt) { float vInv = 1.0f/v; return VEC2(pt.x*vInv, pt.y*vInv); }
#endif

inline void swap(VEC2 &v1, VEC2 &v2)
{
	std::swap(v1.x, v2.x);
	std::swap(v1.y, v2.y);
}

#if defined(USE_DIRECTX) && !defined(DOXYGEN)
	typedef D3DXVECTOR3 VEC3;
#else
	/// Wektor floatw 3D
	struct VEC3
	{
		float x, y, z;

		VEC3() { }
		VEC3(float x, float y, float z) : x(x), y(y), z(z) { }
		VEC3(const float *Array) : x(Array[0]), y(Array[1]), z(Array[2]) { }

		operator float * () { return &x; }
		operator const float * () const { return &x; }

		bool operator == (const VEC3 &v) const { return x == v.x && y == v.y && z == v.z; }
		bool operator != (const VEC3 &v) const { return x != v.x || y != v.y || z != v.z; }

		VEC3 operator + () const { return *this; }
		VEC3 operator - () const { return VEC3(-x, -y, -z); }
		VEC3 operator + (const VEC3 &v) const { return VEC3(x+v.x, y+v.y, z+v.z); }
		VEC3 operator - (const VEC3 &v) const { return VEC3(x-v.x, y-v.y, z-v.z); }
		VEC3 & operator += (const VEC3 &v) { x += v.x; y += v.y; z += v.z; return *this; }
		VEC3 & operator -= (const VEC3 &v) { x -= v.x; y -= v.y; z -= v.z; return *this; }
		VEC3 & operator *= (float v) { x *= v; y *= v; z *= v; return *this; }
		VEC3 & operator /= (float v) { float vInv = 1.0f/v; x *= vInv; y *= vInv; z *= vInv; return *this; }
	};
	inline VEC3 operator * (const VEC3 pt, float v) { return VEC3(pt.x*v, pt.y*v, pt.z*v); }
	inline VEC3 operator / (const VEC3 pt, float v) { float vInv = 1.0f/v; return VEC3(pt.x*vInv, pt.y*vInv, pt.z*vInv); }
	inline VEC3 operator * (float v, const VEC3 pt) { return VEC3(pt.x*v, pt.y*v, pt.z*v); }
	inline VEC3 operator / (float v, const VEC3 pt) { float vInv = 1.0f/v; return VEC3(pt.x*vInv, pt.y*vInv, pt.z*vInv); }
#endif

inline void swap(VEC3 &v1, VEC3 &v2)
{
	std::swap(v1.x, v2.x);
	std::swap(v1.y, v2.y);
	std::swap(v1.z, v2.z);
}

#if defined(USE_DIRECTX) && !defined(DOXYGEN)
	typedef D3DXVECTOR4 VEC4;
#else
	/// Wektor floatw 4D
	struct VEC4
	{
		float x, y, z, w;

		VEC4() { }
		VEC4(float x, float y, float z, float w) : x(x), y(y), z(z), w(w) { }
		VEC4(const float *Array) : x(Array[0]), y(Array[1]), z(Array[2]), w(Array[3]) { }

		operator float * () { return &x; }
		operator const float * () const { return &x; }

		bool operator == (const VEC4 &v) const { return x == v.x && y == v.y && z == v.z && w == v.w; }
		bool operator != (const VEC4 &v) const { return x != v.x || y != v.y || z != v.z || w != v.w; }

		VEC4 operator + () const { return *this; }
		VEC4 operator - () const { return VEC4(-x, -y, -z, -w); }
		VEC4 operator + (const VEC4 &v) const { return VEC4(x+v.x, y+v.y, z+v.z, w+v.w); }
		VEC4 operator - (const VEC4 &v) const { return VEC4(x-v.x, y-v.y, z-v.z, w-v.w); }
		VEC4 & operator += (const VEC4 &v) { x += v.x; y += v.y; z += v.z; w += v.w; return *this; }
		VEC4 & operator -= (const VEC4 &v) { x -= v.x; y -= v.y; z -= v.z; w -= v.w; return *this; }
		VEC4 & operator *= (float v) { x *= v; y *= v; z *= v; w *= v; return *this; }
		VEC4 & operator /= (float v) { float vInv = 1.0f/v; x *= vInv; y *= vInv; z *= vInv; w *= vInv; return *this; }
	};
	inline VEC4 operator * (const VEC4 pt, float v) { return VEC4(pt.x*v, pt.y*v, pt.z*v, pt.w*v); }
	inline VEC4 operator / (const VEC4 pt, float v) { float vInv = 1.0f/v; return VEC4(pt.x*vInv, pt.y*vInv, pt.z*vInv, pt.w*vInv); }
	inline VEC4 operator * (float v, const VEC4 pt) { return VEC4(pt.x*v, pt.y*v, pt.z*v, pt.w*v); }
	inline VEC4 operator / (float v, const VEC4 pt) { float vInv = 1.0f/v; return VEC4(pt.x*vInv, pt.y*vInv, pt.z*vInv, pt.w*vInv); }
#endif

inline void swap(VEC4 &v1, VEC4 &v2)
{
	std::swap(v1.x, v2.x);
	std::swap(v1.y, v2.y);
	std::swap(v1.z, v2.z);
	std::swap(v1.w, v2.w);
}

extern const VEC2 VEC2_ZERO;
extern const VEC2 VEC2_ONE;
extern const VEC2 VEC2_POSITIVE_X;
extern const VEC2 VEC2_POSITIVE_Y;
extern const VEC2 VEC2_NEGATIVE_X;
extern const VEC2 VEC2_NEGATIVE_Y;

extern const VEC3 VEC3_ZERO;
extern const VEC3 VEC3_ONE;
extern const VEC3 VEC3_POSITIVE_X;
extern const VEC3 VEC3_POSITIVE_Y;
extern const VEC3 VEC3_POSITIVE_Z;
extern const VEC3 VEC3_NEGATIVE_X;
extern const VEC3 VEC3_NEGATIVE_Y;
extern const VEC3 VEC3_NEGATIVE_Z;

extern const VEC4 VEC4_ZERO;
extern const VEC4 VEC4_ONE;
extern const VEC4 VEC4_POSITIVE_X;
extern const VEC4 VEC4_POSITIVE_Y;
extern const VEC4 VEC4_POSITIVE_Z;
extern const VEC4 VEC4_POSITIVE_W;
extern const VEC4 VEC4_NEGATIVE_X;
extern const VEC4 VEC4_NEGATIVE_Y;
extern const VEC4 VEC4_NEGATIVE_Z;
extern const VEC4 VEC4_NEGATIVE_W;

/// Zwraca kwadrat dugoci wektora
inline float LengthSq(const VEC2 &v)
{
	return v.x*v.x + v.y*v.y;
}
/// \copydoc LengthSq(const VEC2&)
inline float LengthSq(const VEC3 &v)
{
	return v.x*v.x + v.y*v.y + v.z*v.z;
}
/// \copydoc LengthSq(const VEC2&)
inline float LengthSq(const VEC4 &v)
{
	return v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w;
}

/// Zwraca dugo wektora
inline float Length(const VEC2 &v)
{
	return sqrtf(LengthSq(v));
}
/// \copydoc Length(const VEC2&)
inline float Length(const VEC3 &v)
{
	return sqrtf(LengthSq(v));
}
/// \copydoc Length(const VEC2&)
inline float Length(const VEC4 &v)
{
	return sqrtf(LengthSq(v));
}

/// Zwraca wektor zoony z mniejszych skadowych podanych wektorw
inline void Min(VEC2 *Out, const VEC2 &v1, const VEC2 &v2)
{
	Out->x = std::min(v1.x, v2.x);
	Out->y = std::min(v1.y, v2.y);
}
inline void Min(VEC3 *Out, const VEC3 &v1, const VEC3 &v2)
{
	Out->x = std::min(v1.x, v2.x);
	Out->y = std::min(v1.y, v2.y);
	Out->z = std::min(v1.z, v2.z);
}
inline void Min(VEC4 *Out, const VEC4 &v1, const VEC4 &v2)
{
	Out->x = std::min(v1.x, v2.x);
	Out->y = std::min(v1.y, v2.y);
	Out->z = std::min(v1.z, v2.z);
	Out->w = std::min(v1.w, v2.w);
}

/// Zwraca wektor zoony z wikszych skadowych podanych wektorw
inline void Max(VEC2 *Out, const VEC2 &v1, const VEC2 &v2)
{
	Out->x = std::max(v1.x, v2.x);
	Out->y = std::max(v1.y, v2.y);
}
inline void Max(VEC3 *Out, const VEC3 &v1, const VEC3 &v2)
{
	Out->x = std::max(v1.x, v2.x);
	Out->y = std::max(v1.y, v2.y);
	Out->z = std::max(v1.z, v2.z);
}
inline void Max(VEC4 *Out, const VEC4 &v1, const VEC4 &v2)
{
	Out->x = std::max(v1.x, v2.x);
	Out->y = std::max(v1.y, v2.y);
	Out->z = std::max(v1.z, v2.z);
	Out->w = std::max(v1.w, v2.w);
}

/** \addtogroup math_vec_componentop Operacje na poszczeglnych skadowych wektorw */
//@{

inline void Mul(VEC2 *Out, const VEC2 &v, float f)
{
	Out->x = v.x * f;
	Out->y = v.y * f;
}
inline void Mul(VEC3 *Out, const VEC3 &v, float f)
{
	Out->x = v.x * f;
	Out->y = v.y * f;
	Out->z = v.z * f;
}
inline void Mul(VEC4 *Out, const VEC4 &v, float f)
{
	Out->x = v.x * f;
	Out->y = v.y * f;
	Out->z = v.z * f;
	Out->w = v.w * f;
}

inline void Add(VEC2 *Out, const VEC2 &v1, const VEC2 &v2)
{
	Out->x = v1.x + v2.x;
	Out->y = v1.y + v2.y;
}
inline void Add(VEC3 *Out, const VEC3 &v1, const VEC3 &v2)
{
	Out->x = v1.x + v2.x;
	Out->y = v1.y + v2.y;
	Out->z = v1.z + v2.z;
}
inline void Add(VEC4 *Out, const VEC4 &v1, const VEC4 &v2)
{
	Out->x = v1.x + v2.x;
	Out->y = v1.y + v2.y;
	Out->z = v1.z + v2.z;
	Out->w = v1.w + v2.w;
}

inline void Sub(VEC2 *Out, const VEC2 &v1, const VEC2 &v2)
{
	Out->x = v1.x - v2.x;
	Out->y = v1.y - v2.y;
}
inline void Sub(VEC3 *Out, const VEC3 &v1, const VEC3 &v2)
{
	Out->x = v1.x - v2.x;
	Out->y = v1.y - v2.y;
	Out->z = v1.z - v2.z;
}
inline void Sub(VEC4 *Out, const VEC4 &v1, const VEC4 &v2)
{
	Out->x = v1.x - v2.x;
	Out->y = v1.y - v2.y;
	Out->z = v1.z - v2.z;
	Out->w = v1.w - v2.w;
}

inline void Mul(VEC2 *Out, const VEC2 &v1, const VEC2 &v2)
{
	Out->x = v1.x * v2.x;
	Out->y = v1.y * v2.y;
}
inline void Mul(VEC3 *Out, const VEC3 &v1, const VEC3 &v2)
{
	Out->x = v1.x * v2.x;
	Out->y = v1.y * v2.y;
	Out->z = v1.z * v2.z;
}
inline void Mul(VEC4 *Out, const VEC4 &v1, const VEC4 &v2)
{
	Out->x = v1.x * v2.x;
	Out->y = v1.y * v2.y;
	Out->z = v1.z * v2.z;
	Out->w = v1.w * v2.w;
}

inline void Div(VEC2 *Out, const VEC2 &v1, const VEC2 &v2)
{
	Out->x = v1.x / v2.x;
	Out->y = v1.y / v2.y;
}
inline void Div(VEC3 *Out, const VEC3 &v1, const VEC3 &v2)
{
	Out->x = v1.x / v2.x;
	Out->y = v1.y / v2.y;
	Out->z = v1.z / v2.z;
}
inline void Div(VEC4 *Out, const VEC4 &v1, const VEC4 &v2)
{
	Out->x = v1.x / v2.x;
	Out->y = v1.y / v2.y;
	Out->z = v1.z / v2.z;
	Out->w = v1.w / v2.w;
}

//@}
// math_vec_componentop

/// Iloczyn skalarny wektorw
inline float Dot(const VEC2 &v1, const VEC2 &v2)
{
	return v1.x*v2.x + v1.y*v2.y;
}
inline float Dot(const VEC3 &v1, const VEC3 &v2)
{
	return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
}
inline float Dot(const VEC4 &v1, const VEC4 &v2)
{
	return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z + v1.w*v2.w;
}

/// Odwraca znak skadowych wektora
inline void Minus(VEC2 *Out, VEC2 &v) { Out->x = -v.x; Out->y = -v.y; }
inline void Minus(VEC3 *Out, VEC3 &v) { Out->x = -v.x; Out->y = -v.y; Out->z = -v.z; }
inline void Minus(VEC4 *Out, VEC4 &v) { Out->x = -v.x; Out->y = -v.y; Out->z = -v.z; Out->w = -v.w; }
inline void Minus(VEC2 *InOut) { InOut->x = -InOut->x; InOut->y = -InOut->y; }
inline void Minus(VEC3 *InOut) { InOut->x = -InOut->x; InOut->y = -InOut->y; InOut->z = -InOut->z; }
inline void Minus(VEC4 *InOut) { InOut->x = -InOut->x; InOut->y = -InOut->y; InOut->z = -InOut->z; InOut->w = -InOut->w; }

/// Iloczyn wektorowy dwch wektorw dajcy liczb mwic czy drugi wskazuje na lewo, czy na prawo wzgldem pierwszego
/** W ukadzie gdzie X jest w prawo a Y do gry, > 0 znaczy na lewo \n
W ukadzie gdzie X jest w prawo a Y w d, > 0 znaczy na prawo */
inline float Cross(const VEC2 &v1, const VEC2 &v2)
{
	return v1.x*v2.y - v1.y*v2.x;
}

/// Iloczyn wektorowy dwch wektorw 3D
inline void Cross(VEC3 *Out, const VEC3 &v1, const VEC3 &v2)
{
	Out->x = v1.y * v2.z - v1.z * v2.y;
	Out->y = v1.z * v2.x - v1.x * v2.z;
	Out->z = v1.x * v2.y - v1.y * v2.x;
}

/// Kwadrat odlegoci midzy dwoma punktami
inline float DistanceSq(const VEC2 &p1, const VEC2 &p2)
{
	return LengthSq(p2-p1);
}
inline float DistanceSq(const VEC3 &p1, const VEC3 &p2)
{
	return LengthSq(p2-p1);
}
inline float DistanceSq(const VEC4 &p1, const VEC4 &p2)
{
	return LengthSq(p2-p1);
}

/// Odlego midzy dwoma punktami
inline float Distance(const VEC2 &p1, const VEC2 &p2)
{
	return Length(p2-p1);
}
inline float Distance(const VEC3 &p1, const VEC3 &p2)
{
	return Length(p2-p1);
}
inline float Distance(const VEC4 &p1, const VEC4 &p2)
{
	return Length(p2-p1);
}

/// Liniowa interpolacja wektorw
inline void Lerp(VEC2 *Out, const VEC2 &v1, const VEC2 &v2, float t)
{
	Out->x = v1.x + t*(v2.x-v1.x);
	Out->y = v1.y + t*(v2.y-v1.y);
}
inline void Lerp(VEC3 *Out, const VEC3 &v1, const VEC3 &v2, float t)
{
	Out->x = v1.x + t*(v2.x-v1.x);
	Out->y = v1.y + t*(v2.y-v1.y);
	Out->z = v1.z + t*(v2.z-v1.z);
}
inline void Lerp(VEC4 *Out, const VEC4 &v1, const VEC4 &v2, float t)
{
	Out->x = v1.x + t*(v2.x-v1.x);
	Out->y = v1.y + t*(v2.y-v1.y);
	Out->z = v1.z + t*(v2.z-v1.z);
	Out->w = v1.w + t*(v2.w-v1.w);
}

/// Normalizuje wektor, czyli zmienia mu dugo na 1
inline void Normalize(VEC2 *Out, const VEC2 &v)
{
	float l = 1.0f / Length(v);
	Out->x = v.x * l;
	Out->y = v.y * l;
}
inline void Normalize(VEC3 *Out, const VEC3 &v)
{
	float l = 1.0f / Length(v);
	Out->x = v.x * l;
	Out->y = v.y * l;
	Out->z = v.z * l;
}
inline void Normalize(VEC4 *Out, const VEC4 &v)
{
	float l = 1.0f / Length(v);
	Out->x = v.x * l;
	Out->y = v.y * l;
	Out->z = v.z * l;
	Out->w = v.w * l;
}

/// Normalizuje wektor w miejscu
inline void Normalize(VEC2 *v)
{
	float rcl = 1.0f / Length(*v);
	v->x *= rcl;
	v->y *= rcl;
}
inline void Normalize(VEC3 *v)
{
	float rcl = 1.0f / Length(*v);
	v->x *= rcl;
	v->y *= rcl;
	v->z *= rcl;
}
inline void Normalize(VEC4 *v)
{
	float rcl = 1.0f / Length(*v);
	v->x *= rcl;
	v->y *= rcl;
	v->z *= rcl;
	v->w *= rcl;
}

/// Konwersja z wektora 3D na 4D dopisujca W=1
inline void Vec3ToVec4(VEC4 *Out, const VEC3 &v)
{
	Out->x = v.x;
	Out->y = v.y;
	Out->z = v.z;
	Out->w = 1.0f;
}

/// Konwersja z wektora 4D na 3D ze zignorowaniem skadowej W
inline void Vec4ToVec3_Ignore(VEC3 *Out, const VEC4 &v)
{
	Out->x = v.x;
	Out->y = v.y;
	Out->z = v.z;
}
/// Konwersja z wektora 4D na 3D ze podzieleniem skadowych przez W
inline void Vec4ToVec3_Div(VEC3 *Out, const VEC4 &v)
{
	float rcw = 1.0f / v.w;
	Out->x = v.x * rcw;
	Out->y = v.y * rcw;
	Out->z = v.z * rcw;
}

/// Zwraca punkt w poowie drogi midzy podanymi punktami
inline void MidPoint(VEC2 *Out, const VEC2 &p1, const VEC2 &p2)
{
	Out->x = (p1.x+p2.x) * 0.5f;
	Out->y = (p1.y+p2.y) * 0.5f;
}
inline void MidPoint(VEC3 *Out, const VEC3 &p1, const VEC3 &p2)
{
	Out->x = (p1.x+p2.x) * 0.5f;
	Out->y = (p1.y+p2.y) * 0.5f;
	Out->z = (p1.z+p2.z) * 0.5f;
}
inline void MidPoint(VEC4 *Out, const VEC4 &p1, const VEC4 &p2)
{
	Out->x = (p1.x+p2.x) * 0.5f;
	Out->y = (p1.y+p2.y) * 0.5f;
	Out->z = (p1.z+p2.z) * 0.5f;
	Out->w = (p1.w+p2.w) * 0.5f;
}

/// Zwraca true, jeli podane wektory/punkty s sobie mniej wicej rwne
inline bool VecEqual(const VEC2 &v1, const VEC2 &v2) {
	return float_equal(v1.x, v2.x) && float_equal(v1.y, v2.y);
}
inline bool VecEqual(const VEC3 &v1, const VEC3 &v2) {
	return float_equal(v1.x, v2.x) && float_equal(v1.y, v2.y) && float_equal(v1.z, v2.z);
}
inline bool VecEqual(const VEC4 &v1, const VEC4 &v2) {
	return float_equal(v1.x, v2.x) && float_equal(v1.y, v2.y) && float_equal(v1.z, v2.z) && float_equal(v1.w, v2.w);
}

/** \addtogroup math_vec_comp Porwnania wektorw
Zwraca true, jeli wszystkie skadowe podanego wektora s odpowiednio
mniejsze / wiksze / mniejsze lub rwne / wiksze lub rwne od drugiego.
*/
//@{

inline bool Less(const VEC2 &v1, const VEC2 &v2)
{
	return v1.x < v2.x && v1.y < v2.y;
}
inline bool Less(const VEC3 &v1, const VEC3 &v2)
{
	return v1.x < v2.x && v1.y < v2.y && v1.z < v2.z;
}
inline bool Less(const VEC4 &v1, const VEC4 &v2)
{
	return v1.x < v2.x && v1.y < v2.y && v1.z < v2.z && v1.w < v2.w;
}

inline bool LessOrEqual(const VEC2 &v1, const VEC2 &v2)
{
	return v1.x <= v2.x && v1.y <= v2.y;
}
inline bool LessOrEqual(const VEC3 &v1, const VEC3 &v2)
{
	return v1.x <= v2.x && v1.y <= v2.y && v1.z <= v2.z;
}
inline bool LessOrEqual(const VEC4 &v1, const VEC4 &v2)
{
	return v1.x <= v2.x && v1.y <= v2.y && v1.z <= v2.z && v1.w <= v2.w;
}

inline bool Greater(const VEC2 &v1, const VEC2 &v2)
{
	return v1.x > v2.x && v1.y > v2.y;
}
inline bool Greater(const VEC3 &v1, const VEC3 &v2)
{
	return v1.x > v2.x && v1.y > v2.y && v1.z > v2.z;
}
inline bool Greater(const VEC4 &v1, const VEC4 &v2)
{
	return v1.x > v2.x && v1.y > v2.y && v1.z > v2.z && v1.w > v2.w;
}

inline bool GreaterOrEqual(const VEC2 &v1, const VEC2 &v2)
{
	return v1.x >= v2.x && v1.y >= v2.y;
}
inline bool GreaterOrEqual(const VEC3 &v1, const VEC3 &v2)
{
	return v1.x >= v2.x && v1.y >= v2.y && v1.z >= v2.z;
}
inline bool GreaterOrEqual(const VEC4 &v1, const VEC4 &v2)
{
	return v1.x >= v2.x && v1.y >= v2.y && v1.z >= v2.z && v1.w >= v2.w;
}

//@}
// math_vec_comp

inline bool around(const VEC2 &v1, const VEC2 &v2, float Epsilon) {
	return around(v1.x, v2.x, Epsilon) && around(v1.y, v2.y, Epsilon);
}
inline bool around(const VEC3 &v1, const VEC3 &v2, float Epsilon) {
	return around(v1.x, v2.x, Epsilon) && around(v1.y, v2.y, Epsilon) && around(v1.z, v2.z, Epsilon);
}
inline bool around(const VEC4 &v1, const VEC4 &v2, float Epsilon) {
	return around(v1.x, v2.x, Epsilon) && around(v1.y, v2.y, Epsilon) && around(v1.z, v2.z, Epsilon) && around(v1.w, v2.w, Epsilon);
}

/// Zwraca wektor prostopady do podanego wektora 2D
/** W ukadzie w ktrym X jest w prawo a Y do gry wektor ley na lewo od danego.
W ukadzie w ktrym X jest w prawo a Y w d, ley na prawo. */
inline void Perpedicular(VEC2 *Out, const VEC2 &v)
{
	Out->x = -v.y;
	Out->y = v.x;
}

/// Oblicza wektor kierunku odbicia od paszczyzny o podanej normalnej
/**
\param v wskazuje na kierunek *do* tej paszczyzny.
\param PlaneNormal musi by znormalizowany.
*/
inline void Reflect(VEC2 *Out, const VEC2 &v, const VEC2 &PlaneNormal)
{
	// Wzr: *Out = v - ( 2.0f * Dot(v, PlaneNormal) * PlaneNormal );
	float f = 2.0f * Dot(v, PlaneNormal);
	Out->x = v.x - f * PlaneNormal.x;
	Out->y = v.y - f * PlaneNormal.y;
}
inline void Reflect(VEC3 *Out, const VEC3 &v, const VEC3 &PlaneNormal)
{
	// Wzr: *Out = v - ( 2.0f * Dot(v, PlaneNormal) * PlaneNormal );
	float f = 2.0f * Dot(v, PlaneNormal);
	Out->x = v.x - f * PlaneNormal.x;
	Out->y = v.y - f * PlaneNormal.y;
	Out->z = v.z - f * PlaneNormal.z;
}

/// Rzutowanie wektora V na wektor N
/** Jeli chcemy rozoy wektor V na wektor prostopady i rwnolegy do N,
to wektorem rwnolegym jest Out, a prostopady obliczamy jako V - Out.
\param[out] Out Bdzie rwnolegy do N. */
inline void Project(VEC2 *Out, const VEC2 &V, const VEC2 &N)
{
	// Oryginalnie:
	// *Out = N * ( Dot(V, N) / LengthSq(N) );
	// Wersja szybsza:
	float T = (V.x*N.x + V.y*N.y) / (N.x*N.x + N.y*N.y);
	Out->x = N.x * T;
	Out->y = N.y * T;
}
inline void Project(VEC3 *Out, const VEC3 &V, const VEC3 &N)
{
	float T = (V.x*N.x + V.y*N.y + V.z*N.z) / (N.x*N.x + N.y*N.y + N.z*N.z);
	Out->x = N.x * T;
	Out->y = N.y * T;
	Out->z = N.z * T;
}
/// Wersja common::Project, kiedy wektor N jest ju na pewno znormalizowany.
inline void Project_N(VEC2 *Out, const VEC2 &V, const VEC2 &N)
{
	// Oryginalnie:
	// *Out = N * Dot(V, N);
	// Wersja szybsza:
	float T = V.x*N.x + V.y*N.y;
	Out->x = N.x * T;
	Out->y = N.y * T;
}
inline void Project_N(VEC3 *Out, const VEC3 &V, const VEC3 &N)
{
	float T = V.x*N.x + V.y*N.y + V.z*N.z;
	Out->x = N.x * T;
	Out->y = N.y * T;
	Out->z = N.z * T;
}

/// Oblicza: u cross v dot w
inline float ScalarTripleProduct(const VEC3 &u, const VEC3 &v, const VEC3 &w)
{
	return u.x * (v.y * w.z - v.z * w.y) - u.y * (v.x * w.z - v.z * w.x) + u.z * (v.x * w.y - v.y * w.x);
}

/** \addtogroup math_vec_orthogonalisation Ortogonalizacja trzech wektorw
- Po tym przeksztaceniu bd do siebie prostopade.
- Wektory wejciowe nie musz by znormalizowane.
- Wektory wyjciowe niekoniecznie bd znormalizowane (nawet jeli wejciowe byy).
*/
//@{
/// Ortogonalizacja trzech wektorw - Metoda szybka
void Orthogonalize_Fast(VEC3 *OutR1, VEC3 *OutR2, VEC3 *OutR3, const VEC3 &R1, const VEC3 &R2, const VEC3 &R3);
/// Ortogonalizacja trzech wektorw - Metoda dokadna
void Orthogonalize(VEC3 *OutR1, VEC3 *OutR2, VEC3 *OutR3, const VEC3 &R1, const VEC3 &R2, const VEC3 &R3);
//@}

/// Wyznacza dwa dowolne wektory prostopade do podanego
/** Zwrcone wektory nie s znormalizowane.
\param v nie musi by znormalizowany. */
void PerpedicularVectors(VEC3 *Out1, VEC3 *Out2, const VEC3 &v);

//@}
// math_vec


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_triangle Trjkt */
//@{

/// Oblicza signed pole powierzchni trjkta
/** Wychodzi dodanie, kiedy wierzchoki s CW. */
float TriangleArea(const VEC2 &p1, const VEC2 &p2, const VEC2 &p3);
float TriangleArea(const VEC3 &p1, const VEC3 &p2, const VEC3 &p3);

/// Zwraca true, jeli wierzchoki trjkta s CW.
bool TriangleIsCW(const VEC2 &p1, const VEC2 &p2, const VEC2 &p3);

/// Znajduje obwd trjkta
float TrianglePerimeter(const VEC2 &p1, const VEC2 &p2, const VEC2 &p3);
float TrianglePerimeter(const VEC3 &p1, const VEC3 &p2, const VEC3 &p3);

/// Oblicza punkt na powierzhchni trjkta we wsp. barycentrycznych
/** f to waga punktu p2, g to waga punktu p3. \n
Wzr: *Out = p1 + f*(p2-p1) + g*(p3-p1) \n
Inny wzr: *Out = p1*(1-f-g) + p2*f + p3*g */
inline void Barycentric(VEC2 *Out, const VEC2 &p1, const VEC2 &p2, const VEC2 &p3, float f, float g)
{
	float e = 1.0f-f-g;
	Out->x = e*p1.x + f*p2.x + g*p3.x;
	Out->y = e*p1.y + f*p2.y + g*p3.y;
}
inline void Barycentric(VEC3 *Out, const VEC3 &p1, const VEC3 &p2, const VEC3 &p3, float f, float g)
{
	// Stara wersja, na podstawie DX SDK, duo wolniejsza:
	//VEC3 Diff2 = p2-p1;
	//VEC3 Diff3 = p3-p1;
	//Out->x = p1.x + f*Diff2.x + g*Diff3.x;
	//Out->y = p1.y + f*Diff2.y + g*Diff3.y;
	//Out->z = p1.z + f*Diff2.z + g*Diff3.z;

	float e = 1.0f-f-g;
	Out->x = e*p1.x + f*p2.x + g*p3.x;
	Out->y = e*p1.y + f*p2.y + g*p3.y;
	Out->z = e*p1.z + f*p2.z + g*p3.z;
}
inline void Barycentric(VEC4 *Out, const VEC4 &p1, const VEC4 &p2, const VEC4 &p3, float f, float g)
{
	float e = 1.0f-f-g;
	Out->x = e*p1.x + f*p2.x + g*p3.x;
	Out->y = e*p1.y + f*p2.y + g*p3.y;
	Out->z = e*p1.z + f*p2.z + g*p3.z;
	Out->w = e*p1.w + f*p2.w + g*p3.w;
}

/// Wsprzdne barycentryczne punktu p wzgldem trjkta p1-p2-p3.
/** Wspczynniki e, f, g to wagi wierzchokw p1, p2, p3. \n
\param out_e [out] Mona poda NULL. */
void CalcBarycentric(const VEC2 &p, const VEC2 &p1, const VEC2 &p2, const VEC2 &p3, float *out_e, float *out_f, float *out_g);
/// Wsprzdne barycentryczne punktu p wzgldem trjkta p1-p2-p3.
/** Wspczynniki e, f, g to wagi wierzchokw p1, p2, p3. \n
Punkt p musi lee na jednej paszczynie z trjktem p1-p2-p3. \n
Normal to wektor normalny trjkta p1-p2-p3. Mona go poda jeli jest ju wyliczony albo poda NULL. */
void CalcBarycentric(const VEC3 &p, const VEC3 &p1, const VEC3 &p2, const VEC3 &p3, const VEC3 *Normal, float *out_e, float *out_f, float *out_g);

/// Znajduje okrg wpisany w trjkt
void TriangleInscribedCircle(VEC2 *OutCenter, float *OutRadius, const VEC2 &v1, const VEC2 &v2, const VEC2 &v3);
void TriangleInscribedCircle(VEC3 *OutCenter, float *OutRadius, const VEC3 &v1, const VEC3 &v2, const VEC3 &v3);
/// Znajduje okrg opisany na trjkcie
void TriangleCircumscribedCircle(VEC2 *OutCenter, float *OutRadius, const VEC2 &v1, const VEC2 &v2, const VEC2 &v3);
void TriangleCircumscribedCircle(VEC3 *OutCenter, float *OutRadius, const VEC3 &v1, const VEC3 &v2, const VEC3 &v3);

//@}
// math_triangle


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_ray Promie */
//@{

/// Zwraca punkt na promieniu
inline void PointOnRay(VEC2 *Out, const VEC2 &Origin, const VEC2 &Direction, float t)
{
	Out->x = Origin.x + t * Direction.x;
	Out->y = Origin.y + t * Direction.y;
}
inline void PointOnRay(VEC3 *Out, const VEC3 &Origin, const VEC3 &Direction, float t)
{
	Out->x = Origin.x + t * Direction.x;
	Out->y = Origin.y + t * Direction.y;
	Out->z = Origin.z + t * Direction.z;
}

//@}
// math_ray


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_rect Prostokt */
//@{

/// Prostokt z liczb cakowitych
/**
RECTI jest kompatybilny ze struktur RECT z Win32API.
Konwersja do i ze stringa: "left,top,right,bottom", np. "1,2,3,4".
Niezalenie od ukadu wsprzdnych, prawidowy prostokt jest wtedy, kiedy top
jest mniejsze co do wartoci od bottom, a left od right.
*/
struct RECTI
{
	int left, top, right, bottom;

	RECTI() { }
	RECTI(int left, int top, int right, int bottom) : left(left), top(top), right(right), bottom(bottom) { }
	RECTI(const int *Arr) : left(Arr[0]), top(Arr[1]), right(Arr[2]), bottom(Arr[3]) { }

	operator int * () { return &left; }
	operator const int * () const { return &left; }

	bool operator == (const RECTI &r) const { return left == r.left && top == r.top && right == r.right && bottom == r.bottom; }
	bool operator != (const RECTI &r) const { return left != r.left || top != r.top || right != r.right || bottom != r.bottom; }

	RECTI operator + () const { return *this; }
	RECTI operator - () const { return RECTI(-right, -bottom, -left, -top); }
	RECTI operator + (const POINT_ &p) const { return RECTI(left+p.x, top+p.y, right+p.x, bottom+p.y); }
	RECTI operator - (const POINT_ &p) const { return RECTI(left-p.x, top-p.y, right-p.x, bottom-p.y); }
	RECTI operator * (int s) const { return RECTI(left*s, top*s, right*s, bottom*s); }
	RECTI operator * (const POINT_ &s) const { return RECTI(left*s.x, top*s.y, right*s.x, bottom*s.y); }
	RECTI & operator += (const POINT_ &p) { left += p.x; top += p.y; right += p.x; bottom += p.y; return *this; }
	RECTI & operator -= (const POINT_ &p) { left -= p.x; top -= p.y; right -= p.x; bottom -= p.y; return *this; }
	RECTI & operator *= (int s) { left *= s; top *= s; right *= s; bottom *= s; return *this; }
	RECTI & operator *= (const POINT_ &s) { left *= s.x; top *= s.y; right *= s.x; bottom *= s.y; return *this; }

	bool IsValid() const { return left < right && top < bottom; }
	void Repair() { if (right < left) std::swap(left, right); if (bottom < top) std::swap(top, bottom); }

	void LeftTop(POINT_ *Out) const { Out->x = left; Out->y = top; }
	void RightBottom(POINT_ *Out) const { Out->x = right; Out->y = bottom; }

	/// Powiksza prostokt z kadej strony o podan warto
	void Extend(int d) { left -= d; top -= d; right += d; bottom += d; }
	/// Oblicza i zwraca szeroko
	int Width() const { return right - left; }
	/// Oblicza i zwraca wysoko
	int Height() const { return bottom - top; }
	/// Oblicza i zwraca dugo przektnej
	float Diagonal() const { return sqrtf( (float)((right-left)*(right-left)) + (float)((bottom-top)*(bottom-top)) ); }
	/// Oblicza i zwraca pole powierzchni
	int Field() const { return (right-left)*(bottom-top); }
	/// Zmienia szeroko na podan
	void SetWidth(int NewWidth) { right = left + NewWidth; }
	/// Zmienia wysoko na podan
	void SetHeight(int NewHeight) { bottom = top + NewHeight; }
	/// Zmienia szeroko i wysoko na podan
	void SetSize(int NewWidth, int NewHeight) { right = left + NewWidth; bottom = top + NewHeight; }
	// Oblicza i zwraca pozycj rodka
	void GetCenter(POINT_ *Out) const { Out->x = (left+right)/2; Out->y = (top+bottom)/2; }
	int GetCenterX() const { return (left+right)/2; }
	int GetCenterY() const { return (top+bottom)/2; }
	/// Zwraca true, jeli prostokt jest zerowy
	bool Empty() const { return left == 0 && top == 0 && right == 0 && bottom == 0; }
	/// Czyci prostokt na zerowy
	void Clear() { left = top = right = bottom = 0; }
};

extern const RECTI RECTI_ZERO;

/// Prostokt z 4 floatw.
struct RECTF
{
	float left, top, right, bottom;

	operator float * () { return &left; }
	operator const float * () const { return &left; }

	RECTF() { }
	RECTF(float left, float top, float right, float bottom) : left(left), top(top), right(right), bottom(bottom) { }
	RECTF(const float *Arr) : left(Arr[0]), top(Arr[1]), right(Arr[2]), bottom(Arr[3]) { }

	bool operator == (const RECTF &r) const { return left == r.left && top == r.top && right == r.right && bottom == r.bottom; }
	bool operator != (const RECTF &r) const { return left != r.left || top != r.top || right != r.right || bottom != r.bottom; }

	RECTF operator + () const { return *this; }
	RECTF operator - () const { return RECTF(-right, -bottom, -left, -top); }
	RECTF operator + (const VEC2 &p) const { return RECTF(left+p.x, top+p.y, right+p.x, bottom+p.y); }
	RECTF operator - (const VEC2 &p) const { return RECTF(left-p.x, top-p.y, right-p.x, bottom-p.y); }
	RECTF operator * (float s) const { return RECTF(left*s, top*s, right*s, bottom*s); }
	RECTF operator * (const VEC2 &s) const { return RECTF(left*s.x, top*s.y, right*s.x, bottom*s.y); }
	RECTF & operator += (const VEC2 &p) { left += p.x; top += p.y; right += p.x; bottom += p.y; return *this; }
	RECTF & operator -= (const VEC2 &p) { left -= p.x; top -= p.y; right -= p.x; bottom -= p.y; return *this; }
	RECTF & operator *= (float s) { left *= s; top *= s; right *= s; bottom *= s; return *this; }
	RECTF & operator *= (const VEC2 &s) { left *= s.x; top *= s.y; right *= s.x; bottom *= s.y; return *this; }

	bool IsValid() const { return left < right && top < bottom; }
	void Repair() { if (right < left) std::swap(left, right); if (bottom < top) std::swap(top, bottom); }

	void LeftTop(VEC2 *Out) const { Out->x = left; Out->y = top; }
	void RightBottom(VEC2 *Out) const { Out->x = right; Out->y = bottom; }

	/// Powiksza prostokt z kadej strony o podan warto
	void Extend(float d) { left -= d; top -= d; right += d; bottom += d; }
	/// Oblicza i zwraca szeroko
	float Width() const { return right - left; }
	/// Oblicza i zwraca wysoko
	float Height() const { return bottom - top; }
	/// Oblicza i zwraca dugo przektnej
	float Diagonal() const { return sqrtf( (right-left)*(right-left) + (bottom-top)*(bottom-top) ); }
	/// Oblicza i zwraca pole powierzchni
	float Field() const { return (right-left)*(bottom-top); }
	/// Zmienia szeroko na podan
	void SetWidth(float NewWidth) { right = left + NewWidth; }
	/// Zmienia wysoko na podan
	void SetHeight(float NewHeight) { bottom = top + NewHeight; }
	/// Zmienia szeroko i wysoko na podan
	void SetSize(float NewWidth, float NewHeight) { right = left + NewWidth; bottom = top + NewHeight; }
	/// Oblicza i zwraca pozycj rodka
	void GetCenter(VEC2 *Out) const { Out->x = (left+right)*0.5f; Out->y = (top+bottom)*0.5f; }
	float GetCenterX() const { return (left+right)*0.5f; }
	float GetCenterY() const { return (top+bottom)*0.5f; }
	/// Zwraca true, jeli prostokt jest zerowy
	bool Empty() const { return left == 0.0f && top == 0.0f && right == 0.0f && bottom == 0.0f; }
	/// Czyci prostokt na zerowy
	void Clear() { left = top = right = bottom = 0.0f; }
};

extern const RECTF RECTF_ZERO;

#ifdef WIN32
	/// Konwersja z common::RECTI do RECT z WinAPI
	template <> inline ::RECT & math_cast<::RECT, RECTI>(const RECTI &x) { return (::RECT&)(x); }
	/// Konwersja z RECT z WinAPI do common::RECTI
	template <> inline RECTI & math_cast<RECTI, ::RECT>(const ::RECT &x) { return (RECTI&)(x); }
#endif

inline void swap(RECTI &r1, RECTI &r2)
{
	std::swap(r1.left, r2.left); std::swap(r1.top, r2.top);
	std::swap(r1.right, r2.right); std::swap(r1.bottom, r2.bottom);
}

inline void swap(RECTF &r1, RECTF &r2)
{
	std::swap(r1.left, r2.left); std::swap(r1.top, r2.top);
	std::swap(r1.right, r2.right); std::swap(r1.bottom, r2.bottom);
}

/// Zwraca true, jeli punkt naley do prostokta
inline bool PointInRect(const POINT_ &p, const RECTI &r)
{
	return
		(p.x >= r.left) &&
		(p.x <= r.right) &&
		(p.y >= r.top) &&
		(p.y <= r.bottom);
}
inline bool PointInRect(const VEC2 &p, const RECTF &r)
{
	return
		(p.x >= r.left) &&
		(p.x <= r.right) &&
		(p.y >= r.top) &&
		(p.y <= r.bottom);
}

/// Zwraca true, jeli prostokt sr zawiera si w caoci wewntrz r
inline bool RectInRect(const RECTI &sr, const RECTI &r)
{
	return
		(sr.left >= r.left) &&
		(sr.right <= r.right) &&
		(sr.top >= r.top) &&
		(sr.bottom <= r.bottom);
}
inline bool RectInRect(const RECTF &sr, const RECTF &r)
{
	return
		(sr.left >= r.left) &&
		(sr.right <= r.right) &&
		(sr.top >= r.top) &&
		(sr.bottom <= r.bottom);
}

/// Zwraca true, jeli podane prostokty zachodz na siebie
inline bool RectToRect(const RECTI &r1, const RECTI &r2)
{
	return
		(r1.left <= r2.right) &&
		(r1.right >= r2.left) &&
		(r1.top <= r2.bottom) &&
		(r1.bottom >= r2.top);
}
inline bool RectToRect(const RECTF &r1, const RECTF &r2)
{
	return
		(r1.left <= r2.right) &&
		(r1.right >= r2.left) &&
		(r1.top <= r2.bottom) &&
		(r1.bottom >= r2.top);
}

/// Jeli prostokty posiadaj cz wspln, zwraca j przez Out i zwraca true
inline bool Intersection(RECTI *Out, const RECTI &r1, const RECTI &r2)
{
	*Out = RECTI(
		std::max(r1.left, r2.left),
		std::max(r1.top, r2.top),
		std::min(r1.right, r2.right),
		std::min(r1.bottom, r2.bottom));
	return Out->IsValid();
}
inline bool Intersection(RECTF *Out, const RECTF &r1, const RECTF &r2)
{
	*Out = RECTF(
		std::max(r1.left, r2.left),
		std::max(r1.top, r2.top),
		std::min(r1.right, r2.right),
		std::min(r1.bottom, r2.bottom));
	return Out->IsValid();
}

/// Zwraca prostokt opisany na podanych dwch prostoktach
inline void Union(RECTI *Out, const RECTI &r1, const RECTI &r2)
{
	Out->left = std::min(r1.left, r2.left);
	Out->top = std::min(r1.top, r2.top);
	Out->right = std::max(r1.right, r2.right);
	Out->bottom = std::max(r1.bottom, r2.bottom);
}
inline void Union(RECTF *Out, const RECTF &r1, const RECTF &r2)
{
	Out->left = std::min(r1.left, r2.left);
	Out->top = std::min(r1.top, r2.top);
	Out->right = std::max(r1.right, r2.right);
	Out->bottom = std::max(r1.bottom, r2.bottom);
}

/// Wyznacza najbliszy punkt wewntrz prostokta wzgldem punktu p.
/** Jeli p ley wewntrz tego prostokta, zwrcone zostaje dokadnie p. */
void ClosestPointInRect(POINT_ *Out, const RECTI &Rect, const POINT_ &p);
/// Wyznacza najbliszy punkt wewntrz prostokta wzgldem punktu p.
/** Jeli p ley wewntrz tego prostokta, zwrcone zostaje dokadnie p. */
void ClosestPointInRect(VEC2 *Out, const RECTF &Rect, const VEC2 &p);

//@}
// math_rect


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_aabb AABB
Prostopadocian AABB - Axis Aligned Bounding Box */
//@{

/// AABB z 2 punktw 3D
/**
Skadnia do konwersji na i z acuch: p1x,p1y,p1z;p2x,p2y,p2z, np.:
"1,2,3;4,5,6"
*/
struct BOX
{
	VEC3 p1, p2;

	BOX() { }
	BOX(const VEC3 &p1, const VEC3 &p2) : p1(p1), p2(p2) { }
	BOX(float p1x, float p1y, float p1z, float p2x, float p2y, float p2z) : p1(p1x, p1y, p1z), p2(p2x, p2y, p2z) { }
	BOX(const float *Arr) : p1(&Arr[0]), p2(&Arr[3]) { }

	operator float * () { return &p1.x; }
	operator const float * () const { return &p1.x; }

	bool operator == (const BOX &p) const { return p1 == p.p1 && p2 == p.p2; }
	bool operator != (const BOX &p) const { return p1 != p.p1 || p2 != p.p2; }

	BOX operator + () const { return *this; }
	BOX operator - () const { return BOX(-p2, -p1); }
	BOX operator + (const VEC3 &v) const { return BOX(p1+v, p2+v); }
	BOX operator - (const VEC3 &v) const { return BOX(p1-v, p2-v); }
	BOX operator * (float f) const { return BOX(p1*f, p2*f); }
	BOX operator * (const VEC3 &s) const { return BOX(VEC3(p1.x*s.x, p1.y*s.y, p1.z*s.z), VEC3(p2.x*s.x, p2.y*s.y, p2.z*s.z)); }
	BOX & operator += (const VEC3 &v) { p1 += v; p2 += v; return *this; }
	BOX & operator -= (const VEC3 &v) { p1 -= v; p2 -= v; return *this; }
	BOX & operator *= (float f) { p1 *= f; p2 *= f; return *this; }
	BOX & operator *= (const VEC3 &s) { p1.x *= s.x; p1.y *= s.y; p1.z *= s.z; p2.x *= s.x; p2.y *= s.y; p2.z *= s.z; return *this; }

	bool IsValid() { return Less(p1, p2); }
	void Repair() { if (p2.x < p1.x) std::swap(p1.x, p2.x); if (p2.y < p1.y) std::swap(p1.y, p2.y); if (p2.z < p1.z) std::swap(p1.z, p2.z); }

	/// Powiksza boks z kadej strony o podan warto
	void Extend(float d) { p1.x -= d; p1.y -= d; p1.z -= d; p2.x += d; p2.y += d; p2.z += d; }
	/// Oblicza i zwraca rozmiary boksa w trzech wymiarach
	void GetSize(VEC3 *Out) const { Out->x = p2.x-p1.x; Out->y = p2.y-p1.y; Out->z = p2.z-p1.z; }
	/// Oblicza szeroko
	float GetCX() const { return p2.x - p1.x; }
	/// Oblicza wysoko
	float GetCY() const { return p2.y - p1.y; }
	/// Oblicza gboko
	float GetCZ() const { return p2.z - p1.z; }
	/// Zmienia wielko na podan
	void SetSize(const VEC3 &NewSize) { p2.x = p1.x + NewSize.x; p2.y = p1.y + NewSize.y; p2.z = p1.z + NewSize.z; }
	/// Oblicza i zwraca dugo przektnej
	float Diagonal() const { return sqrtf( (p2.x-p1.x)*(p2.x-p1.x) + (p2.y-p1.y)*(p2.y-p1.y) + (p2.z-p1.z)*(p2.z-p1.z) ); }
	/// Oblicza i zwraca objto
	float Volume() const { return (p2.x-p1.x)*(p2.y-p1.y)*(p2.z-p1.z); }
	/// Oblicza i zwraca pozycj rodka
	void CalcCenter(VEC3 *Out) const { Out->x = (p1.x+p2.x)*0.5f; Out->y = (p1.y+p2.y)*0.5f; Out->z = (p1.z+p2.z)*0.5f; }
	/// Oblicza i zwraca poowy rozmiarw
	void CalcHalfSize(VEC3 *OutHalfSize) const {
		OutHalfSize->x = (p2.x-p1.x)*0.5f;
		OutHalfSize->y = (p2.y-p1.y)*0.5f;
		OutHalfSize->z = (p2.z-p1.z)*0.5f;
	}
	/// Zwraca true, jeli prostokt jest zerowy
	bool Empty() const { return *this == BOX(0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f); }
	/// Czyci prostokt na zerowy
	void Clear() { *this = BOX(0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f); }
	/// Zwraca wybrany wierzchoek
	void GetCorner(VEC3 *Out, int i) const
	{
		Out->x = (i & 1) ? p2.x : p1.x;
		Out->y = (i & 2) ? p2.y : p1.y;
		Out->z = (i & 4) ? p2.z : p1.z;
	}
	/// Zwraca wszystkie 6 wierzchokw w kolejnoci
	/**
	Trzeba poda wskanik do tablicy. Kolejno:
	\verbatim
	p1x, p1y, p1z; p2x, p1y, p1z; p1x, p2y, p1z; p2x, p2y, p1z
	p1x, p1y, p2z; p2x, p1y, p2z; p1x, p2y, p2z; p2x, p2y, p2z
	\endverbatim
	*/
	void GetAllCorners(VEC3 *Out) const
	{
		Out[0] = VEC3(p1.x, p1.y, p1.z); Out[1] = VEC3(p2.x, p1.y, p1.z); Out[2] = VEC3(p1.x, p2.y, p1.z); Out[3] = VEC3(p2.x, p2.y, p1.z);
		Out[4] = VEC3(p1.x, p1.y, p2.z); Out[5] = VEC3(p2.x, p1.y, p2.z); Out[6] = VEC3(p1.x, p2.y, p2.z); Out[7] = VEC3(p2.x, p2.y, p2.z);
	}
	/// Powiksza tak eby na pewno zawiera podany punkt
	void AddInternalPoint(const VEC3 &p)
	{
		p1.x = std::min(p1.x, p.x); p1.y = std::min(p1.y, p.y); p1.z = std::min(p1.z, p.z);
		p2.x = std::max(p2.x, p.x); p2.y = std::max(p2.y, p.y); p2.z = std::max(p2.z, p.z);
	}
	/// Powiksza tak eby na pewno zawiera podany boks
	/** Podany boks musi by prawidowy. */
	void AddInternalBox(const BOX &b)
	{
		p1.x = std::min(p1.x, b.p1.x); p1.y = std::min(p1.y, b.p1.y); p1.z = std::min(p1.z, b.p1.z);
		p2.x = std::max(p2.x, b.p2.x); p2.y = std::max(p2.y, b.p2.y); p2.z = std::max(p2.z, b.p2.z);
	}
};

static const BOX ZERO;

inline void swap(BOX &b1, BOX &b2)
{
	std::swap(b1.p1, b2.p1);
	std::swap(b1.p2, b2.p2);
}

/// Skaluje boks o podan liczb
inline void Scale(BOX *Out, const BOX &b, float s)
{
	Out->p1.x = b.p1.x * s; Out->p1.y = b.p1.y * s; Out->p1.z = b.p1.z * s;
	Out->p2.x = b.p2.x * s; Out->p2.y = b.p2.y * s; Out->p2.z = b.p2.z * s;
}

/// Skaluje boks o podane 3 wymiary
inline void Scale(BOX *Out, const BOX &b, const VEC3 &s)
{
	Out->p1.x = b.p1.x * s.x; Out->p1.y = b.p1.y * s.y; Out->p1.z = b.p1.z * s.z;
	Out->p2.x = b.p2.x * s.x; Out->p2.y = b.p2.y * s.y; Out->p2.z = b.p2.z * s.z;
}

/// Zwraca true, jeli punkt naley do boksa
inline bool PointInBox(const VEC3 &p, const BOX &b)
{
	return
		(p.x >= b.p1.x) && (p.x <= b.p2.x) &&
		(p.y >= b.p1.y) && (p.y <= b.p2.y) &&
		(p.z >= b.p1.z) && (p.z <= b.p2.z);
}

/// Zwraca true, jeli boks sb zawiera si w caoci wewntrz b
inline bool BoxInBox(const BOX &sb, const BOX &b)
{
	return
		(sb.p1.x >= b.p1.x) && (sb.p2.x <= b.p2.x) &&
		(sb.p1.y >= b.p1.y) && (sb.p2.y <= b.p2.y) &&
		(sb.p1.z >= b.p1.z) && (sb.p2.z <= b.p2.z);
}

/// Zwraca true, jeli podane boksy zachodz na siebie
inline bool BoxToBox(const BOX &b1, const BOX &b2)
{
	return
		(b1.p1.x <= b2.p2.x) && (b1.p2.x >= b2.p1.x) &&
		(b1.p1.y <= b2.p2.y) && (b1.p2.y >= b2.p1.y) &&
		(b1.p1.z <= b2.p2.z) && (b1.p2.z >= b2.p1.z);
}

/// Jeli boksy posiadaj cz wspln, zwraca j przez Out i zwraca true
inline bool Intersection(BOX *Out, const BOX &b1, const BOX &b2)
{
	*Out = BOX(
		VEC3(
			std::max(b1.p1.x, b2.p1.x),
			std::max(b1.p1.y, b2.p1.y),
			std::max(b1.p1.z, b2.p1.z)),
		VEC3(
			std::min(b1.p2.x, b2.p2.x),
			std::min(b1.p2.y, b2.p2.y),
			std::min(b1.p2.z, b2.p2.z)));
	return Out->IsValid();
}

/// Zwraca boks opisany na podanych dwch boksach
inline void Union(BOX *Out, const BOX &b1, const BOX &b2)
{
	*Out = BOX(
		VEC3(
			std::min(b1.p1.x, b2.p1.x),
			std::min(b1.p1.y, b2.p1.y),
			std::min(b1.p1.z, b2.p1.z)),
		VEC3(
			std::max(b1.p2.x, b2.p2.x),
			std::max(b1.p2.y, b2.p2.y),
			std::max(b1.p2.z, b2.p2.z)));
}

//@}
// math_aabb


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_aabb OBB
Prostopadocian o dowolnej orientacji OBB - Oriented Bounding Box */
//@{

/// OBB - Oriented Bounding Box - prostopadocian o dowolnej orientacji
struct OBB
{
	/// Pozycja rodka
	VEC3 Center;
	/// Poowa szerokoci, wysokoci, gbokoci
	VEC3 HalfSize;
	/// Osie wskazujce lokalne kierunki X, Y, Z.
	/** Musz by jednostkowe i wzajemnie prostopade. */
	VEC3 Axes[3];
};

/// Zwraca true, jeli dwa prostopadociany OBB nachodz na siebie.
bool ObbToObb(const OBB &a, const OBB &b);

//@}
// math_obb


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_rect3d Rect 3D
Dowolnie zorientowany prostokt w przestrzeni 3D - tak jakby paski OBB. */
//@{

/// OBB - Oriented Bounding Box - prostopadocian o dowolnej orientacji
struct RECT3D
{
	/// Pozycja rodka
	VEC3 Center;
	/// Poowa szerokoci, wysokoci
	VEC2 HalfSize;
	/// Osie wskazujce lokalne kierunki X, Y.
	/** Musz by jednostkowe i wzajemnie prostopade. */
	VEC3 Axes[2];
};

//@}
// math_rect3d


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_colors Kolory
Stae z kolorami, operacje generujce kolory itp. wypeniaj kana alfa jedynk.
*/
//@{

/// Kolor w formacie A8R8G8B8.
/**
Operacje dodawania i odejmowania kolorw, a take dodawania, odejmowania,
mnoenia i dzielenia przez liczb pilnuj zakresu i w razie czego obcinaj,
nie zawijaj.
*/
struct COLOR
{
	union
	{
		/// Cay kolor razem
		uint4 ARGB;
		/// Osobno poszczeglne skadowe
		PACKED struct
		{
			uint1 B, G, R, A;
		};
	};

	COLOR() { }
	COLOR(uint4 c) : ARGB(c) { }
	COLOR(unsigned long c) : ARGB((uint4)c) { }
	COLOR(uint1 R, uint1 G, uint1 B) : B(B), G(G), R(R), A(0xFF) { }
	COLOR(uint1 A, uint1 R, uint1 G, uint1 B) : B(B), G(G), R(R), A(A) { }

	bool operator == (COLOR c) const { return ARGB == c.ARGB; }
	bool operator != (COLOR c) const { return ARGB != c.ARGB; }

	// Operatory unarne - dany kolor i negatyw koloru
	COLOR operator + () const { return *this; }
	COLOR operator - () const { return COLOR(~ARGB); }
	// Dodawanie i odejmowanie kolorw
	COLOR operator + (const COLOR &c) const { return COLOR(safe_add(A, c.A), safe_add(R, c.R), safe_add(G, c.G), safe_add(B, c.B)); }
	COLOR operator - (const COLOR &c) const { return COLOR(safe_sub(A, c.A), safe_sub(R, c.R), safe_sub(G, c.G), safe_sub(B, c.B)); }
	COLOR & operator += (const COLOR &c) { A = safe_add(A, c.A); R = safe_add(R, c.R); G = safe_add(G, c.G); B = safe_add(B, c.B); return *this; }
	COLOR & operator -= (const COLOR &c) { A = safe_sub(A, c.A); R = safe_sub(R, c.R); G = safe_sub(G, c.G); B = safe_sub(B, c.B); return *this; }
	// Mnoenie i dzielenie koloru przez liczb
	// Autor: Tarlandil
	COLOR operator * (float s) const { return COLOR( (uint1)(minmax(0, (int)(A*s), 255)), (uint1)(minmax(0, (int)(R*s), 255)), (uint1)(minmax(0, (int)(G*s), 255)), (uint1)(minmax(0, (int)(B*s), 255)) ); }
	COLOR operator / (float s) const { float sInv = 1.0f/s; return COLOR( (uint1)(minmax(0, (int)(A*sInv), 255)), (uint1)(minmax(0, (int)(R*sInv), 255)), (uint1)(minmax(0, (int)(G*sInv), 255)), (uint1)(minmax(0, (int)(B*sInv), 255)) ); }
	COLOR & operator *= (float s) { A = (uint1)(minmax(0, (int)(A*s), 255)); R = (uint1)(minmax(0, (int)(R*s), 255)); G = (uint1)(minmax(0, (int)(G*s), 255)); B = (uint1)(minmax(0, (int)(B*s), 255)); return *this; }
	COLOR & operator /= (float s) { float sInv = 1.0f/s; A = (uint1)(minmax(0, (int)(A*sInv), 255)); R = (uint1)(minmax(0, (int)(R*sInv), 255)); G = (uint1)(minmax(0, (int)(G*sInv), 255)); B = (uint1)(minmax(0, (int)(B*sInv), 255));  return *this; }
	// Mnoenie dwch kolorw, czyli modulacja
	COLOR operator * (const COLOR &c) const { return COLOR( (uint1)(((int)A*(int)c.A)/255), (uint1)(((int)R*(int)c.R)/255), (uint1)(((int)G*(int)c.G)/255), (uint1)(((int)B*(int)c.B)/255) ); }
	COLOR & operator *= (const COLOR &c) { A = (uint1)(((int)A*(int)c.A)/255); R = (uint1)(((int)R*(int)c.R)/255); G = (uint1)(((int)G*(int)c.G)/255); B = (uint1)(((int)B*(int)c.B)/255); return *this; }
};

extern const COLOR COLOR_TRANSPARENT_;
extern const COLOR COLOR_BLACK;
extern const COLOR COLOR_GRAY;
extern const COLOR COLOR_SILVER;
extern const COLOR COLOR_WHITE;
extern const COLOR COLOR_FUCHSIA;
extern const COLOR COLOR_RED;
extern const COLOR COLOR_ORANGE;
extern const COLOR COLOR_YELLOW;
extern const COLOR COLOR_LIME;
extern const COLOR COLOR_AQUA;
extern const COLOR COLOR_BLUE;
extern const COLOR COLOR_PURPLE;
extern const COLOR COLOR_MAROON;
extern const COLOR COLOR_OLIVE;
extern const COLOR COLOR_GREEN;
extern const COLOR COLOR_TEAL;

/// Kolor z 4 liczb typu float - R, G, B, A.
/**
Operacje dodawania i odejmowania kolorw, a take dodawania, odejmowania,
mnoenia i dzielenia przez liczb nie pilnuj zakresu i wartoci skadowych
mog urosn powyej 1.0 albo zmale poniej 0.0.
*/
struct COLORF
{
	float R, G, B, A;

	COLORF() { }
	COLORF(float R, float G, float B) : R(R), G(G), B(B), A(1.0f) { }
	COLORF(float A, float R, float G, float B) : R(R), G(G), B(B), A(A) { }
	COLORF(const float *Arr) : R(Arr[0]), G(Arr[1]), B(Arr[2]), A(Arr[3]) { }

	operator float * () { return &R; }
	operator const float * () const { return &R; }

	bool operator == (const COLORF &c) const { return A == c.A && R == c.R && G == c.G && B == c.B; }
	bool operator != (const COLORF &c) const { return A != c.A || R != c.R || G != c.G || B != c.B; }

	// Operatory unarne - dany kolor i negatyw koloru
	COLORF operator + () const { return *this; }
	COLORF operator - () const { return COLORF(1.0f-A, 1.0f-R, 1.0f-G, 1.0f-B); }
	// Dodawanie i odejmowanie kolorw
	COLORF operator + (const COLORF &c) const { return COLORF(A+c.A, R+c.R, G+c.G, B+c.B); }
	COLORF operator - (const COLORF &c) const { return COLORF(A-c.A, R-c.R, G-c.G, B-c.B); }
	COLORF & operator += (const COLORF &c) { A += c.A; R += c.R; G += c.G; B += c.B; return *this; }
	COLORF & operator -= (const COLORF &c) { A -= c.A; R -= c.R; G -= c.G; B -= c.B; return *this; }
	// Mnoenie i dzielenie koloru przez liczb
	COLORF operator * (float s) const { return COLORF(A*s, R*s, G*s, B*s); }
	COLORF operator / (float s) const { float sInv = 1.0f/s; return COLORF(A*sInv, R*sInv, G*sInv, B*sInv); }
	COLORF & operator *= (float s) { A *= s; R *= s; G *= s; B *= s; return *this; }
	COLORF & operator /= (float s) { float sInv = 1.0f/s; A *= sInv; R *= sInv; G *= sInv; B *= sInv; return *this; }
	// Mnoenie dwch kolorw, czyli modulacja
	COLORF operator * (const COLORF &c) const { return COLORF(A*c.A, R*c.R, G*c.G, B*c.B); }
	COLORF & operator *= (const COLORF &c) { A *= c.A; R *= c.R; G *= c.G; B *= c.B; return *this; }

	/// Sprawdza, czy skadowe mieszcz si w zakresie 0.0 .. 1.0
	bool IsValid() const { return (A >= 0.0f && A <= 1.0f) && (R >= 0.0f && R <= 1.0f) && (G >= 0.0f && G <= 1.0f) && (B >= 0.0f && B <= 1.0f); }
	/// Sprowadza skadowe do zakresu 0.0 .. 1.0
	void Repair()
	{
		A = minmax(0.0f, A, 1.0f); R = minmax(0.0f, R, 1.0f);
		G = minmax(0.0f, G, 1.0f); B = minmax(0.0f, B, 1.0f);
	}
};

inline void swap(COLORF &c1, COLORF &c2)
{
	std::swap(c1.A, c2.A); std::swap(c1.R, c2.R);
	std::swap(c1.G, c2.G); std::swap(c1.B, c2.B);
}

extern const COLORF COLORF_TRANSPARENT_;
extern const COLORF COLORF_BLACK;
extern const COLORF COLORF_GRAY;
extern const COLORF COLORF_SILVER;
extern const COLORF COLORF_WHITE;
extern const COLORF COLORF_FUCHSIA;
extern const COLORF COLORF_RED;
extern const COLORF COLORF_ORANGE;
extern const COLORF COLORF_YELLOW;
extern const COLORF COLORF_LIME;
extern const COLORF COLORF_AQUA;
extern const COLORF COLORF_BLUE;
extern const COLORF COLORF_PURPLE;
extern const COLORF COLORF_MAROON;
extern const COLORF COLORF_OLIVE;
extern const COLORF COLORF_GREEN;
extern const COLORF COLORF_TEAL;

inline COLORF operator * (float s, const COLORF &c) { return COLORF(c.A*s, c.R*s, c.G*s, c.B*s); }
inline COLORF operator / (float s, const COLORF &c) { float sInv = 1.0f/s; return COLORF(c.A*sInv, c.R*sInv, c.G*sInv, c.B*sInv); }

inline COLOR operator * (float s, const COLOR &c) { return COLOR( (uint1)(minmax(0, (int)(c.A*s), 255)), (uint1)(minmax(0, (int)(c.R*s), 255)), (uint1)(minmax(0, (int)(c.G*s), 255)), (uint1)(minmax(0, (int)(c.B*s), 255)) ); }
inline COLOR operator / (float s, const COLOR &c) { float sInv = 1.0f/s; return COLOR( (uint1)(minmax(0, (int)(c.A*sInv), 255)), (uint1)(minmax(0, (int)(c.R*sInv), 255)), (uint1)(minmax(0, (int)(c.G*sInv), 255)), (uint1)(minmax(0, (int)(c.B*sInv), 255)) ); }

template <> inline uint4 & math_cast<uint4, COLOR>(const COLOR &x) { return (uint4&)(x); }
template <> inline unsigned long & math_cast<unsigned long, COLOR>(const COLOR &x) { return (unsigned long&)(x); }
template <> inline COLOR & math_cast<COLOR, uint4>(const uint4 &x) { return (COLOR&)(x); }
template <> inline COLOR & math_cast<COLOR, unsigned long>(const unsigned long &x) { return (COLOR&)(x); }

#ifdef USE_DIRECTX
	template <> inline D3DXCOLOR & math_cast<D3DXCOLOR, COLORF>(const COLORF &x) { return (D3DXCOLOR&)(x); }
	template <> inline COLORF & math_cast<COLORF, D3DXCOLOR>(const D3DXCOLOR &x) { return (COLORF&)(x); }
#endif

/// Konwersje midzy tymi dwoma reprezentacjami kolorw
inline void ColorToColorf(COLORF *Out, COLOR c)
{
	Out->R = c.R / 255.0f;
	Out->G = c.G / 255.0f;
	Out->B = c.B / 255.0f;
	Out->A = c.A / 255.0f;
}
/** Tu musi by valid, inaczej wyjdzie zawinite. */
inline COLOR ColorfToColor(const COLORF &c)
{
	return COLOR(
		(uint1)(minmax(0.0f, c.A, 1.0f)*255.0f+0.5f),
		(uint1)(minmax(0.0f, c.R, 1.0f)*255.0f+0.5f),
		(uint1)(minmax(0.0f, c.G, 1.0f)*255.0f+0.5f),
		(uint1)(minmax(0.0f, c.B, 1.0f)*255.0f+0.5f));
}

/// Konwersja koloru na acuch
/** Formatem moe by:
- 'x' = RRGGBB szesnastkowo
- 'X' = AARRGGBB szesnastkowo
- 'f' = R,G,B floaty
- 'F' = A,R,G,B floaty
*/
void ColorToStr(tstring *Out, COLOR Color, char Format = 'X');
void ColorToStr(tstring *Out, const COLORF &Color, char Format = 'F');
/// Konwersja acucha na kolor
/** Jeli kana alfa nie podany, zostaje wypeniony wartoci maksymaln.
acuchem moe by:
- jedna z 16 nazw standardowych kolorw (wielko liter dowolna)
- warto szesnastkowa RRGGBB
- warto szesnastkowa AARRGGBB
- liczby float R,G,B
- liczby float A,R,G,B
*/
bool StrToColor(COLOR *Out, const tstring &Str);
bool StrToColor(COLORF *Out, const tstring &Str);

inline void Add(COLORF *Out, const COLORF &c1, const COLORF &c2)
{
	Out->A = c1.A + c2.A; Out->R = c1.R + c2.R;
	Out->G = c1.G + c2.G; Out->B = c1.B + c2.B;
}
inline void Sub(COLORF *Out, const COLORF &c1, const COLORF &c2)
{
	Out->A = c1.A - c2.A; Out->R = c1.R - c2.R;
	Out->G = c1.G - c2.G; Out->B = c1.B - c2.B;
}
inline void Mul(COLORF *Out, const COLORF &c1, const COLORF &c2)
{
	Out->A = c1.A * c2.A; Out->R = c1.R * c2.R;
	Out->G = c1.G * c2.G; Out->B = c1.B * c2.B;
}
inline void Mul(COLORF *Out, const COLORF &c, float s)
{
	Out->A = c.A * s; Out->R = c.R * s;
	Out->G = c.G * s; Out->B = c.B * s;
}
inline void Div(COLORF *Out, const COLORF &c, float s)
{
	Out->A = c.A / s; Out->R = c.R / s;
	Out->G = c.G / s; Out->B = c.B / s;
}
inline void Negative(COLORF *Out, const COLORF &c)
{
	Out->A = 1.0f - c.A; Out->R = 1.0f - c.R;
	Out->G = 1.0f - c.G; Out->B = 1.0f - c.B;
}

/// Konwertuje kolor midzy systemami ARGB i ABGR
inline COLOR ARGB_ABGR(COLOR color)
{
	return COLOR(
		(color.ARGB & 0xFF00FF00) |
		((color.ARGB >> 16) & 0x000000FF) |
		((color.ARGB & 0x000000FF) << 16)
		);
}

/// Konwertuje zwyky kolor COLOR (A8R8G8B8) na R5G6B5
inline uint2 ColorToR5G6B5(COLOR color)
{
	return
		((unsigned short)(color.R >> 3) << 11) |
		((unsigned short)(color.G >> 2) <<  5) |
		((unsigned short)(color.B >> 3));
}

/// Konwertuje kolor R5G6B5 na zwyky kolor COLOR (A8R8G8B8)
inline COLOR R5G6B5ToColor(uint2 color)
{
	return COLOR(
		255,
		(uint1)(color >> 11 << 3),
		(uint1)(color >>  5 << 2),
		(uint1)(color <<  3));
}

inline void Lerp(COLORF *Out, const COLORF &c1, const COLORF &c2, float t)
{
	Out->A = Lerp(c1.A, c2.A, t);
	Out->R = Lerp(c1.R, c2.R, t);
	Out->G = Lerp(c1.G, c2.G, t);
	Out->B = Lerp(c1.B, c2.B, t);
}
inline COLOR Lerp(COLOR c1, COLOR c2, float t)
{
	return COLOR(
		(uint1)(minmax(0, (int)(c1.A + t*((float)c2.A-(float)c1.A)), 255)),
		(uint1)(minmax(0, (int)(c1.R + t*((float)c2.R-(float)c1.R)), 255)),
		(uint1)(minmax(0, (int)(c1.G + t*((float)c2.G-(float)c1.G)), 255)),
		(uint1)(minmax(0, (int)(c1.B + t*((float)c2.B-(float)c1.B)), 255)) );
}

/// Konwertuje kolor midzy HSB a RGB
/**
- Hue, Saturation, Brightness = 0..1
- Kana alfa zwraca 1.
*/
void HsbToRgb(COLORF *Out, float hue, float saturation, float brightness);
/// Konwertuje kolor RGB na HSB
void RgbToHsb(float *OutHue, float *OutSaturation, float *OutBrightness, const COLORF &Color);

/// Zwraca kolor tczy od czerwonego (odpowiada mu t = 0) do niebieskiego (t = 1)
/** Kana alfa zwraca 0. */
inline void ColorRainbow(COLORF *Out, float t)
{
	HsbToRgb(Out, t*0.75f, 1.0f, 1.0f);
}

/// Zwraca jasno podanego koloru (odcie szaroci) z zakresu 0..1
inline float ColorToGrayscale(const COLORF &c)
{
	return 0.299f * c.R + 0.587f * c.G + 0.114f * c.B;
}
inline float ColorToGrayscale(COLOR &c)
{
	return 0.299f * (c.R/255.0f) + 0.587f * (c.G/255.0f) + 0.114f * (c.B/255.0f);
}

/// Zwraca kolor szary o podanej jasnoci t = 0..1
inline void GrayscaleToColor(COLORF *Out, float t)
{
	Out->A = 1.0f;
	Out->R = Out->G = Out->B = t;
}
inline void GrayscaleToColor(COLOR *Out, float t)
{
	Out->A = 255;
	Out->R = Out->G = Out->B = (uint1)(t*255.0f);
}

/// Zmienia kolorowi kontrast na miejscu
/** Dla t = 0 wychodzi idealnie powkowy szary. \n
Dla t = 1 wychodzi kolor oryginalny. \n
t moe te by wiksze ni 1. \n
Alfa zostaje bez zmian. */
inline void AdjustContrast(COLORF *Color, float t)
{
	Color->R = 0.5f + t * (Color->R - 0.5f);
	Color->G = 0.5f + t * (Color->G - 0.5f);
	Color->B = 0.5f + t * (Color->B - 0.5f);
}

/// Zmienia kolorowi nasycenie na miejscu
/** Dla t = 0 wychodzi dany kolor skonwertowany do odcienia szaroci. \n
Dla t = 1 wychodzi kolor oryginalny. \n
t < 0 zmniejsza nasycenie, t > 0 zwiksza nasycenie \n
Alfa zostaje bez zmian. */
inline void AdjustSaturation(COLORF *Color, float t)
{
	float gray = ColorToGrayscale(*Color);
	Color->R = gray + t * (Color->R - gray);
	Color->G = gray + t * (Color->G - gray);
	Color->B = gray + t * (Color->B - gray);
}

/// Przetwarza znormalizowany wektor na kolor XRGB. A zwraca 0.
inline COLOR VectorToColor(const VEC3 &v)
{
	int R = minmax(0, (int)(127.0f * v.x + 128.0f), 255);
	int G = minmax(0, (int)(127.0f * v.y + 128.0f), 255);
	int B = minmax(0, (int)(127.0f * v.z + 128.0f), 255);
	return COLOR((uint4)((R << 16) | (G << 8) | B));
}

/// Przetwarza kolor na wektor
inline void ColorToVector(VEC3 *Out, COLOR color)
{
	const float o = 1.0f / 127.0f;
	Out->x = (((color.ARGB >> 16) & 0xFF) - 128.0f) * o;
	Out->y = (((color.ARGB >>  8) & 0xFF) - 128.0f) * o;
	Out->z = (((color.ARGB      ) & 0xFF) - 128.0f) * o;
}

/// Zwraca "odlego" dwch kolorw wg jakiejtam byle jakiej metryki. Uwzgldnia skadowe R, G, B.
inline uint ColorDistance(COLOR Color1, COLOR Color2)
{
	int rd = (int)Color1.R - (int)Color2.R;
	int gd = (int)Color1.G - (int)Color2.G;
	int bd = (int)Color1.B - (int)Color2.B;
	return (uint)(rd*rd + gd*gd + bd*bd);
}

//@}
// math_colors


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_plane Paszczyzna */
//@{

#if defined(USE_DIRECTX) && !defined(DOXYGEN)
	typedef D3DXPLANE PLANE;
#else
	/// Paszczyzna
	/**
	Reprezentuje paszczyzn w przestrzeni 3D z jedn stron dodatni, a drug
	ujemn. Opisana jest rwnaniem A x + B y + C z + D = 0. Wektor [A,B,C] to wektor
	normalny wskazujcy w kierunku dodatniej pprzestrzeni. Jeli paszczyzna jest
	znormalizowana, wektor normalny jest znormalizowany, a D to odlego
	paszczyzny od rodka ukadu wsprzdnych.
	Reprezentacja jako string to A,R,G,B np. "1,2,3,0.2".
	Paszczyzna majca a=b=c=0 (czyli staa ZERO) jest niepoprawna, zdegenerowana.
	*/
	struct PLANE
	{
		float a, b, c, d;

		PLANE() { }
		PLANE(float a, float b, float c, float d) : a(a), b(b), c(c), d(d) { }
		PLANE(const float *Arr) : a(Arr[0]), b(Arr[1]), c(Arr[2]), d(Arr[3]) { }

		operator float * () { return &a; }
		operator const float * () const { return &a; }

		bool operator == (const PLANE &p) const { return a == p.a && b == p.b && c == p.c && d == p.d; }
		bool operator != (const PLANE &p) const { return a != p.a || b != p.b || c != p.c || d != p.d; }

		PLANE operator + () const { return *this; }
		/// Zwraca paszczyzn zwrcon w przeciwn stron
		PLANE operator - () const { return PLANE(-a, -b, -c, -d); }

		PLANE operator * (float s) const { return PLANE(a*s, b*s, c*s, d*s); }
		PLANE operator / (float s) const { float sInv = 1.0f/s; return PLANE(a*sInv, b*sInv, c*sInv, d*sInv); }
		PLANE & operator *= (float s) { a *= s; b *= s; c *= s; d *= s; return *this; }
		PLANE & operator /= (float s) { float sInv = 1.0f/s; a *= sInv; b *= sInv; c *= sInv; d *= sInv; return *this; }
	};
	inline PLANE operator * (float s, const PLANE &p) { return PLANE(p.a*s, p.b*s, p.c*s, p.d*s); }
#endif

extern const PLANE PLANE_ZERO;
extern const PLANE PLANE_POSITIVE_X; ///< Paszczyzna pionowa, rwnolega do Y i Z, wskazujca na prawo na X
extern const PLANE PLANE_POSITIVE_Y; ///< Paszczyzna pozioma, rwnolega do X i Z, wskazujca do gry na Y
extern const PLANE PLANE_POSITIVE_Z; ///< Paszczyzna pionowa, rwnolega do X i Y, wskazujca w gb na Z
extern const PLANE PLANE_NEGATIVE_X; ///< Paszczyzna pionowa, rwnolega do Y i Z, wskazujca na lewo na X
extern const PLANE PLANE_NEGATIVE_Y; ///< Paszczyzna pozioma, rwnolega do X i Z, wskazujca w d na Y
extern const PLANE PLANE_NEGATIVE_Z; ///< Paszczyzna pionowa, rwnolega do X i Y, wskazujca do gracza na Z

inline void swap(PLANE &p1, PLANE &p2)
{
	std::swap(p1.a, p2.a); std::swap(p1.b, p2.b);
	std::swap(p1.c, p2.c); std::swap(p1.d, p2.d);
}

/// Daje dostp do wektora normalnego podanej paszczyzny, czyli jej skadowych a, b, c zrzutowanych jako VEC3.
inline VEC3 & GetPlaneNormal(PLANE &Plane) { return (VEC3&)Plane.a; }
/// Zwraca wektor normalny paszczyzny, czyli jej skadowe a, b, c zrzutowane jako VEC3.
inline const VEC3 & GetPlaneNormal(const PLANE &Plane) { return (const VEC3&)Plane.a; }

/// Zwraca jaki punkt lecy na tej paszczynie
/** \param Plane Musi by znormalizowana. */
inline void GetPlaneMemberPoint(VEC3 *Out, const PLANE &Plane)
{
	*Out = GetPlaneNormal(Plane) * -Plane.d;
}

/// Zmienia odlego od pocztku ukadu wsprzdnych.
/** Paszczyzna pozostajc tak samo zorientowana zostaa przesunita tak, eby
zawiera w sobie podany punkt.
\param[in,out] Plane Musi by znormalizowana.
*/
inline void RecalcPlaneD(PLANE *Plane, const VEC3 &p)
{
	Plane->d = -Dot(GetPlaneNormal(*Plane), p);
}

/// Zwraca true, jeli podane paszczyzny s sobie mniej wicej rwne
inline bool PlaneEqual(const PLANE &p1, const PLANE &p2) {
	return float_equal(p1.a, p2.a) && float_equal(p1.b, p2.b) && float_equal(p1.c, p2.c) && float_equal(p1.d, p2.d);
}

/// Tworzy paszczyzn na podstawie trzech punktw
/** Paszczyzna bdzie zorientowana dodatnio w kierunku tej pprzestrzeni,
z ktrej wierzchoki trjkta s widoczne jako zorientowane zgodnie z ruchem
wskazwek zegara w ukadzie LH. */
inline void PointsToPlane(PLANE *Out, const VEC3 &p1, const VEC3 &p2, const VEC3 &p3) {
	Cross(&GetPlaneNormal(*Out), p2-p1, p3-p1);
	Normalize(&GetPlaneNormal(*Out));
	Out->d = -Dot(p1, GetPlaneNormal(*Out));
}
/// Tworzy paszczyzn z wektora normalnego i punktu lecego na tej paszczynie
/** Czy Normal musi by znormalizowany? Nie wiem. */
inline void PointNormalToPlane(PLANE *Out, const VEC3 &Point, const VEC3 &Normal) {
	Out->a = Normal.x; Out->b = Normal.y; Out->c = Normal.z;
	Out->d = -Dot(Normal, Point);
}

/// Skalowanie paszczyzny
inline void Mul(PLANE *Out, const PLANE &p, float s)
{
	Out->a = p.a * s; Out->b = p.b * s;
	Out->c = p.c * s; Out->d = p.d * s;
}
inline void Div(PLANE *Out, const PLANE &p, float s)
{
	Out->a = p.a / s; Out->b = p.b / s;
	Out->c = p.c / s; Out->d = p.d / s;
}

/// Iloczyn skalarny paszczyzny i punktu we wsprzdnych jednorodnych
/** Mona uy do wyznaczenia czy tak zapisany punkt ley na paszczynie,
po ktrej stronie i jak daleko. */
inline float Dot(const PLANE &p, const VEC4 &pt)
{
	return p.a*pt.x + p.b*pt.y + p.c*pt.z + p.d*pt.w;
}

/// Iloczyn skalarny paszczyzny z punktem 3D: a*x+b*y+c*z+d*1
/** Mona uy do wyznaczenia czy punkt ley na paszczynie, po ktrej stronie
i jak daleko. */
inline float DotCoord(const PLANE &p, const VEC3 &pt)
{
	return p.a*pt.x + p.b*pt.y + p.c*pt.z + p.d;
}

/// Iloczyn skalarny paszczyzny z wektorem 3D: a*x+b*y+c*z+d*0
/** Przydatny do wyznaczania kta midzy paszczyzn (jej normaln) a wektorem. */
inline float DotNormal(const PLANE &p, const VEC3 &v)
{
	return p.a*v.x + p.b*v.y + p.c*v.z;
}

/// Normalizacja paszczyzny
/** Po tej operacji wektor normalny paszczyzny jest znormalizowany.
Paszczyzna pozostaje pooona i zorientowana tam gdzie bya. */
inline void Normalize(PLANE *Out, const PLANE &p)
{
	float nlsq = 1.0f / Length(GetPlaneNormal(p));
	Out->a = p.a * nlsq; Out->b = p.b * nlsq;
	Out->c = p.c * nlsq; Out->d = p.d * nlsq;
}
/// Normalizacja paszczyzny w miejscu
inline void Normalize(PLANE *p)
{
	float nlsq = 1.0f / Length(GetPlaneNormal(*p));
	p->a *= nlsq; p->b *= nlsq;
	p->c *= nlsq; p->d *= nlsq;
}

//@}
// math_plane


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_line2d Prosta 2D */
//@{

/// Prosta 2D
/**
Reprezentuje prost w przestrzeni 2D z jedn stron dodatni, a drug
ujemn. Opisana jest rwnaniem A x + B y + C = 0. Wektor [A,B] to wektor
normalny wskazujcy w kierunku dodatniej ppaszczyzny. Jeli prosta jest
znormalizowana, wektor normalny jest znormalizowany, a C to odlego prostej
od rodka ukadu wsprzdnych.
Reprezentacja jako string to A,B,C, np. "2,0,2.1"
Prosta majca a = b = 0 (czyli np. staa ZERO) jest niepoprawna, zdegenerowana.
*/
struct LINE2D
{
	float a, b, c;

	LINE2D() { }
	LINE2D(float a, float b, float c) : a(a), b(b), c(c) { }
	LINE2D(const VEC2 &Normal, float c) : a(Normal.x), b(Normal.y), c(c) { }
	LINE2D(const float *Arr) : a(Arr[0]), b(Arr[1]), c(Arr[2]) { }

	operator float * () { return &a; }
	operator const float * () const { return &a; }

	VEC2 & GetNormal() { return (VEC2&)a; }
	const VEC2 & GetNormal() const { return (VEC2&)a; }

	bool operator == (const LINE2D &L) const { return a == L.a && b == L.b && c == L.c; }
	bool operator != (const LINE2D &L) const { return a != L.a || b != L.b || c != L.c; }

	LINE2D operator + () const { return *this; }
	/// Zwraca prost zwrcon w przeciwn stron
	LINE2D operator - () const { return LINE2D(-a, -b, -c); }

	// Skalowanie
	LINE2D operator * (float s) const { return LINE2D(a*s, b*s, c*s); }
	LINE2D operator / (float s) const { float sInv = 1.0f/s; return LINE2D(a*sInv, b*sInv, c*sInv); }
	LINE2D & operator *= (float s) { a *= s; b *= s; c *= s; return *this; }
	LINE2D & operator /= (float s) { float sInv = 1.0f/s; a *= sInv; b *= sInv; c *= sInv; return *this; }

	/// Zwraca wektor wskazujcy w kierunku prostej
	void GetTangent(VEC2 *Out) { Out->x = b; Out->y = -a; }

	/// Zwraca jaki punkt lecy na tej prostej
	/** Musi by znormalizowana. */
	void GetMemberPoint(VEC2 *Out)
	{
		*Out = GetNormal() * -c;
	}

	/// Zmienia odlego od pocztku ukadu wsprzdnych
	/** Prosta pozostajc tak samo
	zorientowana zostaje przesunita tak, eby zawiera w sobie podany punkt.
	Musi by znormalizowana. */
	void RecalcD(const VEC2 &p)
	{
		c = -Dot(GetNormal(), p);
	}
};
inline LINE2D operator * (float s, const LINE2D &p) { return LINE2D(p.a*s, p.b*s, p.c*s); }
inline LINE2D operator / (float s, const LINE2D &p) { float sInv = 1.0f/s; return LINE2D(p.a*sInv, p.b*sInv, p.c*sInv); }

extern const LINE2D LINE2D_ZERO;
extern const LINE2D LINE2D_POSITIVE_X;
extern const LINE2D LINE2D_NEGATIVE_X;
extern const LINE2D LINE2D_POSITIVE_Y;
extern const LINE2D LINE2D_NEGATIVE_Y;

/// Zwraca true, jeli podane proste s sobie mniej wicej rwne
inline bool LineEqual(const LINE2D &p1, const LINE2D &p2) {
	return float_equal(p1.a, p2.a) && float_equal(p1.b, p2.b) && float_equal(p1.c, p2.c);
}

inline void swap(LINE2D &p1, LINE2D &p2)
{
	std::swap(p1.a, p2.a); std::swap(p1.b, p2.b); std::swap(p1.c, p2.c);
}

/// Tworzy prost na podstwie dwch punktw
/** Wynikowa prosta nie jest znormalizowana. */
inline void PointsToLine(LINE2D *Out, const VEC2 &p1, const VEC2 &p2) {
	Out->a = p1.y - p2.y;
	Out->b = p2.x - p1.x;
	Out->c = - Out->a * p1.x - Out->b * p1.y;
}

/// Tworzy prost z wektora normalnego i punktu lecego na tej prostej
inline void PointNormalToLine(LINE2D *Out, const VEC2 &Point, const VEC2 &Normal) {
	Out->a = Normal.x; Out->b = Normal.y;
	Out->c = -Dot(Normal, Point);
}

/// Zwraca true, jeli podane proste s rwnolege
inline bool LinesParallel(const LINE2D &L1, const LINE2D &L2) {
	return FLOAT_ALMOST_ZERO(L1.a*L2.b - L2.a*L1.b);
}

/// Zwraca true, jeli podane proste s prostopade
inline bool LinesPerpedicular(const LINE2D &L1, const LINE2D &L2) {
	return FLOAT_ALMOST_ZERO(L1.a*L2.a + L1.b*L2.b);
}

/// Zwraca dot z wektorw normalnych dwch prostych
/** Jeli te proste s znormalizowane, wynikiem jest cos kta midzy tymi prostymi. */
inline float Dot(const LINE2D &L1, const LINE2D &L2) {
	return L1.a*L2.a + L1.b*L2.b;
}

/// Zwraca signed odlego dwch prostych na paszczynie
/** Proste musz by znormalizowane. */
inline float LineDistance(const LINE2D &L1, const LINE2D &L2) {
	return L1.c - L2.c;
}

/// Oblicza punkt przeciecia dwch prostych
/** Jeli proste nie przecinaj si, tylko s rwnolege lub si pokrywan, to nie oblicza tylko zwraca false. */
bool LinesIntersection(VEC2 *Out, const LINE2D &L1, const LINE2D &L2);

/// Zwraca true, jeli podane trzy punkty le na jednej prostej
bool PointsColinear(const VEC2 &p1, const VEC2 &p2, const VEC2 &p3);

/// Skalowanie prostej
inline void Mul(LINE2D *Out, const LINE2D &p, float s) {
	Out->a = p.a * s; Out->b = p.b * s; Out->c = p.c * s;
}
inline void Div(LINE2D *Out, const LINE2D &p, float s) {
	Out->a = p.a / s; Out->b = p.b / s; Out->c = p.c / s;
}

/// Iloczyn skalarny paszczyzny i punktu we wsprzdnych jednorodnych 2D
/** Mona uy do wyznaczenia czy tak zapisany punkt ley na paszczynie,
po ktrej stronie i jak daleko. */
inline float Dot(const LINE2D &p, const VEC3 &pt) {
	return p.a*pt.x + p.b*pt.y + p.c*pt.z;
}

/// Iloczyn skalarny prostej 2D z punktem 2D: a*x + b*y + c*1
/** Mona uy do wyznaczenia czy punkt ley na prostej 2D, po ktrej stronie i jak daleko. */
inline float DotCoord(const LINE2D &p, const VEC2 &pt) {
	return p.a*pt.x + p.b*pt.y + p.c;
}

/// Iloczyn skalarny prostej 2D z wektorem 2D: a*x + b*y + c*z + d*0
/** Przydatny do wyznaczania kta midzy prost 2D (jej normaln) a wektorem. */
inline float DotNormal(const LINE2D &p, const VEC2 &v) {
	return p.a*v.x + p.b*v.y;
}

/// Normalizacja prostej 2D
/** Po tej operacji wektor normalny prostej jest znormalizowany.
Prosta pozostaje pooona i zorientowana tam gdzie bya. */
inline void Normalize(LINE2D *Out, const LINE2D &p) {
	float nlsq = 1.f / Length(p.GetNormal());
	Out->a = p.a * nlsq; Out->b = p.b * nlsq; Out->c = p.c * nlsq;
}
/// Normalizacja prostej w miejscu
inline void Normalize(LINE2D *p) {
	float nlsq = 1.f / Length(p->GetNormal());
	p->a *= nlsq; p->b *= nlsq; p->c *= nlsq;
}

//@}
// math_line3d


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_matrix Macierz */
//@{

#if defined(USE_DIRECTX) && !defined(DOXYGEN)
	typedef D3DXMATRIX MATRIX;
#else
	/// Macierz 4 x 4
	/**
	Elementy s indeksowane [wiersz,kolumna].
	Reprezentacja tekstowa to:
		a11,a12,a13,a14;a21,a22,a23,a24;a31,a32,a33,a34;a41,a42,a43,a44
	*/
	struct MATRIX
	{
		union
		{
			struct
			{
				float _11, _12, _13, _14;
				float _21, _22, _23, _24;
				float _31, _32, _33, _34;
				float _41, _42, _43, _44;
			};
			float m[4][4]; // [wiersz][kolumna]
		};

		MATRIX() { }
		MATRIX(float _11, float _12, float _13, float _14, float _21, float _22, float _23, float _24, float _31, float _32, float _33, float _34, float _41, float _42, float _43, float _44) : _11(_11), _12(_12), _13(_13), _14(_14), _21(_21), _22(_22), _23(_23), _24(_24), _31(_31), _32(_32), _33(_33), _34(_34), _41(_41), _42(_42), _43(_43), _44(_44) { }
		MATRIX(const float *Array) : _11(Array[0]), _12(Array[1]), _13(Array[2]), _14(Array[3]), _21(Array[4]), _22(Array[5]), _23(Array[6]), _24(Array[7]), _31(Array[8]), _32(Array[9]), _33(Array[10]), _34(Array[11]), _41(Array[12]), _42(Array[13]), _43(Array[14]), _44(Array[15]) { }

		operator float * () { return &_11; }
		operator const float * () const { return &_11; }

		bool operator == (const MATRIX &m) const { return _11 == m._11 && _12 == m._12 && _13 == m._13 && _14 == m._14 && _21 == m._21 && _22 == m._22 && _23 == m._23 && _24 == m._24 && _31 == m._31 && _32 == m._32 && _33 == m._33 && _34 == m._34 && _41 == m._41 && _42 == m._42 && _43 == m._43 && _44 == m._44; }
		bool operator != (const MATRIX &m) const { return _11 != m._11 || _12 != m._12 || _13 != m._13 || _14 != m._14 || _21 != m._21 || _22 != m._22 || _23 != m._23 || _24 != m._24 || _31 != m._31 || _32 != m._32 || _33 != m._33 || _34 != m._34 || _41 != m._41 || _42 != m._42 || _43 != m._43 || _44 != m._44; }

		float & operator () (unsigned Row, unsigned Col) { return m[Row][Col]; }
		float operator () (unsigned Row, unsigned Col) const { return m[Row][Col]; }

		MATRIX operator + () const { return *this; }
		MATRIX operator - () const;
		MATRIX operator + (const MATRIX &m) const;
		MATRIX operator - (const MATRIX &m) const;
		MATRIX operator * (const MATRIX &m) const
		{
			return MATRIX(
				_11 * m._11 + _12 * m._21 + _13 * m._31 + _14 * m._41,
				_11 * m._12 + _12 * m._22 + _13 * m._32 + _14 * m._42,
				_11 * m._13 + _12 * m._23 + _13 * m._33 + _14 * m._43,
				_11 * m._14 + _12 * m._24 + _13 * m._34 + _14 * m._44,

				_21 * m._11 + _22 * m._21 + _23 * m._31 + _24 * m._41,
				_21 * m._12 + _22 * m._22 + _23 * m._32 + _24 * m._42,
				_21 * m._13 + _22 * m._23 + _23 * m._33 + _24 * m._43,
				_21 * m._14 + _22 * m._24 + _23 * m._34 + _24 * m._44,

				_31 * m._11 + _32 * m._21 + _33 * m._31 + _34 * m._41,
				_31 * m._12 + _32 * m._22 + _33 * m._32 + _34 * m._42,
				_31 * m._13 + _32 * m._23 + _33 * m._33 + _34 * m._43,
				_31 * m._14 + _32 * m._24 + _33 * m._34 + _34 * m._44,

				_41 * m._11 + _42 * m._21 + _43 * m._31 + _44 * m._41,
				_41 * m._12 + _42 * m._22 + _43 * m._32 + _44 * m._42,
				_41 * m._13 + _42 * m._23 + _43 * m._33 + _44 * m._43,
				_41 * m._14 + _42 * m._24 + _43 * m._34 + _44 * m._44);
		}
		MATRIX & operator += (const MATRIX &m);
		MATRIX & operator -= (const MATRIX &m);
		MATRIX & operator *= (const MATRIX &m);

		MATRIX operator * (float s) const;
		MATRIX operator / (float s) const;
		MATRIX & operator *= (float s);
		MATRIX & operator /= (float s);
	};
#endif

extern const MATRIX MATRIX_ZERO;
extern const MATRIX MATRIX_IDENTITY;

void MatrixToStr(tstring *Out, const MATRIX &m);
bool StrToMatrix(MATRIX *Out, const tstring &Str);

inline void swap(MATRIX &m1, MATRIX &m2)
{
	std::swap(m1._11, m2._11); std::swap(m1._12, m2._12); std::swap(m1._13, m2._13); std::swap(m1._14, m2._14);
	std::swap(m1._21, m2._21); std::swap(m1._22, m2._22); std::swap(m1._23, m2._23); std::swap(m1._24, m2._24);
	std::swap(m1._31, m2._31); std::swap(m1._32, m2._32); std::swap(m1._33, m2._33); std::swap(m1._34, m2._34);
	std::swap(m1._41, m2._41); std::swap(m1._42, m2._42); std::swap(m1._43, m2._43); std::swap(m1._44, m2._44);
}

/// Zwraca true, jeli podane macierze s sobie mniej wicej rwne
bool MatrixEqual(const MATRIX &m1, const MATRIX &m2);
/// Negacja macierzy w miejscu
void Minus(MATRIX *m);
/// Dodawanie macierzy
void Add(MATRIX *Out, const MATRIX &m1, const MATRIX &m2);
/// Odejmowanie macierzy
void Sub(MATRIX *Out, const MATRIX &m1, const MATRIX &m2);
/// Mnoenie macierzy
void Mul(MATRIX *Out, const MATRIX &m1, const MATRIX &m2);
/// Mnoenie macierzy przez skalar w miejscu
void Mul(MATRIX *m, float s);
/// Dzielenie macierzy przez skalar w miejscu
void Div(MATRIX *m, float s);

/// Mnoenie wektora 2D jako [x,y,0,1] przez macierz, wychodzi wektor 4D
void Transform(VEC4 *Out, const VEC2 &v, const MATRIX &m);
/// Mnoenie wektora 2D jako [x,y,0,1] przez macierz z olaniem wyjciowego W
void Transform(VEC2 *Out, const VEC2 &v, const MATRIX &m);
/// Mnoenie wektora 2D jako [x,y,0,1] przez macierz i sprowadzenie z powrotem do 2D przez podzielenie przez W
void TransformCoord(VEC2 *Out, const VEC2 &v, const MATRIX &m);
/// Mnoenie wektora 2D jako [x,y,0,0] przez macierz
void TransformNormal(VEC2 *Out, const VEC2 &v, const MATRIX &m);

/// Mnoenie wektora 3D jako [X,Y,Z,1] przez macierz, wychodzi wektor 4D
void Transform(VEC4 *Out, const VEC3 &v, const MATRIX &m);
/// Mnoenie wektora 3D jako [X,Y,Z,1] przez macierz z olaniem wyjciowego W
/** Dobre do przeksztacania wektorw 3D jeli macierz reprezentuje przeksztacenie liniowe. */
void Transform(VEC3 *Out, const VEC3 &v, const MATRIX &m);
/// Mnoenie wektora 3D jako [X,Y,Z,1] przez macierz i sprowadzenie z powrotem do 3D przez podzielenie przez W
/** Dobre do niezawodnego przeksztacania wektorw 3D przez dowoln macierz np. rzutowania. */
void TransformCoord(VEC3 *Out, const VEC3 &v, const MATRIX &m);
/// Mnoenie wektora 3D jako [X,Y,Z,0] przez macierz
/** Dobre do przeksztacania wektorw (np. normalnych), nie punktw, bo nie robi translacji. */
void TransformNormal(VEC3 *Out, const VEC3 &v, const MATRIX &m);

/// Przeksztaca na raz ca tablic wektorw - w miejscu lub z tablicy wejciowej do wyjciowej
void TransformArray(VEC3 OutPoints[], const VEC3 InPoints[], size_t PointCount, const MATRIX &M);
void TransformArray(VEC3 InOutPoints[], size_t PointCount, const MATRIX &M);
void TransformNormalArray(VEC3 OutPoints[], const VEC3 InPoints[], size_t PointCount, const MATRIX &M);
void TransformNormalArray(VEC3 InOutPoints[], size_t PointCount, const MATRIX &M);
void TransformCoordArray(VEC3 OutPoints[], const VEC3 InPoints[], size_t PointCount, const MATRIX &M);
void TransformCoordArray(VEC3 InOutPoints[], size_t PointCount, const MATRIX &M);

/// Mnoenie wektora 4D przez macierz
/** Czyli przeksztacenie wektora we wsprzdnych jednorodnych przez t macierz. */
void Transform(VEC4 *Out, const VEC4 &v, const MATRIX &m);

/// Mnoy wektor przez transpozycj podanej macierzy m (jej podmacierzy 3x3)
void TransformNormalByTranspose(VEC3 *Out, const VEC3 &v, const MATRIX &m);
/// Mnoy punkt przez odwrotnos podanej macierzy m (jej podmacierzy 4x3), ale bez odwracania.
/** Czyli jeli macierz zawiera rotacj i translacj, to wykonuje na punkcie
przeksztacenie odwrotne do tego ktre jest w tej macierzy. \n
Pomysodawca: Krzysiek K. */
void Untransform(VEC3 *Out, const VEC3 &v, const MATRIX &m);
/// Mnoy wektor przez odwrotnos podanej macierzy m (jej podmacierzy 3x3), ale bez odwracania.
/** Czyli jeli macierz zawiera rotacj i translacj, to wykonuje na punkcie
przeksztacenie odwrotne do rotacji, ktra jest w tej macierzy. */
void UntransformNormal(VEC3 *Out, const VEC3 &v, const MATRIX &m);

/// Przeksztaca paszczyzn przez macierz
/** \param p Paszczyzna ma by znormalizowana.
\param m Macierz ma by odwrotna i transponowana wzgldem normalnej macierzy. */
void Transform(PLANE *Out, const PLANE &p, const MATRIX &m);
/// Przeksztaca promie przez macierz
void TransformRay(VEC3 *OutOrigin, VEC3 *OutDir, const VEC3 &RayOrigin, const VEC3 &RayDir, const MATRIX &m);

/// Przeksztaca AABB przez podan macierz i wylicza AABB tego co wyszo po przeksztaceniu
void TransformBox(BOX *Out, const BOX &In, const MATRIX &M);
/// Przeksztaca AABB przez podan macierz i wylicza AABB tego co wyszo po przeksztaceniu
/** Wersja dzielca przez W - mona stosowa te do macierzy projekcji. */
void TransformBoxCoord(BOX *Out, const BOX &In, const MATRIX &M);

/// Ustawia translacj w macierzy
void MatrixSetTranslation(MATRIX *InOut, float TranslationX, float TranslationY, float TranslationZ);
/// Ustawia translacj w macierzy
void MatrixSetTranslation(MATRIX *InOut, const VEC3 &Translation);
/// Pobiera translacj z macierzy
void MatrixGetTranslation(float *OutTranslationX, float *OutTranslationY, float *OutTranslationZ, const MATRIX &m);
/// Pobiera translacj z macierzy
void MatrixGetTranslation(VEC3 *OutTranslation, const MATRIX &m);

/// Tworzy macierz przeksztacajc do lokalnego ukadu wsprzdnych wyznaczonego przez podane 3 osie
/** Nastpi obrt, a jeli nie s znormalizowane to i skalowanie. */
void AxesToMatrix(MATRIX *Out, const VEC3 &AxisX, const VEC3 &AxisY, const VEC3 &AxisZ);
/// Tworzy macierz przeksztacajc do lokalnego ukadu wsprzdnych wyznaczonego przez podane 3 osie i pozycj pocztku tego ukadu.
/** Nastpi obrt, a jeli osie s znormalizowane to i skalowanie. */
void AxesToMatrixTranslation(MATRIX *Out, const VEC3 &Origin, const VEC3 &AxisX, const VEC3 &AxisY, const VEC3 &AxisZ);

/// Macierz kamery
/** Eye to punkt, Forward i Up to wektory. */
void LookAtLH(MATRIX *Out, const VEC3 &Eye, const VEC3 &Forward, const VEC3 &Up);
void LookAtRH(MATRIX *Out, const VEC3 &Eye, const VEC3 &Forward, const VEC3 &Up);

/// Tworzy macierz identycznociow
void Identity(MATRIX *Out);
/// Tworzy macierz translacji (przesuwania)
void Translation(MATRIX *Out, float x, float y, float z);
void Translation(MATRIX *Out, const VEC3 &v);
/// Tworzy macierz skalowania (powikszania, pomniejszania, rozcigania)
void Scaling(MATRIX *Out, float sx, float sy, float sz);
void Scaling(MATRIX *Out, const VEC3 &sv);
void Scaling(MATRIX *Out, float s);
/// Tworzy macierz skalowania wzdu podanego kierunku
/** \param k   wspczynnik skalowania
\param n   wektor kierunku, musi by znormalizowany */
void ScalingAxis(MATRIX *Out, const VEC3 &n, float k);

/// Macierz obrotu 2D
void Rotation2D(MATRIX *Out, float Angle);
/// Macierz obrotu wok osi X
void RotationX(MATRIX *Out, float Angle);
/// Macierz obrotu wok osi Y
void RotationY(MATRIX *Out, float Angle);
/// Macierz obrotu wok osi Z
void RotationZ(MATRIX *Out, float Angle);
/** Axis musi by znormalizowany. */
void RotationAxisLH(MATRIX *Out, const VEC3 &Axis, float Angle);
inline void RotationAxisRH(MATRIX *Out, const VEC3 &Axis, float Angle) { RotationAxisLH(Out, Axis, -Angle); }

/// Macierz obrotu wok trzech osi w kolejnoci Yaw-Pitch-Roll (ZXY) - tzw. kty Eulera
/** W przypadku wektora skadowe odpowiadaj obrotom wok poszczeglnych osi, czyli:
  X = Pitch, Y = Yaw, Z = Roll
Przeksztaca ze wsp. obiektu do wiata (object -> interial space).
Czyli tak naprawd robi ZXY.
\param Yaw jest wok osi Y.
\param Pitch jest wok osi X.
\param Roll jest wok osi Z. */
void RotationYawPitchRoll(MATRIX *Out, float Yaw, float Pitch, float Roll);
void RotationYawPitchRoll(MATRIX *Out, const VEC3 &EulerAngles);
/// Przeksztaca ze wsp. wiata do obiektu (interial -> object space).
/** Czyli tworzy macierz dokadnie odwrotn do tej z RotationYawPitchRoll. */
void RotationYawPitchRollInv(MATRIX *Out, float Yaw, float Pitch, float Roll);
void RotationYawPitchRollInv(MATRIX *Out, const VEC3 &EulerAngles);

/*/// Przetwarza macierz obrotu (to musi by czysta macierz obrotu) na kty Eulera
void RotationMatrixToEulerAngles(const MATRIX &m, float *Yaw, float *Pitch, float *Roll);*/

/// Tworzy macierz rzutowania ortogonalnego
/** Dane s w ukadzie kamery. */
void OrthoLH(MATRIX *Out, float Width, float Height, float ZNear, float ZFar);
void OrthoRH(MATRIX *Out, float Width, float Height, float ZNear, float ZFar);
void OrthoOffCenterLH(MATRIX *Out, float Left, float Right, float Bottom, float Top, float ZNear, float ZFar);
void OrthoOffCenterRH(MATRIX *Out, float Left, float Right, float Bottom, float Top, float ZNear, float ZFar);
/// Tworzy macierz rzutowania perspektywicznego
/** Width i Height s wymiarami na paszczynie bliskiej (near).
Left, Top, Right, Bottom prawdopodobnie te.
Tworzone macierze s "W-Friendly" - wektor przemnoony przez tak macierz ma w skadowej W komponent Z.
*/
void PerspectiveLH(MATRIX *Out, float Width, float Height, float ZNear, float ZFar);
void PerspectiveRH(MATRIX *Out, float Width, float Height, float ZNear, float ZFar);
void PerspectiveFovLH(MATRIX *Out, float FovY, float Aspect, float ZNear, float ZFar);
void PerspectiveFovRH(MATRIX *Out, float FovY, float Aspect, float ZNear, float ZFar);
void PerspectiveOffCenterLH(MATRIX *Out, float Left, float Right, float Bottom, float Top, float ZNear, float ZFar);
void PerspectiveOffCenterRH(MATRIX *Out, float Left, float Right, float Bottom, float Top, float ZNear, float ZFar);
/// Wersja tworzca macierz z Z-Far = INFINITY
void PerspectiveFovLH_Inf(MATRIX *Out, float FovY, float Aspect, float ZNear);
/// Tworzy macierz spaszczajc geometri na podan paszczyzn w podanym kierunku padania wiata
/** Paszczyzna musi by znormalizowana. */
void MatrixShadow(MATRIX *Out, const VEC4 &Light, const PLANE &Plane);
/// Tworzy macierz odbijajc geometri wzgldem podanej paszczyzny
/** Paszczyzna musi by znormalizowana. */
void MatrixReflect(MATRIX *Out, const PLANE &p);

/// Transpozycja macierzy na miejscu
void Transpose(MATRIX *m);
/// Transpozycja macierzy
void Transpose(MATRIX *Out, const MATRIX &m);
/// Interpolacja liniowa macierzy
void Lerp(MATRIX *Out, const MATRIX &m1, const MATRIX &m2, float t);
/// Wyznacznik macierzy
float Det(const MATRIX &m);
/// Odwrotno macierzy
bool Inverse(MATRIX *Out, const MATRIX &m);


/// Macierz 3 x 3
/**
Elementy s indeksowane [wiersz,kolumna].
Reprezentacja tekstowa to:
	a11,a12,a13,a21,a22,a23,a31,a32,a33
*/
struct MATRIX33
{
	union
	{
		struct
		{
			float _11, _12, _13;
			float _21, _22, _23;
			float _31, _32, _33;
		};
		float m[3][3]; // [wiersz][kolumna]
	};

	MATRIX33() { }
	MATRIX33(float _11, float _12, float _13, float _21, float _22, float _23, float _31, float _32, float _33) : _11(_11), _12(_12), _13(_13), _21(_21), _22(_22), _23(_23), _31(_31), _32(_32), _33(_33) { }
	MATRIX33(const float *Array) : _11(Array[0]), _12(Array[1]), _13(Array[2]), _21(Array[4]), _22(Array[5]), _23(Array[6]), _31(Array[8]), _32(Array[9]), _33(Array[10]) { }

	operator float * () { return &_11; }
	operator const float * () const { return &_11; }

	bool operator == (const MATRIX33 &m) const { return _11 == m._11 && _12 == m._12 && _13 == m._13 && _21 == m._21 && _22 == m._22 && _23 == m._23 && _31 == m._31 && _32 == m._32 && _33 == m._33; }
	bool operator != (const MATRIX33 &m) const { return _11 != m._11 || _12 != m._12 || _13 != m._13 || _21 != m._21 || _22 != m._22 || _23 != m._23 || _31 != m._31 || _32 != m._32 || _33 != m._33; }

	float & operator () (unsigned Row, unsigned Col) { return m[Row][Col]; }
	float operator () (unsigned Row, unsigned Col) const { return m[Row][Col]; }

	MATRIX33 operator + () const { return *this; }
	MATRIX33 operator - () const;
	MATRIX33 operator + (const MATRIX33 &m) const;
	MATRIX33 operator - (const MATRIX33 &m) const;
	MATRIX33 operator * (const MATRIX33 &m) const
	{
		return MATRIX33(
			_11 * m._11 + _12 * m._21 + _13 * m._31,
			_11 * m._12 + _12 * m._22 + _13 * m._32,
			_11 * m._13 + _12 * m._23 + _13 * m._33,

			_21 * m._11 + _22 * m._21 + _23 * m._31,
			_21 * m._12 + _22 * m._22 + _23 * m._32,
			_21 * m._13 + _22 * m._23 + _23 * m._33,

			_31 * m._11 + _32 * m._21 + _33 * m._31,
			_31 * m._12 + _32 * m._22 + _33 * m._32,
			_31 * m._13 + _32 * m._23 + _33 * m._33);
	}
	MATRIX33 & operator += (const MATRIX33 &m);
	MATRIX33 & operator -= (const MATRIX33 &m);
	MATRIX33 & operator *= (const MATRIX33 &m);

	MATRIX33 operator * (float s) const;
	MATRIX33 operator / (float s) const;
	MATRIX33 & operator *= (float s);
	MATRIX33 & operator /= (float s);
};

extern const MATRIX33 MATRIX33_ZERO;
extern const MATRIX33 MATRIX33_IDENTITY;

/// Konwertuje macierz 3x3 na macierz 4x4
void Matrix33ToMatrix(MATRIX *Out, const MATRIX33 &m);
/// Wyciga podmacierz 3x3 z macierzy 4x4
void MatrixToMatrix33(MATRIX33 *Out, const MATRIX &m);

/// Tworzy macierz identycznociow
void Identity(MATRIX33 *Out);

/// Transpozycja macierzy na miejscu
void Transpose(MATRIX33 *m);
/// Transpozycja macierzy
void Transpose(MATRIX33 *Out, const MATRIX33 &m);

//@}
// math_matrix


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_quaternion Kwaternion */
//@{

#if defined(USE_DIRECTX) && !defined(DOXYGEN)
	typedef D3DXQUATERNION QUATERNION;
#else
	/// Kwaternion
	/**
	Reprezentacja jako acuch to: "x,y,z,w", np.: "1,2,3.5,-10".
	Kwaterniony s mnoone od drugiej strony ni matematycznie, dziki czemu mona
	czy obroty ktre reprezentuj w porzdku naturalnym, tak jak macierze.
	*/
	struct QUATERNION
	{
		float x, y, z, w;

		QUATERNION() { }
		QUATERNION(float x, float y, float z, float w) : x(x), y(y), z(z), w(w) { }
		QUATERNION(const float *Array) : x(Array[0]), y(Array[1]), z(Array[2]), w(Array[3]) { }

		operator float * () { return &x; }
		operator const float * () const { return &x; }

		bool operator == (const QUATERNION &v) const { return x == v.x && y == v.y && z == v.z && w == v.w; }
		bool operator != (const QUATERNION &v) const { return x != v.x || y != v.y || z != v.z || w != v.w; }

		QUATERNION operator + () const { return *this; }
		QUATERNION operator - () const { return QUATERNION(-x, -y, -z, -w); }
		QUATERNION operator + (const QUATERNION &q) const { return QUATERNION(x+q.x, y+q.y, z+q.z, w+q.w); }
		QUATERNION operator - (const QUATERNION &q) const { return QUATERNION(x-q.x, y-q.y, z-q.z, w-q.w); }
		QUATERNION operator * (const QUATERNION &q) const { return QUATERNION(w*q.x + x*q.w + y*q.z - z*q.y, w*q.y + y*q.w + z*q.x - x*q.z, w*q.z + z*q.w + x*q.y - y*q.x, w*q.w - x*q.x - y*q.y - z*q.z); }
		QUATERNION & operator += (const QUATERNION &q) { x += q.x; y += q.y; z += q.z; w += q.w; return *this; }
		QUATERNION & operator -= (const QUATERNION &q) { x -= q.x; y -= q.y; z -= q.z; w -= q.w; return *this; }
		QUATERNION & operator *= (const QUATERNION &q) { QUATERNION R; R.x = w*q.x + x*q.w + y*q.z - z*q.y; R.y = w*q.y + y*q.w + z*q.x - x*q.z; R.z = w*q.z + z*q.w + x*q.y - y*q.x; R.w = w*q.w - x*q.x - y*q.y - z*q.z; *this = R; return *this; }
		QUATERNION & operator *= (float v) { x *= v; y *= v; z *= v; w *= v; return *this; }
		QUATERNION & operator /= (float v) { float vInv = 1.0f/v; x *= vInv; y *= vInv; z *= vInv; w *= vInv; return *this; }
	};
	inline QUATERNION operator * (const QUATERNION pt, float v) { return QUATERNION(pt.x*v, pt.y*v, pt.z*v, pt.w*v); }
	inline QUATERNION operator / (const QUATERNION pt, float v) { float vInv = 1.0f/v; return QUATERNION(pt.x*vInv, pt.y*vInv, pt.z*vInv, pt.w*vInv); }
	inline QUATERNION operator * (float v, const QUATERNION pt) { return QUATERNION(pt.x*v, pt.y*v, pt.z*v, pt.w*v); }
	inline QUATERNION operator / (float v, const QUATERNION pt) { float vInv = 1.0f/v; return QUATERNION(pt.x*vInv, pt.y*vInv, pt.z*vInv, pt.w*vInv); }
#endif

extern const QUATERNION QUATERNION_ZERO;
extern const QUATERNION QUATERNION_IDENTITY;

inline void swap(QUATERNION &v1, QUATERNION &v2)
{
	std::swap(v1.x, v2.x);
	std::swap(v1.y, v2.y);
	std::swap(v1.z, v2.z);
	std::swap(v1.w, v2.w);
}

/// Zwraca kt obrotu jaki reprezentuje ten kwaternion
inline float CalcQuaternionAngle(const QUATERNION &Q) { return safe_acos(Q.w) * 2.0f; }
/// Zwraca o obrotu jak reprezentuje ten kwaternion
void CalcQuaternionAxis(VEC3 *Out, const QUATERNION &Q);

/// Zwraca true, jeli podane kwaterniony s sobie mniej wicej rwne
inline bool QuaternionEqual(const QUATERNION &q1, const QUATERNION &q2) {
	return float_equal(q1.x, q2.x) && float_equal(q1.y, q2.y) && float_equal(q1.z, q2.z) && float_equal(q1.w, q2.w);
}

/// Ujemny kwaternion
void Minus(QUATERNION *q);
inline void Minus(QUATERNION *Out, const QUATERNION &q) { *Out = q; Minus(Out); }
/// Dodawanie, odejmowanie, mnoenie kwaternionw
void Add(QUATERNION *Out, const QUATERNION &q1, const QUATERNION &q2);
void Sub(QUATERNION *Out, const QUATERNION &q1, const QUATERNION &q2);
void Mul(QUATERNION *Out, const QUATERNION &q1, const QUATERNION &q2);
/// Mnoenie i dzielenie kwaterniona przez skalar
void Mul(QUATERNION *q, float v);
void Div(QUATERNION *q, float v);
inline void Mul(QUATERNION *Out, const QUATERNION &q, float v) { *Out = q; Mul(Out, v); }
inline void Div(QUATERNION *Out, const QUATERNION &q, float v) { *Out = q; Div(Out, v); }

/// Tworzy kwaternion na podstawie macierzy obrotu
void RotationMatrixToQuaternion(QUATERNION *Out, const MATRIX &RotationMatrix);
/// Przeksztaca kwaternion w macierz obrotu
void QuaternionToRotationMatrix(MATRIX *Out, const QUATERNION &q);
/// Przeksztaca punkt/wektor przez kwaternion obracajc go o ten kwaternion
/**
- To nie jest wcale super szybkie i nie przydaje si a tak czsto.
  Lepiej sobie przeksztaci kwaternion na macierz.
- Out = q * p * q^-1
*/
void QuaternionTransform(VEC3 *Out, const VEC3 &p, const QUATERNION &q);
/// Konwersja ktw Eulera na kwaternion (ktry obraca ze wsp. obiektu do wiata, czyli object -> interial space) [a moe odwrotnie?]
void EulerAnglesToQuaternionO2I(QUATERNION *Out, float Yaw, float Pitch, float Roll);
/// Konwersja ktw Eulera na kwaternion (ktry obraca ze wsp. wiata do obiektu, czyli interial -> object space) [a moe odwrotnie?]
void EulerAnglesToQuaternionI2O(QUATERNION *Out, float Yaw, float Pitch, float Roll);
/// Konwersja kwaterniona ktry obraca ze wsp. obiektu do wiata, czyli object -> interial space, na kty Eulera
void QuaternionO2IToEulerAngles(float *Yaw, float *Pitch, float *Roll, const QUATERNION &q);
/// Konwersja kwaterniona ktry obraca ze wsp. wiata do obiektu, czyli interial -> object space, na kty Eulera
void QuaternionI2OToEulerAngles(float *Yaw, float *Pitch, float *Roll, const QUATERNION &q);

/// Tworzy kwaternion obracajcy wok osi X
void QuaternionRotationX(QUATERNION *Out, float a);
/// Tworzy kwaternion obracajcy wok osi Y
void QuaternionRotationY(QUATERNION *Out, float a);
/// Tworzy kwaternion obracajcy wok osi Z
void QuaternionRotationZ(QUATERNION *Out, float a);
/// Tworzy kwaternion na podstawie osi obrotu i kta
void AxisToQuaternion(QUATERNION *Out, const VEC3 &Axis, float Angle);

/// Oblica "rnic" dwch kwaternionw reprezentujc taki obrt e a*Out = b
/** To jest obliczane tak: Out = a^-1 * b, wic jeli masz ju a^-1 to nie uywaj. */
void QuaternionDiff(QUATERNION *Out, const QUATERNION &a, const QUATERNION &b);
/// Kwaternion sprzony do danego
void Conjugate(QUATERNION *q);
inline void Conjugate(QUATERNION *Out, const QUATERNION &q) { *Out = q; Conjugate(Out); }
/// Odwrotno kwaterniona
void Inverse(QUATERNION *Out, const QUATERNION &q);
/// Normalizacja kwaternion
void Normalize(QUATERNION *Out, const QUATERNION &q);
void Normalize(QUATERNION *InOut);
/// Iloczyn skalarny dwch kwaternionw
float Dot(const QUATERNION &q1, const QUATERNION &q2);
/// Kwadrat dugoci kwaterniona
float LengthSq(const QUATERNION &q);
/// Dugo kwaterniona
float Length(const QUATERNION &q);
void Log(QUATERNION *Out, const QUATERNION &q);
void Exp(QUATERNION *Out, const QUATERNION &q);
/// Podnie kwaternion do potgi.
/** Wykadnik 0..1 daje interpolacje kwaterniona od identycznociowego do q. */
void Pow(QUATERNION *InOut, float t);
inline void Pow(QUATERNION *Out, const QUATERNION &q, float t) { *Out = q; Pow(Out, t); }
/// Interpolacja normalna kwaternionw
void Lerp(QUATERNION *Out, const QUATERNION &q1, const QUATERNION &q2, float t);
/// Interpolacja sferyczna kwaternionw
/** Sprawdzona, daje wyniki dokadnie tekie same jak D3DXQuaternionSlerp. */
void Slerp(QUATERNION *Out, const QUATERNION &q0, const QUATERNION &q1, float t);
/** NIE TESTOWANA */
void Squad(QUATERNION *Out, const QUATERNION &Q1, const QUATERNION &A, const QUATERNION &B, const QUATERNION &C, float t);
/** NIE TESTOWANA */
void SquadSetup(QUATERNION *OutA, QUATERNION *OutB, QUATERNION *OutC, const QUATERNION &Q0, const QUATERNION &Q1, const QUATERNION &Q2, const QUATERNION &Q3);

//@}
// math_quaternion


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_frustum Frustum */
//@{

struct FRUSTUM_PLANES;
struct FRUSTUM_POINTS;

/// Frustum opisany 6 paszczyznami z wektorami normalnymi zwrconymi do wewntrz.
/**
Podczas tworzenia:
- Jeli WorldViewProj jest macierz Proj, otrzymane paszczyzny s we wsp. widoku.
- Jeli WorldViewProj jest macierz View*Proj, otrzymane paszczyzny s we wsp. wiata.
- Jeli WorldViewProj jest macierz World*View*Proj, otrzymane paszczyzny s we wsp. lokalnych obiektu.
*/
struct FRUSTUM_PLANES
{
	/** \name Indeksy do tablicy paszczyzn
	Przedrostek PLANE_ jest, bo NEAR zostao gdzie zdefiniowane jako makro - grrrrr!
	*/
	//@{
	static const size_t PLANE_LEFT   = 0;
	static const size_t PLANE_RIGHT  = 1;
	static const size_t PLANE_TOP    = 2;
	static const size_t PLANE_BOTTOM = 3;
	static const size_t PLANE_NEAR   = 4;
	static const size_t PLANE_FAR    = 5;
	//@}

	PLANE Planes[6];

	/// Tworzy niezainicjalizowany
	FRUSTUM_PLANES() { }
	/// Inicjalizuje na podstawie macierzy
	FRUSTUM_PLANES(const MATRIX &WorldViewProj) { Set(WorldViewProj); }
	/// Inicjalizuje na podstawie reprezentacji punktowej
	FRUSTUM_PLANES(const FRUSTUM_POINTS &FrustumPoints) { Set(FrustumPoints); }
	/// Wypenia
	void Set(const MATRIX &WorldViewProj);
	void Set(const FRUSTUM_POINTS &FrustumPoints);
	/// Normalizuje paszczyzny
	void Normalize();

	PLANE & operator [] (size_t Index) { return Planes[Index]; }
	const PLANE & operator [] (size_t Index) const { return Planes[Index]; }
};

/// Frustum zapisany jako 8 wierzchokw
/**
Uzyskiwanie przez konwersj macierzy ViewProj do FRUSTUM_PLANES i z niego
do FRUSTUM_POINTS jest duo szybsze ni budowanie FRUSTUM_POINTS z macierzy
ViewProjInv - 2 (Release) - 8 (Debug) razy szybsze!
*/
struct FRUSTUM_POINTS
{
	/** \name indeksy do tablicy punktw
	Kolejno na przeciciu paszczyzn. */
	//@{
	static const size_t NEAR_LEFT_BOTTOM  = 0;
	static const size_t NEAR_RIGHT_BOTTOM = 1;
	static const size_t NEAR_LEFT_TOP     = 2;
	static const size_t NEAR_RIGHT_TOP    = 3;
	static const size_t FAR_LEFT_BOTTOM   = 4;
	static const size_t FAR_RIGHT_BOTTOM  = 5;
	static const size_t FAR_LEFT_TOP      = 6;
	static const size_t FAR_RIGHT_TOP     = 7;
	//@}

	VEC3 Points[8];

	/// Tworzy niezainicjalizowany
	FRUSTUM_POINTS() { }
	/// Inicjalizuje na podstawie paszczyzn
	FRUSTUM_POINTS(const FRUSTUM_PLANES &FrustumPlanes) { Set(FrustumPlanes); }
	/// Inicjalizuje na podstawie ODWROTNOCI macierzy View*Projection
	FRUSTUM_POINTS(const MATRIX &WorldViewProjInv) { Set(WorldViewProjInv); }
	/// Wypenia
	void Set(const FRUSTUM_PLANES &FrustumPlanes);
	void Set(const MATRIX &WorldViewProjInv);

	VEC3 & operator [] (size_t Index) { return Points[Index]; }
	const VEC3 & operator [] (size_t Index) const { return Points[Index]; }

	/// Wylicza i zwraca rodek frustuma
	void CalcCenter(VEC3 *Out) const;
	/// Oblicza najmniejszy AABB otaczajcy frustum dany tymi punktami
	void CalcBoundingBox(BOX *Box) const;
	/// Oblicza sfer otaczajc frustum
	/** Moe nie najlepsz, ale jaktam... (a moe to jest najlepsza? nie wiem :P) */
	void CalcBoundingSphere(VEC3 *SphereCenter, float *SphereRadius) const;
};

/// Frustum - reprezentacja radarowa.
/**
- Bardzo szybki jest test kolizji z punktem i ze sfer (common::PointInFrustum,
  common::SphereToFrustum).
- UWAGA! Jako wektor Up podawa prawdziwy wektor kamery do gry, tzw. RealUp,
  ktry jest naprawd wzajemnie prostopady do Forward i Right, a nie ten ktry
  zawsze wynosi (0,1,0) - to by bd ktrego si naszukaem.
*/
class FRUSTUM_RADAR
{
private:
	VEC3 m_Eye;
	VEC3 m_Forward;
	VEC3 m_Up;
	VEC3 m_Right;
	float m_RFactor;
	float m_UFactor;
	float m_RSphereFactor;
	float m_USphereFactor;
	float m_ZNear;
	float m_ZFar;

public:
	/// Tworzy niezainicjalizowany
	FRUSTUM_RADAR() { }
	/// Tworzy w peni zainicjalizowany
	FRUSTUM_RADAR(const VEC3 &Eye, const VEC3 &Forward, const VEC3 &Up, const VEC3 &Right, float FovY, float Aspect, float ZNear, float ZFar) { Set(Eye, Forward, Up, Right, FovY, Aspect, ZNear, ZFar); }

	/** \name Pamitane pola
	Zwraca poszczeglne pamitane pola (FovY i Aspect nie pamita bezporednio)
	*/
	//@{
	const VEC3 & GetEye() const     { return m_Eye; }
	const VEC3 & GetForward() const { return m_Forward; }
	const VEC3 & GetUp() const      { return m_Up; }
	const VEC3 & GetRight() const   { return m_Right; }
	float GetZNear() const   { return m_ZNear; }
	float GetZFar() const    { return m_ZFar; }
	//@}

	/** \name Dane pomocnicze
	Zwraca dane pomocnicze
	*/
	//@{
	float GetRFactor() const { return m_RFactor; }
	float GetUFactor() const { return m_UFactor; }
	float GetRSphereFactor() const { return m_RSphereFactor; }
	float GetUSphereFactor() const { return m_USphereFactor; }
	//@}

	/** \name Ustawianie poszczeglnych pl */
	//@{
	void SetEye    (const VEC3 &Eye)     { m_Eye = Eye; }
	void SetForward(const VEC3 &Forward) { m_Forward = Forward; }
	void SetUp     (const VEC3 &Up)      { m_Up = Up; }
	void SetRight  (const VEC3 &Right)   { m_Right = Right; }
	void SetZNear(float ZNear) { m_ZNear = ZNear; }
	void SetZFor (float ZFar)  { m_ZFar = ZFar; }
	void SetFovAndAspect(float FovY, float Aspect);
	//@}
	
	/// Kompletnie wypenia
	void Set(const VEC3 &Eye, const VEC3 &Forward, const VEC3 &Up, const VEC3 &Right, float FovY, float Aspect, float ZNear, float ZFar) { SetProjection(FovY, Aspect, ZNear, ZFar); SetView(Eye, Forward, Up, Right); }
	/// Wypenia jedn powk danych
	void SetProjection(float FovY, float Aspect, float ZNear, float ZFar);
	/// Wypenia drug powk danych
	void SetView(const VEC3 &Eye, const VEC3 &Forward, const VEC3 &Up, const VEC3 &Right);
};

//@}
// math_frustum


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup mathcapsule Kapsua */
//@{

/// Objto kapsuy
float CapsuleVolume(const VEC3 &p1, const VEC3 &p2, float R);
/// Losuje punkt z wntrza kapsuy. Rozkad rwnomierny.
void RandomPointInCapsule(VEC3 *Out, const VEC3 &p1, const VEC3 &p2, float R, RandomGenerator &Rand);
void RandomPointInCapsule(VEC3 *Out, const VEC3 &p1, const VEC3 &p2, float R);

//@}


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
// INNE

/** \addtogroup math_smoothcd Wygadzanie zmiennej
Funkcja wygadzajca zmian zmiennej w czasie - SmoothCD.
Nakadki na t funkcj - szablon specjalizowany dla: float, common::VEC2, common::VEC3.
Klasa szablonowa enkapsulujca pamitanie parametrw wygadzania. */
//@{

template <typename T>
inline void SmoothCD_T(T *InOutPos, const T &Dest, T *InOutVel, float SmoothTime, float TimeDelta)
{
	assert(0 && "SmoothCD_T for invalid template type.");
}

template <>
inline void SmoothCD_T<float>(float *InOutPos, const float &Dest, float *InOutVel, float SmoothTime, float TimeDelta)
{
	SmoothCD(InOutPos, Dest, InOutVel, SmoothTime, TimeDelta);
}

template <>
inline void SmoothCD_T<VEC2>(VEC2 *InOutPos, const VEC2 &Dest, VEC2 *InOutVel, float SmoothTime, float TimeDelta)
{
	SmoothCD(&InOutPos->x, Dest.x, &InOutVel->x, SmoothTime, TimeDelta);
	SmoothCD(&InOutPos->y, Dest.y, &InOutVel->y, SmoothTime, TimeDelta);
}

template <>
inline void SmoothCD_T<VEC3>(VEC3 *InOutPos, const VEC3 &Dest, VEC3 *InOutVel, float SmoothTime, float TimeDelta)
{
	SmoothCD(&InOutPos->x, Dest.x, &InOutVel->x, SmoothTime, TimeDelta);
	SmoothCD(&InOutPos->y, Dest.y, &InOutVel->y, SmoothTime, TimeDelta);
	SmoothCD(&InOutPos->z, Dest.z, &InOutVel->z, SmoothTime, TimeDelta);
}

template <typename T>
class SmoothCD_obj
{
public:
	T Pos;
	T Dest; ///< Zapamitana ostatnia warto docelowa - moe si przyda.
	T Vel;
	float SmoothTime;

	/// Tworzy niezainicjalizowany
	SmoothCD_obj() { }
	/// Tworzy zainicjalizowany
	SmoothCD_obj(const T &Pos, float SmoothTime, const T &Vel) : Pos(Pos), Dest(Pos), Vel(Vel), SmoothTime(SmoothTime) { }
	/// Ustawia parametry
	void Set(const T &Pos, const T &Vel) { this->Pos = Pos; this->Dest = Pos; this->Vel = Vel; }

	/// Wykonuje krok czasowy
	void Update(const T &Dest, float TimeDelta) {
		this->Dest = Dest;
		SmoothCD_T<T>(&Pos, Dest, &Vel, SmoothTime, TimeDelta);
	}
};

//@}
// math_smoothcd


/** \addtogroup math_spherical Wsprzdne sferyczne
- Ukad kartezjaski: DirectX: X w prawo, Y w gr, Z w gb
- Ukad sferyczny: Yaw to dugo geograficzna, Pitch to szeroko geograficzna, R to promie wodzcy
*/
//@{

/// Tu mona podawa co si chce.
void SphericalToCartesian(VEC3 *Out, float Yaw, float Pitch, float R);

/**
\param[out] Yaw
  Wychodzi -PI..PI.
  Mona poda NULL, jeli akurat nas nie interesuje - wtedy liczy szybciej.
\param[out] Pitch
  Wychodzi -PI/2..PI/2.
  Mona poda NULL, jeli akurat nas nie interesuje - wtedy liczy szybciej.
\param[out] R
  Wychodzi > 0.
  Mona poda NULL, jeli akurat nas nie interesuje - wtedy liczy szybciej.
\param Pos
  Nie podawa wektora zerowego bo wyjd buraki.
*/
void CartesianToSpherical(float *OutYaw, float *OutPitch, float *OutR, const VEC3 &Pos);

//@}
// math_spherical

/// Zwraca wypuk otoczk zbioru punktw 2D.
/**
- Na wejciu musi by co najmniej jeden punkt.
- Punkty bd uoone zgodnie z ruchem wskazwek zegara, jeli ukad jest X w prawo a Y w d.
- Punkty powtarzajce si s chyba usuwane, tzn. wynik jest bez powtrze.
- Punkty lece na prostej krawdzi raz pojawiaj si w wyniku a raz nie - niezdefiniowane.
*/
void ConvexHull2D(std::vector<VEC2> *OutPoints, const VEC2 InPoints[], size_t InPointCount);

/**
Macierz rotacji przeksztacajca wektory z przestrzeni Half-Life 2 do
przestrzeni stycznej Tangent Space i jej transpozycja przeksztacajca
z powrotem. To s macierz ortogonalne.
\verbatim
b0 = ( -1/sqrt(6), -1/sqrt(2), 1/sqrt(3) )
b1 = ( -1/sqrt(6),  1/sqrt(2), 1/sqrt(3) )
b2 = ( sqrt(2/3),   0,         1/sqrt(3) )
\endverbatim
*/
extern const MATRIX HL2_TO_TANGENT_MATRIX;
extern const MATRIX TANGENT_TO_HL2_MATRIX;
/// Wektory bazowe HL2 Space wyraone w Tangent Space
extern const VEC3 HL2_BASIS_VECTORS[3];


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_collisions Kolizje */
//@{

/// Funkcja wielofunkcyjna
/**
- Wynik > 0 - punkt c ley na lewo od skierowanego odcinka a-b.
  Trjkt abc jest skierowany przeciwnie do ruchu wskazwek zegara.
- Wynik < 0 - punkt c ley na prawo od skierowanego odcinka a-b.
  Trjkt abc jest skierowany zgodnie z ruchem wskazwek zegara.
- Wynik = 0 - podane trzy punkty le w jednej linii.
- abs(Wynik) - podwojona powierzchnia trjta abc.
*/
float Orient2D(const VEC2 &a, const VEC2 &b, const VEC2 &c);
/// Funkcja wielofunkcyjna
/**
- Wynik < 0 - punkt d ley nad paszczyzn definiowan przez trjkt abc,
  widzian tak e ten trjkt jest zorientowany przeciwnie do ruchu wskazwek
  zegara.
- Wynik > 0 - punkt d ley pod paszczyzn definiowan przez trjkt abc,
  widzian tak e ten trjkt jest zorientowany przeciwnie do ruchu wskazwek
  zegara.
- Wynik = 0 - wszystkie cztery punkty le w jednej paszczynie.
- abs(Wynik) - szeciokrotna objto czworocianu tworzonego przez punkty abcd.
*/
float Orient3D(const VEC3 &a, const VEC3 &b, const VEC3 &c, const VEC3 &d);

/// Zwraca true, jeli punkt p naley do trjkta abc
bool PointInTriangle(const VEC2 &p, const VEC2 &a, const VEC2 &b, const VEC2 &c);
/// Zwraca true, jeli podany czworokt jest wypuky.
/** Nie testowana, moe le dziaa dla zdegenerowanych - TODO. */
bool IsQuadConvex(const VEC2 &a, const VEC2 &b, const VEC2 &c, const VEC2 &d);

/// Zwraca true, jeli podany czworokt jest wypuky (nie zdegenerowany, nie wklsy, nie samoprzecinajcy si).
/** Wszystkie cztery wierzchoki musz lee w jednej paszczynie. */
bool IsQuadConvex(const VEC3 &a, const VEC3 &b, const VEC3 &c, const VEC3 &d);
/// Dany punkt p i promie (RayOrig, RayDir), oblicza parametr t dla punktu na tym promieniu bdcego rzutem tego punktu na ten promie.
/** W ten sposb mona obliczy rzut punktu na promie (punkt z promienia najbliszy
danemu punktowi) jako RayOrig + RayDir*t. Mona te policzy najbliszy punkt
na pprostej lub odcinku limitujc najpierw t od 0 do 1 czy od 0 do dlugoci odcinka.
UWAGA! Jeli RayDir nie jest jednostkowy, wynik trzeba podzieli przez LengthSq(RayDir). */
float ClosestPointOnLine(const VEC3 &p, const VEC3 &RayOrig, const VEC3 &RayDir);
/// Oblicza najblisze sobie punkty na dwch odcinkach
/** Dane s dwa odcinki: Seg1_Beg..Seg1_End i Seg2_Beg..Seg2_End.
Funkcja oblicza najblisze sobie punkty na tych odcinkach.
Parametry do tych odcinkw zwraca przez OutSeg1_t, OutSeg2_t.
Te punkty zwraca przez OutSeg1_pt, OutSeg2_pt.
Caa funkcja zwraca kwadrat odlegoci midzy tymi punktami.
*/
float ClosestPointSegmentSegment(
	const VEC3 &Seg1_Beg, const VEC3 &Seg1_End, const VEC3 &Seg2_Beg, const VEC3 &Seg2_End,
	float &OutSeg1_t, float &OutSeg2_t, VEC3 &OutSeg1_pt, VEC3 &OutSeg2_pt);
/// Zwraca kwadrat odlegoci punktu od prostej
/** \param LineDir   musi by znormalizowane! */
float PointToLineDistanceSq(const VEC2 &P, const VEC2 &LineOrig, const VEC2 &LineDir);
float PointToLineDistanceSq(const VEC3 &P, const VEC3 &LineOrig, const VEC3 &LineDir);
/// Zwraca kwadrat odlegoci punktu od odcinka
float PointToSegmentDistanceSq(const VEC2 &p, const VEC2 &segment_p1, const VEC2 &segment_p2);
/// Zwraca kwadrat odlegoci punktu od odcinka
float PointToSegmentDistanceSq(const VEC3 &p, const VEC3 &segment_p1, const VEC3 &segment_p2);
/// Zwraca true, jeli punkt ley na prostej
/** \param LineDir   musi by znormalizowane! */
bool PointOnLine(const VEC3 &P, const VEC3 &LineOrig, const VEC3 &LineDir);
/// Zwraca parametr T punktu lecego na prostej
/** \param P   Punkt musi naprawd lee na tej prostej! */
float PointToLine(const VEC3 &P, const VEC3 &LineOrig, const VEC3 &LineDir);
/// Zwraca true, jeli punkt ley na odcinku.
/** \param P   Punkt musi lee na prostej przechodzcej przez ten odcinek! */
bool PointInLineSegment(const VEC3 &P, const VEC3 &SegmentBeg, const VEC3 &SegmentEnd);
/// Wyznacza najbliszy punkt lecy na odcinku p1-p2 wzgldem punktu p
/** \param Out [out] Mona poda NULL.
\return Zwraca parametr t dla odcinka, wyraony w dugociach odcinka. */
float ClosestPointOnSegment(VEC2 *Out, const VEC2 &p, const VEC2 &p1, const VEC2 &p2);
float ClosestPointOnSegment(VEC3 *Out, const VEC3 &p, const VEC3 &p1, const VEC3 &p2);
/// Wyznacza najbliszy punkt lecy na pprostej wzgldem punktu p
/** \param Out [out] Mona poda NULL.
\return Zwraca parametr t dla pprostej, wyraony w dugociach odcinka. */
float ClosestPointOnRay(VEC2 *Out, const VEC2 &p, const VEC2 &RayOrig, const VEC2 &RayDir);
float ClosestPointOnRay(VEC3 *Out, const VEC3 &p, const VEC3 &RayOrig, const VEC3 &RayDir);
/// Wyznacza najbliszy punkt wewntrz prostopadocianu wzgldem punktu p.
/** Jeli p ley wewntrz tego prostopadocianu, zwrcone zostaje dokadnie p. */
void ClosestPointInBox(VEC3 *Out, const BOX &Box, const VEC3 &p);
/// Znajduje punkt OutPoint nalecy do podanego OBB, najbliszy do punktu Point.
void ClosestPointInObb(VEC3 *OutPoint, const VEC3 &Point, const OBB &Obb);
/// Znajduje punkt OutPoint nalecy do podanego prostokta 3D, najbliszy do punktu Point.
void ClosestPointInRect3d(VEC3 *OutPoint, const VEC3 &Point, const RECT3D &Rect3d);
/// Znajduje punkt OutPoint nalecy do podanego prostokta 3D wyznaczonego przez trzy wierzchoki: a, b, c, najbliszy do punktu Point.
void ClosestPointInRect3d(VEC3 *OutPoint, const VEC3 &p, const VEC3 &a, const VEC3 &b, const VEC3 &c);
/// Znajduje punkt OutPoint nalecy do trjkta (a, b, c) najbliszy do punktu p
void ClosestPointInTriangle(VEC3 *OutPoint, const VEC3 &p, const VEC3 &a, const VEC3 &b, const VEC3 &c);
/// Znajduje punkt OutPoint nalecy do podanego czworocianu wyznaczonego przez wierzchoki: a, b, c, d, najbliszy do punktu p.
void ClosestPointInTetrahedron(VEC3 *OutPoint, const VEC3 &p, const VEC3 &a, const VEC3 &b, const VEC3 &c, const VEC3 &d);
/// Kwadrat odlegoci punktu od prostopadocianu
/** Jeli p ley wewntrz tego prostopadocianu, zwraca 0. */
float PointToBoxDistanceSq(const VEC3 &p, const BOX &box);
/// Kwadrat odlegoci punktu od OBB.
float PointToObbDistanceSq(const VEC3 &Point, const OBB &Obb);
/// Zwraca true, jeli podany punkt naley do sfery
bool PointInSphere(const VEC3 &p, const VEC3 &SphereCenter, float SphereRadius);
/// Zwraca odlego punktu od powierzchni sfery
/** Jeli punkt ley wewntrz sfery, zwrci odlego ujemn. */
float PointToSphereDistance(const VEC3 &p, const VEC3 &SphereCenter, float SphereRadius);
/// Wyznacza najbliszy punkt na powierzchni sfery wzgldem punktu p.
/** Punkt moe by wewntrz sfery - te jest OK, zwrci z powierzchni sfery.
Punkt nie moe by dokadnie na rodku sfery - wtedy jest dzielenie przez zero. */
void ClosestPointOnSphere(VEC3 *Out, const VEC3 &SphereCenter, float SphereRadius, const VEC3 &p);
/// Zwraca true, jeli podany punkt naley do paszczyzny
/** Paszczyzna nie musi by znormalizowana. */
bool PointOnPlane(const VEC3 &p, const PLANE &plane);
/// Wyznacza najbliszy punkt na paszczynie wzgldem punktu p.
/** Paszczyzna musi by znormalizowana. */
void ClosestPointOnPlane(VEC3 *Out, const PLANE &Plane, const VEC3 &p);
/// Zwraca true, jeli punkt naley do trjkta
bool PointInTriangle(const VEC3 &point, const VEC3 &pa, const VEC3 &pb, const VEC3 &pc);
/// Zwraca true, jeli punkt naley do wntrza podanego frustuma
bool PointInFrustum(const VEC3 &p, const FRUSTUM_PLANES &Frustum);
bool PointInFrustum(const VEC3 &p, const FRUSTUM_RADAR &Frustum);
/// Funkcja wyznacza parametry t1 i t2 dla punktw na dwch prostych, w ktrych te proste s najbliej siebie.
/** Dane s dwie proste (RayOrig1, RayDir1) i (RayOrig2, RayDir2).
Funkcja wyznacza parametry t1 i t2 dla punktw na tych prostych, w ktrych te proste s najbliej siebie.
Jeli nie da si takich znale (proste s rwnolege), funkcja zwraca false.
Te punkty mona policzy ze wzorw p1=RayOrig1+t1*RayDir1 i p2=RayOrig2+t2*RayDir2.
Proste faktycznie przecinaj si jeli odlego midzy tymi punktami wynosi ok. 0. */
bool ClosestPointsOnLines(float *OutT1, float *OutT2, const VEC3 &RayOrig1, const VEC3 &RayDir1, const VEC3 &RayOrig2, const VEC3 &RayDir2);
/// Kolizja promienia z prostopadocianem
/** Jeli promie nie przecina prostopadocianu, zwraca false.
Jeli promie przecina prostopadocian, zwraca true i przez OutT zwraca odlego w wielokrotnociach dugoci RayDir.
Jeli promie przecina prostopadocian od tyu, funkcja te zwraca true i zwraca OutT ujemne.
Jeli RayOrig jest wewntrz prostopadocianu, funkcja zwraca true i OutT = 0. */
bool RayToBox(float *OutT, const VEC3 &RayOrig, const VEC3 &RayDir, const BOX &Box);
/// Wylicza kolizj promienia ze sfer
/** Jeli nie ma kolizji, zwraca false.
Jeli jest kolizja, zwraca true i parametr T.
Jeli kolizja jest z tyu promienia - wczeniej ni jego pocztek - zwraca true i T ujemne.
Jeli pocztek promienia jest wewntrz sfery, zwraca true i T=0 */
bool RayToSphere(const VEC3 &RayOrig, const VEC3 &RayDir, const VEC3 &SphereCenter, float SphereRadius, float *OutT);
/// Przecicie promienia z paszczyzn
/** \param[out] OutT   Zwrcony parametr dla promienia (NIE mona poda 0)
\param[out] OutVD   Zwrcona liczba mwica o zwrocie paszczyzny (NIE mona poda 0)
  OutVD > 0 oznacza, e paszczyzna jest tyem do promienia
\return   false jeli promie by rwnolegy do paszczyzny i nie udao si nic policzy. */
bool RayToPlane(const VEC3 &RayOrig, const VEC3 &RayDir, const PLANE &Plane, float *OutT, float *OutVD);
/// Kolizja promienia z trjktem
/** \param[out] OutT   Mona poda NULL jeli nie interesuje nas odlego kolizji.
\param BackfaceCulling   Jeli true, funkcja zawsze uwzgldnia wycznie kolizj
z trjktem zwrconym w stron CW (jak w DirectX). */
bool RayToTriangle(
	const VEC3 &RayOrig, const VEC3 &RayDir,
	const VEC3 &p0, const VEC3 &p1, const VEC3 &p2,
	bool BackfaceCulling,
	float *OutT);
/// Kolizja promienia z czworoktem w 3D
/** Czworokt musi by paski, wypuky i zorientowany odpowiednio.
Kolizja dziaa tylko w jedn stron.
Dla T ujemnego rwnie zwraca false. Niestety nie zwraca T.
\param OutPoint Parametr opcjonalny. */
bool RayToQuad(const VEC3 &RayOrig, const VEC3 &RayDir, const VEC3 QuadPoints[4], VEC3 *OutPoint = NULL);
/// Kolizja promienia z frustumem
bool RayToFrustum(const VEC3 &RayOrig, const VEC3 &RayDir, const FRUSTUM_PLANES &Frustum, float *t_near, float *t_far);
/// Kolizja promienia z dowolnie zorientowanym cylindrem
/** Jeli kolizja jest z tyu, te zwraca true i OutT wychodzi ujemne.
Zwraca zawsze ten z dwch punktw przecicia ktry ma mniejsze OutT. */
bool RayToCylinder(
	const VEC3 &RayOrig, const VEC3 &RayDir,
	const VEC3 &CylinderBeg, const VEC3 &CylinderEnd, float CylinderRadius,
	float *OutT);
/// Czy punkt ley wewntrz wielociana wypukego.
/** Paszczyzny wielocianu maj mie wektory normalne zwrcone do wewntrz. */
bool PointInConvexPolyhedron(const VEC3 &Point, const PLANE PolyhedronPlanes[], size_t PolyhedronPlaneCount);
bool PointInConvexPolyhedron(const VEC3 &Point, const void *PolyhedronPlanes, size_t PolyhedronPlaneCount, int PolyhedronPlaneStride);
/// Kolizja promienia z wielocianem wypukym.
/** Paszczyzny wielocianu maj mie wektory normalne zwrcone do wewntrz.
Ta funkcja testuje tylko kolizj pprostej.
Jeli wielocian jest z tyu promienia, funkcja zwraca false.
T zawsze wychodz nieujemne. */
bool RayToConvexPolyhedron(
	const VEC3 &RayOrig, const VEC3 &RayDir,
	const PLANE PolyhedronPlanes[], size_t PolyhedronPlaneCount,
	float *OutBegT, float *OutEndT);
bool RayToConvexPolyhedron(
	const VEC3 &RayOrig, const VEC3 &RayDir,
	const void *PolyhedronPlanes, size_t PolyhedronPlaneCount, int PolyhedronPlaneStride,
	float *OutBegT, float *OutEndT);
/// Zwraca true jeli kula koliduje z prostopadocianem (take jeli jest w jego rodku)
bool SphereToBox(const VEC3 &SphereCenter, float SphereRadius, const BOX &Box);
/// Zwraca true, jeli sfera koliduje z prostopadocianem zorientowanym
bool SphereToObb(const VEC3 &SphereCenter, float SphereRadius, const OBB &Obb);
/// Zwraca true, jeli sfera koliduje z trjktem
bool SphereToTriangle(const VEC3 &SphereCenter, float SphereRadius, const VEC3 &TriA, const VEC3 &TriB, const VEC3 &TriC);
/// Zwraca true, jeli sfera zawiera si w caoci w prostopadocianie
bool SphereInBox(const VEC3 &SphereCenter, float SphereRadius, const BOX &Box);
/// Zwraca true, jeli prostopadocian zawiera si w caoci w sferze
bool BoxInSphere(const BOX &Box, const VEC3 &SphereCenter, float SphereRadius);
/// Klasyfikuje pooenie prostopadocianu wzgldem paszczyzny
/**
- -1 = prostopadocian w caoci po ujemnej stronie paszczyzny
-  0 = prostopadocian przecina paszczyzn
- +1 = prostopadocian w caoci po dodatniej stronie paszczyzny
\param plane Paszczyzna musi by znormalizowana */
int ClassifyBoxToPlane(const PLANE &plane, const BOX &box);
/// Klasyfikuje pooenie zorientowanego prostopadocianu OBB wzgldem paszczyzny. Zwraca -1, 0, 1.
int ClassifyObbToPlane(const PLANE &Plane, const OBB &Obb);
/// Kolizja paszczyzny z prostopadocianem
/** \param plane Paszczyzna musi by znormalizowana */
inline bool PlaneToBox(const PLANE &plane, const BOX &box) { return ClassifyBoxToPlane(plane, box) == 0; }
/// Zwraca true, jeli trjkt zawiera si w prostopadocianie
bool TriangleInBox(const VEC3 &p1, const VEC3 &p2, const VEC3 &p3, const BOX &Box);
/// Zwraca true, jeli trjkt przecina prostopadocian lub si w nim zawiera
/** \param[in] OptTrianglePlane   Mona poda jeli si ju ma wyliczone,
eby przyspieszy. Nie musi by znormalizowane. */
bool TriangleToBox(const VEC3 &TriA, const VEC3 &TriB, const VEC3 &TriC, const BOX &Box);
/// Zwraca true, jeli podany prostopadocian jest widoczny cho troch w bryle widzenia
/** Uwaga! W rzadkich przypadkach moe stwierdzi kolizj chocia jej nie ma. */
bool BoxToFrustum_Fast(const BOX &Box, const FRUSTUM_PLANES &Frustum);
/// Zwraca true, jeli podany prostopadocian jest widoczny cho troch w bryle widzenia
/**
Test dokadny, ale wolniejszy i wymaga te reprezentacji punktowej frustuma.
\param[in] OptFrustumPoints to parametr opcjonalny - podaj jeli masz ju wyliczony. */
bool BoxToFrustum(const BOX &Box, const FRUSTUM_PLANES &FrustumPlanes, const FRUSTUM_POINTS *OptFrustumPoints = NULL);
bool BoxToFrustum(const BOX &Box, const FRUSTUM_RADAR &Frustum);
/// Zwraca true, jeli AABB jest w caoci wewntrz frustuma
bool BoxInFrustum(const BOX &Box, const FRUSTUM_PLANES &Frustum);
/// Zwraca true jeli podane dwie sfery koliduj ze sob
bool SphereToSphere(const VEC3 &Center1, float Radius1, const VEC3 &Center2, float Radius2);
/// Zwraca true, jeli sfera koliduje z paszczyzn
/** Paszczyzna musi by znormalizowana. */
bool SphereToPlane(const VEC3 &SphereCenter, float SphereRadius, const PLANE &Plane);
/// Klasyfikuje pooenie sfery wzgldem paszczyzny
/**
- -1 = sfera w caoci po ujemnej stronie paszczyzny
-  0 = sfera przecina paszczyzn
- +1 = sfera w caoci po dodatniej stronie paszczyzny
*/
int ClassifySphereToPlane(const VEC3 &SphereCenter, float SphereRadius, const PLANE &Plane);
/// Kolizja kapsuy (lub sfery poruszajcej si od CapsuleBeg do CapsuleEnd) z paszczyzn
/** \param Plane Paszczyzna musi by znormalizowana. */
bool CapsuleToPlane(const VEC3 &CapsuleBeg, const VEC3 &CapsuleEnd, float CapsuleRadius, const PLANE &Plane);
/// Zwraca true, jeli sfera koliduje z frustumem
/** Uwaga! W rzadkich przypadkach moe stwierdzi kolizj chocia jej nie ma. */
bool SphereToFrustum_Fast(const VEC3 &SphereCenter, float SphereRadius, const FRUSTUM_PLANES &Frustum);
/// Zwraca true, jeli sfera koliduje z frustumem
/** Test dokadny, ale wolniejszy i wymaga te reprezentacji punktowej frustuma.
\param[in] OptFrustumPoints   Parametr opcjonalny - podaj jeli masz ju wyliczony. */
bool SphereToFrustum(
	const VEC3 &SphereCenter, float SphereRadius,
	const FRUSTUM_PLANES &FrustumPlanes, const FRUSTUM_POINTS *OptFrustumPoints = NULL);
/// Testuje przecicie sfery z frustumem.
/** Uwaga! Wyglda na to, e z nienznaych przyczyn ta funkcja nie zwraca true
take dla sfery ktra nie jest cile kolidujca z frustumem, ale z pewnym
niewielkim marginesem bdu w kierunku +/- Right.
*/
bool SphereToFrustum(const VEC3 &SphereCenter, float SphereRadius, const FRUSTUM_RADAR &Frustum);
/// Zwraca true, jeli sfera zawiera si w caoci wewntrz frustuma
bool SphereInFrustum(const VEC3 &SphereCenter, float SphereRadius, const FRUSTUM_PLANES &Frustum);
/// Wylicza kolizj dwch paszczyzn zwracajc parametry prostej tworzcej ich przecicie
/** Jesli nie ma kolizji bo paszczyzny s rwnolege, zwraca false.
Paszczyzny musz by znormalizowane.
\param OutLineDir [out] nie jest znormalizowany. */
bool Intersect2Planes(const PLANE &Plane1, const PLANE &Plane2, VEC3 *OutLineOrig, VEC3 *OutLineDir);
/// Zwraca punkt przecicia trzech paszczyzn lub false, jeli taki punkt nie istnieje
/** Paszczyzny musz by znormalizowane. */
bool Intersect3Planes(const PLANE &P1, const PLANE &P2, const PLANE &P3, VEC3 *OutP);
/// Kolizja paszczyzny z trjktem
bool PlaneToTriangle(const PLANE &Plane, const VEC3 &p0, const VEC3 &p1, const VEC3 &p2);
/// Klasyfikuje pooenie trjkta wzgldem paszczyzny
/**
- -1 = trjkt w caoci po ujemnej stronie paszczyzny
-  0 = trjkt przecina paszczyzn
- +1 = trjkt w caoci po dodatniej stronie paszczyzny
*/
int ClassifyPlaneToTriangle(const PLANE &Plane, const VEC3 &p0, const VEC3 &p1, const VEC3 &p2);
/// Kolizja paszczyzny z frustumem (klasyfikacja)
int ClassifyFrustumToPlane(const FRUSTUM_POINTS &Frustum, const PLANE &Plane);
/// Zwraca true, jeli dwa odcinki 2D zachodz na siebie
/** \param OutSeg1_t Parametr opcjonalny. Zwraca parametr 0..1 miejsca przecicia na odcinku Seg1. */
bool SegmentToSegment(const VEC2 &Seg1_Beg, const VEC2 &Seg1_End, const VEC2 &Seg2_Beg, const VEC2 &Seg2_End, float *OutSeg1_t = NULL);
/// Testuje kolizj dwch trjktw 3D
/** Nie wiem co si dzieje jeli trjkty s zdegenerowane. */
bool TriangleToTriangle(
	const VEC3 &V0,const VEC3 &V1,const VEC3 &V2,
	const VEC3 &U0,const VEC3 &U1,const VEC3 &U2);
/// Testuje, czy trjkt ley w caoci wewntrz frustuma
bool TriangleInFrustum(const VEC3 &t1, const VEC3 &t2, const VEC3 &t3, const FRUSTUM_PLANES &Frustum);
/// Testuje kolizj trjkta z frustumem
/** \param[in] OptTrianglePlane   Podaj jeli masz ju wyliczone, aby przyspieszy.
\param[in] OptFrustumPoints   Podaj jeli masz ju wyliczone, aby przyspieszy. */
bool TriangleToFrustum(
	const VEC3 &t1, const VEC3 &t2, const VEC3 &t3, const PLANE *OptTrianglePlane,
	const FRUSTUM_PLANES &FrustumPlanes, const FRUSTUM_POINTS *OptFrustumPoints);
/// Testuje kolizj midzy dwoma frustumami
bool FrustumToFrustum(
	const FRUSTUM_PLANES &Frustum1_Planes, const FRUSTUM_POINTS &Frustum1_Points,
	const FRUSTUM_PLANES &Frustum2_Planes, const FRUSTUM_POINTS &Frustum2_Points);
/// Kolizja sfery z kapsu
bool SphereToCapsule(
	const VEC3 &SphereCenter, float SphereRadius,
	const VEC3 &Capsule_Beg, const VEC3 &Capsule_End, float Capsule_Radius);
/// Kolizja kapsuy z kapsu
bool CapsuleToCapsule(
	const VEC3 &Capsule1_Beg, const VEC3 &Capsule1_End, float Capsule1_Radius,
	const VEC3 &Capsule2_Beg, const VEC3 &Capsule2_End, float Capsule2_Radius);
/// Sprawdza, czy punkt ley wewntrz wypukego wielokta 2D.
/** Wielokt musi mie wierzchoki zorientowane CW. */
bool PointInConvexPolygon(const VEC2 &Point, const VEC2 PolygonPoints[], size_t PolygonPointCount);
bool PointInConvexPolygon(const VEC2 &Point, const void *PolygonPoints, size_t PolygonPointCount, int PolygonPointStride);
/// Zwraca true, jeli podany punkt ley wewntrz podanego wielokta (2D).
/** Wielokt moe mie dowolny ksztat, take niewypuky, a nawet sam si przecina.
Jego wierzchoki mog biec w dowolnym kierunku. */
bool PointInPolygon(const VEC2 &Point, const VEC2 PolygonPoints[], size_t PolygonPointCount);
bool PointInPolygon(const VEC2 &Point, const void *PolygonPoints, size_t PolygonPointCount, int PolygonPointStride);
/// Zwraca true, jeli promie przecina wielokt wypuky 3D i wtedy przez OutT zwraca parametr dla promienia.
/** Wierzchoki wielokta musz lee w jednej paszczynie.
Jeli kolizja jest od tyu pocztku promienia, zwraca true i OutT ujemne.
Uwaga! Stosuje rzutowanie na jedn z gwnych paszczyzn, testowana tylko dla jednej z trzech moliwych. BackfaceCulling moe dziaa le.
\param BackfaceCulling true oznacza, e promie trafia tylko jeli wchodzi z jednej strony,
z tej z ktrej wierzchoki wielokta s widoczne jako zorientowane w kierunku CW.
\param PolygonPlane Opcjonalny. Podaj, jeli masz ju wyliczony. */
bool RayToConvexPolygon(
	const VEC3 &RayOrig, const VEC3 &RayDir,
	const VEC3 PolygonPoints[], size_t PolygonPointCount,
	bool BackfaceCulling, float *OutT, VEC3 *OutPoint, const PLANE *PolygonPlane = NULL);
bool RayToConvexPolygon(
	const VEC3 &RayOrig, const VEC3 &RayDir,
	const void *PolygonPoints, size_t PolygonPointCount, int PolygonPointStride,
	bool BackfaceCulling, float *OutT, VEC3 *OutPoint, const PLANE *PolygonPlane = NULL);
/// Zwraca true, jeli promie przecina wielokt 3D i wtedy przez OutT zwraca parametr dla promienia.
/** Wierzchoki wielokta musz lee w jednej paszczynie.
Wielokt moe by dowolny, nie musi by wypuky.
Uwaga! Stosuje rzutowanie na jedn z gwnych paszczyzn, testowana tylko dla jednej z trzech moliwych. BackfaceCulling moe dziaa le.
\param BackfaceCulling true oznacza, e promie trafia tylko jeli wchodzi z jednej strony,
z tej z ktrej wierzchoki wielokta s widoczne jako zorientowane w kierunku CW.
\param PolygonPlane Opcjonalny. Podaj, jeli masz ju wyliczony. */
bool RayToPolygon(
	const VEC3 &RayOrig, const VEC3 &RayDir,
	const VEC3 PolygonPoints[], size_t PolygonPointCount,
	bool BackfaceCulling, float *OutT, VEC3 *OutPoint, const PLANE *PolygonPlane = NULL);
bool RayToPolygon(
	const VEC3 &RayOrig, const VEC3 &RayDir,
	const void *PolygonPoints, size_t PolygonPointCount, int PolygonPointStride,
	bool BackfaceCulling, float *OutT, VEC3 *OutPoint, const PLANE *PolygonPlane = NULL);

/// Liczy kolizj poruszajcej si sfery z paszczyzn
/**
- Paszczyzna musi by znormalizowana.
- Jako OutT0 i OutT1 mona podawa NULL, jeli nas akurat nie interesuje.
- Jeli sfera porusza si w drug stron, te wychodzi true i T wychodz ujemne.
- Jeli sfera koliduje z paszczyzn i porusza si w kierunku do niej rwnolegym, jest T0=0, T1=MAXFLOAT i return true.
*/
bool SweptSphereToPlane(const VEC3 &SphereCenter, float SphereRadius, const VEC3 &SphereSweepDir, const PLANE &Plane, float *OutT0, float *OutT1);
/// Oblicza kolizj poruszajcej si sfery z frustumem
/**
- Frustum musi by znormalizowany.
- Jeli sfera ley w rodku, te oczywicie zwraca true.
*/
bool SweptSphereToFrustum(const VEC3 &SphereCenter, float SphereRadius, const VEC3 &SphereSweepDir, const FRUSTUM_PLANES &Frustum);
/// Analityczna kolizja dwch poruszajcych si sfer.
/**
- Jeli nie ma kolizji, zwraca false. T1 i T2 niezdefiniowane.
- Jeli jest kolizja, zwraca true. Wwczas przez T1 i T2 zwraca czas pocztku i koca kolizji wyraony we wielokrotnociach dugoci SphereSweepDir1 (T1 <= T2).
- Jeli kolizja nastpuje z tyu, te zwraca true, a T1 i/lub T2 bd ujemne.

\param SphereSweepDir1    Kierunek ruchu sfery 1 w stosunku do sfery 2. Nie musi by znormalizowany.
*/
bool SweptSphereToSphere(
	const VEC3 &SphereCenter1, float SphereRadius1,
	const VEC3 &SphereCenter2, float SphereRadius2,
	const VEC3 &SphereSweepDir1,
	float *OutT1, float *OutT2);
/// Kolizja poruszajcego si AABB z drugim AABB.
/**
- Jeli nie zachodzi, zwraca false.
- Jeli zachodzi, zwraca true i T jako czas pocztkowy kontaktu.
- Jeli zachodzi z tyu, zwraca true i T ujemne.
- Jeli zachodzi w chwili pocztkowej, zwraca true i T=0.

\param SweepDirBox1    Nie musi by znormalizowany. T bdzie wyraone we wielokrotnociach jego dugoci.
*/
bool SweptBoxToBox(const BOX &Box1, const BOX &Box2, const VEC3 &SweepDirBox1, float *OutT1, float *OutT2);

//@}
// math_collisions

/// Z podanej tablicy punktw typu VEC2 znajduje punkt najbliszy wzgldem p, zwraca jego indeks.
/** \param OutDistance jest opcjonalny, mona poda NULL Jeli odlego od niego nas nie interesuje. */
size_t ClosestPoint(const VEC2 &p, const VEC2 Points[], size_t PointCount, float *OutDistance = NULL);
size_t ClosestPoint(const VEC2 &p, const void *Data, size_t PointCount, int Stride, float *OutDistance = NULL);
/// Z podanej tablicy punktw typu VEC3 znajduje punkt najbliszy wzgldem p, zwraca jego indeks.
/** \param OutDistance jest opcjonalny, mona poda NULL Jeli odlego od niego nas nie interesuje. */
size_t ClosestPoint(const VEC3 &p, const VEC3 Points[], size_t PointCount, float *OutDistance = NULL);
size_t ClosestPoint(const VEC3 &p, const void *Data, size_t PointCount, int Stride, float *OutDistance = NULL);
/// Z podanej tablicy punktw typu VEC2 znajduje punkt najdalszy wzgldem p, zwraca jego indeks.
/** \param OutDistance jest opcjonalny, mona poda NULL Jeli odlego od niego nas nie interesuje. */
size_t FurthestPoint(const VEC2 &p, const VEC2 Points[], size_t PointCount, float *OutDistance = NULL);
size_t FurthestPoint(const VEC2 &p, const void *Data, size_t PointCount, int Stride, float *OutDistance = NULL);
/// Z podanej tablicy punktw typu VEC3 znajduje punkt najdalszy wzgldem p, zwraca jego indeks.
/** \param OutDistance jest opcjonalny, mona poda NULL Jeli odlego od niego nas nie interesuje. */
size_t FurthestPoint(const VEC3 &p, const VEC3 Points[], size_t PointCount, float *OutDistance = NULL);
size_t FurthestPoint(const VEC3 &p, const void *Data, size_t PointCount, int Stride, float *OutDistance = NULL);
/// Z podanej tablicy punktw typu VEC2 znajduje najbardziej wysunity w kierunku okrelonym przez Dir. Zwraca jego indeks.
size_t FurthestPointForDir(const VEC2 &Dir, const VEC2 Points[], size_t PointCount);
size_t FurthestPointForDir(const VEC2 &Dir, const void *Data, size_t PointCount, int Stride);
/// Z podanej tablicy punktw typu VEC3 znajduje najbardziej wysunity w kierunku okrelonym przez Dir. Zwraca jego indeks.
size_t FurthestPointForDir(const VEC3 &Dir, const VEC3 Points[], size_t PointCount);
size_t FurthestPointForDir(const VEC3 &Dir, const void *Data, size_t PointCount, int Stride);

/// Tworzy najmniejszy prostokt otaczajcy podany zbir punktw typu VEC2.
void RectBoundingPoints(RECTF *OutRect, const VEC2 Points[], size_t PointCount);
void RectBoundingPoints(RECTF *OutRect, const void *Data, size_t PointCount, int Stride);
/// Tworzy najmniejszy boks otaczajcy podany zbir punktw typu VEC3.
void BoxBoundingPoints(BOX *box, const VEC3 points[], size_t PointCount);
void BoxBoundingPoints(BOX *box, const void *Data, size_t PointCount, int Stride);

/// Znajduje okrg otaczajcy podany zbir punktw typu VEC2.
/** Nie jest to moe najmniejszy okrg, ale przynajmniej ten algorytm jest prosty i szybki. */
void CircleBoundingPoints(VEC2 *OutSphereCenter, float *OutSphereRadius, const VEC2 Points[], size_t PointCount);
void CircleBoundingPoints(VEC2 *OutSphereCenter, float *OutSphereRadius, const void *PointData, size_t PointCount, int PointStride);
/// Znajduje sfer otaczajc podany zbir punktw typu VEC3.
/** Nie jest to moe najmniejsza sfera, ale przynajmniej ten algorytm jest prosty i szybki. */
void SphereBoundingPoints(VEC3 *OutSphereCenter, float *OutSphereRadius, const VEC3 Points[], size_t PointCount);
void SphereBoundingPoints(VEC3 *OutSphereCenter, float *OutSphereRadius, const void *PointData, size_t PointCount, int PointStride);

/// Dla podanego zbioru punktw typu VEC2 oblicza ich centroid, czyli po prostu redni.
void CalcCentroid(VEC2 *OutCentroid, const VEC2 Points[], size_t PointCount);
void CalcCentroid(VEC2 *OutCentroid, const void *PointData, size_t PointCount, int PointStride);
/// Dla podanego zbioru punktw typu VEC3 oblicza ich centroid, czyli po prostu redni.
void CalcCentroid(VEC3 *OutCentroid, const VEC3 Points[], size_t PointCount);
void CalcCentroid(VEC3 *OutCentroid, const void *PointData, size_t PointCount, int PointStride);

/// Oblicza macierz kowariancji dla podanego zbioru punktw typu VEC3.
void CalcCovarianceMatrix(MATRIX33 *OutCov, const VEC3 Points[], size_t PointCount);
void CalcCovarianceMatrix(MATRIX33 *OutCov, const void *PointData, size_t PointCount, int PointStride);
/// Oblicza wartoci wasne i wektory wasne dla podanej macierzy symetrycznej (np. macierz kowariancji).
/** Uwaga! Duo oblicze, procedura iteracyjna. Moe dziaa wolno.
\param InCovOutEigenvalues [in,out] Jako wejcie - dana macierz. Jako wyjcie - na przektnej ma wartoci wasne.
\param OutEigenvectors [out] Wektory wasne zapisane w kolumnach. */
void CalcEigenvaluesAndEigenvectors(MATRIX33 *InCovOutEigenvalues, MATRIX33 *OutEigenvectors);

/// Oblicza sfer otaczajc box
void BoxBoundingSphere(VEC3 *SphereCenter, float *SphereRadius, const BOX &Box);
/// Oblicza box otaczajcy sfer
void SphereBoundingBox(BOX *Out, const VEC3 &SphereCenter, float SphereRadius);
/// Wylicza AABB otaczajcy podany zbir sfer
/** \param SphereCount   Musi by wiksze od 0. */
void BoxBoundingSpheres(BOX *OutBox, const VEC3 SpheresCenter[], const float SpheresRadius[], size_t SphereCount);

/// Losuje punkt wewntrz kuli o rodku w (0,0,0) i promieniu 1. Rozkad rwnomierny.
void RandomPointInUnitSphere(VEC3 *Out, RandomGenerator &Rand);
void RandomPointInUnitSphere(VEC3 *Out);
/// Losuje punkt wewntrz koa o rodku w (0,0) i promieniu 1. Rozkad rwnomierny.
void RandomPointInUnitCircle(VEC2 *Out, RandomGenerator &Rand);
void RandomPointInUnitCircle(VEC2 *Out);

/// Klasa, ktra dziki pamitaniu wynikw porednich przyspiesza testowanie kolizji jednego trjkta z wieloma promieniami.
/** Na podstawie ksiki: Real-Time Collision Detection, Christer Ericson */
class RayToTriangleCalc
{
public:
	/// Tworzy NIEZAINICJALIZOWANY
	RayToTriangleCalc() { }
	/// Tworzy i inicjalizuje
	RayToTriangleCalc(const VEC3 &TriA, const VEC3 &TriB, const VEC3 &TriC) { Init(TriA, TriB, TriC); }
	/// Inicjalizuje
	void Init(const VEC3 &TriA, const VEC3 &TriB, const VEC3 &TriC);

	/// Liczy kolizj podanego promienia z wpisanym do obiektu trjktem.
	/** Jeli kolizja jest od tyu promienia, rwnie zwraca true i OutT ujemne.
	\param BackfaceCulling Jeli true, uwzgldnia kolizj tylko od strony z ktrej wierzchoki trjkta s CW.
	\param OutT [out] Parametr do promienia.
	\param OutU [out] Wsprzdna barycentryczna.
	\param OutV [out] Wsprzdna barycentryczna.
	\param OutW [out] Wsprzdna barycentryczna.
	\param OutPoint [out] Punkt kolizji. */
	bool RayToTriangle(
		const VEC3 &RayOrig, const VEC3 &RayDir, bool BackfaceCulling,
		float *OutT, float *OutU, float *OutV, float *OutW, VEC3 *OutPoint) const;

private:
	PLANE m_Plane;
	PLANE m_EdgePlaneBC;
	PLANE m_EdgePlaneCA;
};


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
/** \addtogroup math_poisson_disc Poisson Disc */
//@{

extern const float POISSON_DISC_1D[];
const uint4 POISSON_DISC_1D_COUNT = 100;

extern const VEC2 POISSON_DISC_2D[];
const uint4 POISSON_DISC_2D_COUNT = 100;

extern const VEC3 POISSON_DISC_3D[];
const uint4 POISSON_DISC_3D_COUNT = 100;

//@}
// math_poisson_disc

//@}
// code_math

} // namespace common


//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
// Konwersje

/** \addtogroup code_sthtostr */
//@{

template <>
struct SthToStr_obj<common::POINT_>
{
	void operator () (tstring *Str, const common::POINT_ &Sth)
	{
		*Str = common::Format(_T("#,#")) % Sth.x % Sth.y;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::POINT_>
{
	bool operator () (common::POINT_ *Sth, const tstring &Str)
	{
		size_t CommaPos = Str.find(_T(','));
		if (CommaPos == tstring::npos) return false;
		if (!StrToSth<int>(&Sth->x, Str.substr(0, CommaPos))) return false;
		if (!StrToSth<int>(&Sth->y, Str.substr(CommaPos+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::VEC2>
{
	void operator () (tstring *Str, const common::VEC2 &Sth)
	{
		*Str = common::Format(_T("#,#")) % Sth.x % Sth.y;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::VEC2>
{
	bool operator () (common::VEC2 *Sth, const tstring &Str)
	{
		size_t CommaPos = Str.find(_T(','));
		if (CommaPos == tstring::npos) return false;
		if (!StrToSth<float>(&Sth->x, Str.substr(0, CommaPos))) return false;
		if (!StrToSth<float>(&Sth->y, Str.substr(CommaPos+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::VEC3>
{
	void operator () (tstring *Str, const common::VEC3 &Sth)
	{
		*Str = common::Format(_T("#,#,#")) % Sth.x % Sth.y % Sth.z;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::VEC3>
{
	bool operator () (common::VEC3 *Sth, const tstring &Str)
	{
		size_t CommaPos1 = Str.find(_T(','));
		if (CommaPos1 == tstring::npos) return false;
		size_t CommaPos2 = Str.find(_T(','), CommaPos1+1);
		if (CommaPos2 == tstring::npos) return false;
		if (!StrToSth<float>(&Sth->x, Str.substr(0, CommaPos1))) return false;
		if (!StrToSth<float>(&Sth->y, Str.substr(CommaPos1+1, CommaPos2-(CommaPos1+1)))) return false;
		if (!StrToSth<float>(&Sth->z, Str.substr(CommaPos2+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::VEC4>
{
	void operator () (tstring *Str, const common::VEC4 &Sth)
	{
		*Str = common::Format(_T("#,#,#,#")) % Sth.x % Sth.y % Sth.z % Sth.w;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::VEC4>
{
	bool operator () (common::VEC4 *Sth, const tstring &Str)
	{
		size_t CommaPos1 = Str.find(_T(','));
		if (CommaPos1 == tstring::npos) return false;
		size_t CommaPos2 = Str.find(_T(','), CommaPos1+1);
		if (CommaPos2 == tstring::npos) return false;
		size_t CommaPos3 = Str.find(_T(','), CommaPos2+1);
		if (CommaPos3 == tstring::npos) return false;
		if (!StrToSth<float>(&Sth->x, Str.substr(0, CommaPos1))) return false;
		if (!StrToSth<float>(&Sth->y, Str.substr(CommaPos1+1, CommaPos2-(CommaPos1+1)))) return false;
		if (!StrToSth<float>(&Sth->z, Str.substr(CommaPos2+1, CommaPos3-(CommaPos2+1)))) return false;
		if (!StrToSth<float>(&Sth->w, Str.substr(CommaPos3+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::RECTI>
{
	void operator () (tstring *Str, const common::RECTI &Sth)
	{
		*Str = common::Format(_T("#,#,#,#")) % Sth.left % Sth.top % Sth.right % Sth.bottom;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::RECTI>
{
	bool operator () (common::RECTI *Sth, const tstring &Str)
	{
		size_t CommaPos1 = Str.find(_T(','));
		if (CommaPos1 == tstring::npos) return false;
		size_t CommaPos2 = Str.find(_T(','), CommaPos1+1);
		if (CommaPos2 == tstring::npos) return false;
		size_t CommaPos3 = Str.find(_T(','), CommaPos2+1);
		if (CommaPos3 == tstring::npos) return false;
		if (!StrToSth<int>(&Sth->left,   Str.substr(0, CommaPos1))) return false;
		if (!StrToSth<int>(&Sth->top,    Str.substr(CommaPos1+1, CommaPos2-(CommaPos1+1)))) return false;
		if (!StrToSth<int>(&Sth->right,  Str.substr(CommaPos2+1, CommaPos3-(CommaPos2+1)))) return false;
		if (!StrToSth<int>(&Sth->bottom, Str.substr(CommaPos3+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::RECTF>
{
	void operator () (tstring *Str, const common::RECTF &Sth)
	{
		*Str = common::Format(_T("#,#,#,#")) % Sth.left % Sth.top % Sth.right % Sth.bottom;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::RECTF>
{
	bool operator () (common::RECTF *Sth, const tstring &Str)
	{
		size_t CommaPos1 = Str.find(_T(','));
		if (CommaPos1 == tstring::npos) return false;
		size_t CommaPos2 = Str.find(_T(','), CommaPos1+1);
		if (CommaPos2 == tstring::npos) return false;
		size_t CommaPos3 = Str.find(_T(','), CommaPos2+1);
		if (CommaPos3 == tstring::npos) return false;
		if (!StrToSth<float>(&Sth->left,   Str.substr(0, CommaPos1))) return false;
		if (!StrToSth<float>(&Sth->top,    Str.substr(CommaPos1+1, CommaPos2-(CommaPos1+1)))) return false;
		if (!StrToSth<float>(&Sth->right,  Str.substr(CommaPos2+1, CommaPos3-(CommaPos2+1)))) return false;
		if (!StrToSth<float>(&Sth->bottom, Str.substr(CommaPos3+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::BOX>
{
	void operator () (tstring *Str, const common::BOX &Sth)
	{
		*Str = common::Format(_T("#;#")) % Sth.p1 % Sth.p2;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::BOX>
{
	bool operator () (common::BOX *Sth, const tstring &Str)
	{
		size_t SemicolonPos = Str.find(_T(';'));
		if (SemicolonPos == tstring::npos) return false;
		if (!StrToSth<common::VEC3>(&Sth->p1, Str.substr(0, SemicolonPos))) return false;
		if (!StrToSth<common::VEC3>(&Sth->p2, Str.substr(SemicolonPos+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <> struct SthToStr_obj<common::COLOR> {
	void operator () (tstring *Str, const common::COLOR &Sth) { common::ColorToStr(Str, Sth); }
	static inline bool IsSupported() { return true; }
};
template <> struct StrToSth_obj<common::COLOR> {
	bool operator () (common::COLOR *Sth, const tstring &Str) { return common::StrToColor(Sth, Str); }
	static inline bool IsSupported() { return true; }
};

template <> struct SthToStr_obj<common::COLORF> {
	void operator () (tstring *Str, const common::COLORF &Sth) { common::ColorToStr(Str, Sth); }
	static inline bool IsSupported() { return true; }
};
template <> struct StrToSth_obj<common::COLORF> {
	bool operator () (common::COLORF *Sth, const tstring &Str) { return common::StrToColor(Sth, Str); }
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::PLANE>
{
	void operator () (tstring *Str, const common::PLANE &Sth)
	{
		*Str = common::Format(_T("#,#,#,#")) % Sth.a % Sth.b % Sth.c % Sth.d;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::PLANE>
{
	bool operator () (common::PLANE *Sth, const tstring &Str)
	{
		size_t CommaPos1 = Str.find(_T(','));
		if (CommaPos1 == tstring::npos) return false;
		size_t CommaPos2 = Str.find(_T(','), CommaPos1+1);
		if (CommaPos2 == tstring::npos) return false;
		size_t CommaPos3 = Str.find(_T(','), CommaPos2+1);
		if (CommaPos3 == tstring::npos) return false;
		if (!StrToSth<float>(&Sth->a, Str.substr(0, CommaPos1))) return false;
		if (!StrToSth<float>(&Sth->b, Str.substr(CommaPos1+1, CommaPos2-(CommaPos1+1)))) return false;
		if (!StrToSth<float>(&Sth->c, Str.substr(CommaPos2+1, CommaPos3-(CommaPos2+1)))) return false;
		if (!StrToSth<float>(&Sth->d, Str.substr(CommaPos3+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::LINE2D>
{
	void operator () (tstring *Str, const common::LINE2D &Sth)
	{
		*Str = common::Format(_T("#,#,#")) % Sth.a % Sth.b % Sth.c;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::LINE2D>
{
	bool operator () (common::LINE2D *Sth, const tstring &Str)
	{
		size_t CommaPos1 = Str.find(_T(','));
		if (CommaPos1 == tstring::npos) return false;
		size_t CommaPos2 = Str.find(_T(','), CommaPos1+1);
		if (CommaPos2 == tstring::npos) return false;
		if (!StrToSth<float>(&Sth->a, Str.substr(0, CommaPos1))) return false;
		if (!StrToSth<float>(&Sth->b, Str.substr(CommaPos1+1, CommaPos2-(CommaPos1+1)))) return false;
		if (!StrToSth<float>(&Sth->c, Str.substr(CommaPos2+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

template <> struct SthToStr_obj<common::MATRIX> {
	void operator () (tstring *Str, const common::MATRIX &Sth) { common::MatrixToStr(Str, Sth); }
	static inline bool IsSupported() { return true; }
};
template <> struct StrToSth_obj<common::MATRIX> {
	bool operator () (common::MATRIX *Sth, const tstring &Str) { return common::StrToMatrix(Sth, Str); }
	static inline bool IsSupported() { return true; }
};

template <>
struct SthToStr_obj<common::QUATERNION>
{
	void operator () (tstring *Str, const common::QUATERNION &Sth)
	{
		*Str = common::Format(_T("#,#,#,#")) % Sth.x % Sth.y % Sth.z % Sth.w;
	}
	static inline bool IsSupported() { return true; }
};

template <>
struct StrToSth_obj<common::QUATERNION>
{
	bool operator () (common::QUATERNION *Sth, const tstring &Str)
	{
		size_t CommaPos1 = Str.find(_T(','));
		if (CommaPos1 == tstring::npos) return false;
		size_t CommaPos2 = Str.find(_T(','), CommaPos1+1);
		if (CommaPos2 == tstring::npos) return false;
		size_t CommaPos3 = Str.find(_T(','), CommaPos2+1);
		if (CommaPos3 == tstring::npos) return false;
		if (!StrToSth<float>(&Sth->x, Str.substr(0, CommaPos1))) return false;
		if (!StrToSth<float>(&Sth->y, Str.substr(CommaPos1+1, CommaPos2-(CommaPos1+1)))) return false;
		if (!StrToSth<float>(&Sth->z, Str.substr(CommaPos2+1, CommaPos3-(CommaPos2+1)))) return false;
		if (!StrToSth<float>(&Sth->w, Str.substr(CommaPos3+1))) return false;
		return true;
	}
	static inline bool IsSupported() { return true; }
};

//@}
// code_sthtostr
#endif
